Package: box
Title: Write Reusable, Composable and Modular R Code
Version: 1.2.2
Authors@R: c(
        person(
            'Konrad', 'Rudolph',
            email = 'konrad.rudolph@gmail.com',
            role = c('aut', 'cre'),
            comment = c(ORCID = '0000-0002-9866-7051')
        ),
        person(
            'Michael', 'Schubert',
            email = 'mschu.dev@gmail.com',
            role = 'ctb',
            comment = c(ORCID = '0000-0002-6862-5221')
        )
    )
URL: https://klmr.me/box/, https://github.com/klmr/box
BugReports: https://github.com/klmr/box/issues
Description: A modern module system for R. Organise code into hierarchical,
    composable, reusable modules, and use it effortlessly across projects via a
    flexible, declarative dependency loading syntax.
Depends: R (>= 3.6.0)
Imports: tools
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: devtools, knitr (>= 1.40), rmarkdown, R6, rlang, roxygen2 (>=
        7.2.1), shiny, stringr, testthat (>= 3.1.7)
Enhances: rstudioapi
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-08 18:29:47 UTC; konrad
Author: Konrad Rudolph [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9866-7051>),
  Michael Schubert [ctb] (ORCID: <https://orcid.org/0000-0002-6862-5221>)
Maintainer: Konrad Rudolph <konrad.rudolph@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-10 06:50:15 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-10 09:24:51 UTC; unix
Archs: box.so.dSYM

Package: box
Title: Write Reusable, Composable and Modular R Code
Version: 1.2.2
Authors@R: c(
        person(
            'Konrad', 'Rudolph',
            email = 'konrad.rudolph@gmail.com',
            role = c('aut', 'cre'),
            comment = c(ORCID = '0000-0002-9866-7051')
        ),
        person(
            'Michael', 'Schubert',
            email = 'mschu.dev@gmail.com',
            role = 'ctb',
            comment = c(ORCID = '0000-0002-6862-5221')
        )
    )
URL: https://klmr.me/box/, https://github.com/klmr/box
BugReports: https://github.com/klmr/box/issues
Description: A modern module system for R. Organise code into hierarchical,
    composable, reusable modules, and use it effortlessly across projects via a
    flexible, declarative dependency loading syntax.
Depends: R (>= 3.6.0)
Imports: tools
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: devtools, knitr (>= 1.40), rmarkdown, R6, rlang, roxygen2 (>=
        7.2.1), shiny, stringr, testthat (>= 3.1.7)
Enhances: rstudioapi
VignetteBuilder: knitr
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-08 18:29:47 UTC; konrad
Author: Konrad Rudolph [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9866-7051>),
  Michael Schubert [ctb] (ORCID: <https://orcid.org/0000-0002-6862-5221>)
Maintainer: Konrad Rudolph <konrad.rudolph@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-10 06:50:15 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-15 04:54:03 UTC; unix
Archs: box.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("$","box$mod")
S3method("$","box$ns")
S3method(as.character,"box$mod_info")
S3method(as.character,"box$pkg_info")
S3method(as.character,"box$spec")
S3method(path,"box$mod")
S3method(path,"box$ns")
S3method(print,"box$info")
S3method(print,"box$mod")
S3method(print,"box$spec")
S3method(spec_name,"box$mod_spec")
S3method(spec_name,"box$pkg_spec")
export(export)
export(file)
export(help)
export(name)
export(purge_cache)
export(register_S3_method)
export(reload)
export(script_path)
export(set_script_path)
export(topenv)
export(unload)
export(use)
useDynLib(box, .registration = TRUE)

build_shared_lib = function (object_names) {
    # Change working directory so R finds the Makevars.
    old_dir = setwd(box::file())
    on.exit(setwd(old_dir))
    rbin = file.path(R.home('bin'), 'R')
    exitcode = system2(rbin, c('CMD', 'SHLIB', paste0(object_names, '.c')))
    stopifnot(exitcode == 0L)
}

build_shared_lib('hello')

source_file = function (path, language) {
    if (missing(language) || is.null(language)) {
        language = regmatches(path, regexpr('[^.]*$', path))
    }
    code = paste(readLines(path), collapse = '\n')
    sprintf('```%s\n%s\n```', language, code)
}

.on_load = function (ns) {
    knitr::knit_hooks$set(box_file = function (before, options, envir) {
        if (! before) {
            source_file(options$box_file, options$box_lang)
        }
    })
}

box::export()

context('active bindings')

test_that('active bindings can be exported', {
    box::use(mod/active)
    expect_setequal(ls(active), 'binding')
    expect_true(bindingIsActive('binding', active))
    expect_equal(active$binding, 1L)
    expect_message(active$binding, 'get')
})

test_that('active bindings can be attached', {
    box::use(mod/active[...])
    expect_true(bindingIsActive('binding', parent.env(environment())))
    expect_equal(binding, 1L)
    expect_message(binding, 'get')
})

test_that('active binding can be exported from .on_load()', {
    box::use(mod/active2)
    expect_setequal(ls(active2), 'binding')
    expect_true(bindingIsActive('binding', active2))
    expect_equal(active2$binding, 1L)
    expect_message(active2$binding, 'get')
})

test_that('active binding can be attached from .on_load()', {
    box::use(mod/active2[...])
    expect_true(bindingIsActive('binding', parent.env(environment())))
    expect_equal(binding, 1L)
    expect_message(binding, 'get')
})

test_that('active binding is lazily evaluted', {
    box::use(active = mod/active[...])

    f = function (x) {
        message('f')
        x
    }

    expect_messages(f(active$binding), c('^f', '^get'))
    expect_messages(f(binding), c('^f', '^get'))
})

#' Biological sequences
#'
#' The \code{bio/seq} module provides a type for representing DNA sequences.
'.__module__.'

#' Test whether input is valid biological sequence
#' @param seq a character vector or \code{seq} object
#' @name seq
#' @export
is_valid = function (seq) {
    UseMethod('is_valid')
}

is_valid.default = function (seq) {
    nucleotides = unlist(strsplit(seq, ''))
    nuc_index = match(nucleotides, c('A', 'C', 'G', 'T'))
    ! any(is.na(nuc_index))
}

`is_valid.bio/seq` = function (seq) {
    TRUE
}

#' Create a biological sequence
#'
#' \code{seq()} creates a set of nucleotide sequences from one or several
#' character vectors consisting of \code{A}, \code{C}, \code{G} and \code{T}.
#' @param ... optionally named character vectors representing DNA sequences.
#' @return Biological sequence equivalent to the input string.
#' @export
seq = function (...) {
    x = toupper(c(...))
    stopifnot(is_valid(x))
    structure(x, class = 'bio/seq')
}

#' Print one or more biological sequences
`print.bio/seq` = function (x) {
    box::use(stringr[trunc = str_trunc])

    if (is.null(names(x))) names(x) = paste('seq', seq_along(x))

    cat(
        sprintf('%d DNA sequence%s:\n', length(x), if (length(x) == 1L) '' else 's'),
        sprintf('  >%s\n  %s\n', names(x), trunc(x, 30L)),
        sep = ''
    )
    invisible(x)
}

box::register_S3_method('print', 'bio/seq')

#' Reverse complement
#'
#' The reverse complement of a sequence is its reverse, with all nucleotides
#' substituted by their base complement.
#' @param seq character vector of biological sequences
#' @name seq
#' @export
revcomp = function (seq) {
    nucleotides = strsplit(seq, '')
    complement = lapply(nucleotides, chartr, old = 'ACGT', new = 'TGCA')
    revcomp = lapply(complement, rev)
    seq(vapply(revcomp, paste, character(1L), collapse = ''))
}

#' Tabulate nucleotides present in sequences
#' @param seq sequences
#' @return A \code{\link[base::table]{table}} for the nucleotides of each
#'  sequence in the input.
#' @name seq
#' @export
table = function (seq) {
    box::use(stats[set_names = setNames])
    nucleotides = lapply(strsplit(seq, ''), factor, c('A', 'C', 'G', 'T'))
    set_names(lapply(nucleotides, base::table, dnn = NULL), names(seq))
}

if (is.null(box::name())) {
    box::use(./`__tests__`)
}

test_that('valid sequences can be created', {
    expect_error((s = seq('GATTACA')), NA)
    expect_true(is_valid(s))

    expect_error(seq('cattaga'), NA)
})

test_that('invalid sequences cannot be created', {
    expect_error(seq('GATTXA'))
})

libname = function (name) {
    box::file(paste0(name, .Platform$dynlib.ext))
}

.on_load = function (ns) {
    ns$dll = dyn.load(libname('hello'))
}

.on_unload = function (ns) {
    dyn.unload(libname('hello'))
}

#' @export
hello_world = function (name) {
    .Call(dll$hello_world, name)
}

box::use(utils[tail])

#' @export
global_path = tail(box:::mod_search_path(environment()), 1L)

#' @export
path_in_fun = function () {
    tail(box:::mod_search_path(environment()), 1L)
}

#' @export
path_in_nested_fun = function () {
    f = function () {
        tail(box:::mod_search_path(environment()), 1L)
    }
    f()
}

box::use(mod/a)

# Muffle message
silently = function (expr) {
    on.exit({sink(); close(file)})
    file = textConnection('out', 'w', local = TRUE)
    sink(file)
    expr
}

silently(box::use(local_a = ./a))

#' @export
a_which = function () a$which()

#' @export
local_a_which = function () local_a$which()

context('relative imports')

test_that('Imports are absolute by default', {
    old_opts = options(box.path = getwd())
    on.exit(options(old_opts))

    old_env = Sys.getenv('R_BOX_PATH', NA)
    if (!is.na(old_env)) {
        Sys.unsetenv('R_BOX_PATH')
        on.exit(Sys.setenv(R_BOX_PATH = old_env), add = TRUE)
    }

    box::use(mod/nested/relative_a)
    expect_equal(relative_a$a_which(), '/a')
})

test_that('Relative import are always local', {
    old_opts = options(box.path = getwd())
    on.exit(options(old_opts))

    old_env = Sys.getenv('R_BOX_PATH', NA)
    if (!is.na(old_env)) {
        Sys.unsetenv('R_BOX_PATH')
        on.exit(Sys.setenv(R_BOX_PATH = old_env), add = TRUE)
    }

    box::use(mod/nested/relative_a)
    expect_equal(relative_a$local_a_which(), 'nested/a')
})

`%==%` = function (x, y) {
    identical(x, y)
}

`%!=%` = function (x, y) {
    ! identical(x, y)
}

check_dots_unnamed = function (definition = sys.function(sys.parent()), call = sys.call(sys.parent())) {
    dots = match.call(definition = definition, call = call, expand.dots = FALSE)$...
    if (is.null(names(dots))) return()

    throw('unexpected named argument', call)
}

# Set up R so that R called from a unit test finds everything in order.

local({
    # First, find the original user profile, if any. The environment variable is
    # set by the test code calling R to the value of `R_PROFILE_USER`.

    user_profile = Sys.getenv('R_ORIGINAL_PROFILE_USER')
    if (user_profile == '') {
        user_profile = if (file.exists('.Rprofile')) '.Rprofile' else '~/.Rprofile'
    }

    # Next, ensure that the ‘box’ package that’s loaded is the source version
    # we’re currently testing, rather than something loaded by the user
    # configuration.

    unloadNamespace('box')
    devtools::load_all(Sys.getenv('BOX_TESTING_BASEDIR'), quiet = TRUE, export_all = FALSE)
    detach('package:box')

    # … and load the original user profile, if it exists.

    if (file.exists(user_profile)) {
        sys.source(user_profile, envir = .GlobalEnv)
    }

    old_first = get0('.First', envir = .GlobalEnv, mode = 'function', ifnotfound = function () {})
    .GlobalEnv$.First = function () {
        old_first()
        # This is required by `interactive_r` to verify that invocation succeeded.
        options(prompt = '> ', continue = '+ ')
    }
})

#' Explicitly declare module exports
#'
#' \code{box::export} explicitly marks a source file as a \pkg{box} module. If
#' can be used as an alternative to the \code{@export} tag comment to declare a
#' module’s exports.
#'
#' @usage \special{box::export(\dots)}
#' @param \dots zero or more unquoted names that should be exported from the
#' module.
#' @return \code{box::export} has no return value. It is called for its
#' side effect.
#'
#' @details
#' \code{box::export} can be called inside a module to specify the module’s
#' exports. If a module contains a call to \code{box::export}, this call
#' overrides any declarations made via the \code{@export} tag comment. When a
#' module contains multiple calls to \code{box::export}, the union of all thus
#' defined names is exported.
#'
#' A module can also contain an argument-less call to \code{box::export}. This
#' ensures that the module does not export any names. Otherwise, a module that
#' defines names but does not mark them as exported would be treated as a
#' \emph{legacy module}, and all default-visible names would be exported from
#' it. Default-visible names are names not starting with a dot (\code{.}).
#' Another use of \code{box::export()} is to enable a module without exports to
#' use \link[=mod-hooks]{module event hooks}.
#'
#' @note The preferred way of declaring exports is via the \code{@export} tag
#' comment. The main purpose of \code{box::export} is to explicitly prevent
#' exports, by being called without arguments.
#'
#' @seealso
#' \code{\link[=use]{box::use}} for information on declaring exports via
#' \code{@export}.
#' @export
export = function (...) {
    mod_ns = mod_topenv(parent.frame())
    if (! is_namespace(mod_ns)) {
        throw('{"export";"} can only be called from inside a module')
    }

    check_dots_unnamed()

    names = as.list(match.call())[-1L]
    for (name in names) {
        if (! is.name(name)) {
            throw(
                '{"export()";"} requires a list of unquoted names; ',
                '{name;"} is not a name'
            )
        }
    }

    namespace_info(mod_ns, 'exports') = union(
        namespace_info(mod_ns, 'exports'),
        as.character(unlist(names))
    )
    namespace_info(mod_ns, 'legacy') = FALSE
    invisible()
}

#' Find exported names in parsed module source
#'
#' @param info The module info.
#' @param exprs The list of expressions of the parsed module.
#' @param mod_ns The module namespace after evaluating the expressions.
#' @return \code{parse_export_specs} returns a potentially empty character
#' vector of exported names.
#'
#' @note
#' There are two situations in which the \code{@export} tag can be applied:
#' \enumerate{
#'   \item When applied to assignments, the assigned name is exported.
#'   \item When applied to a \code{box::use} call, the imported names are
#'     exported. This can be the module name itself, any attached names, or
#'     both. All names introduced by the \code{box::use} call are exported. See
#'     \code{\link[box]{use}} for the rules governing what names are introduced
#'     into the scope, and thus exported.
#' }
#' In any other situation, applying the \code{@export} tag is an error.
#' @keywords internal
parse_export_specs = function (info, exprs, mod_ns) {
    parse_export = function (export) {
        if (block_is_use_call(export)) {
            imports = attr(export, 'call')[-1L]
            aliases = names(imports) %||% character(length(imports))
            flatmap_chr(reexport_names, imports, aliases, list(export))
        } else {
            # Attempt to extract a name from an arbitrary expressions.
            block_name(export) %||% block_error(export)
        }
    }

    reexport_names = function (declaration, alias, export) {
        # Permit empty expression resulting from trailing comma.
        if (declaration %==% quote(expr =) && alias %==% '') return()
        spec = parse_spec(declaration, alias)
        import = find_matching_import(namespace_info(mod_ns, 'imports'), spec)

        if (is_mod_still_loading(import$info)) {
            if (! is.null(spec$attach)) {
                throw(
                    'invalid attempt to export names from an incompletely ',
                    'loaded, cyclic import (module {name;"}) in line {line}',
                    name = spec$name,
                    line = attr(export, 'location')[1L]
                )
            }

            return(spec$alias)
        }

        export_names = mod_export_names(import$info, import$ns)
        real_alias = if (is.null(spec$attach) || spec$explicit) spec$alias
        c(names(attach_list(spec, export_names)), real_alias)
    }

    find_matching_import = function (imports, reexport) {
        for (import in imports) {
            if (import$spec %==% reexport) return(import)
        }
    }

    exports = parse_export_tags(info, exprs, mod_ns)
    unique(flatmap_chr(parse_export, exports))
}

block_error = function (export) {
    throw(
        'the {"@export";"} tag may only be applied to assignments or ',
        '{"use";"} declarations;\n',
        'used incorrectly in line {location}:\n',
        '    {paste(code, collapse = "\n    ")}',
        code = deparse(attr(export, 'call'), backtick = TRUE),
        location = attr(export, 'location')[1L]
    )
}

#' @keywords internal
#' @rdname parse_export_specs
use_call = quote(box::use)

#' @keywords internal
#' @rdname parse_export_specs
static_assign_calls = c(quote(`<-`), quote(`=`), quote(`:=`), quote(`<<-`))

#' @keywords internal
#' @rdname parse_export_specs
assign_calls = c(static_assign_calls, quote(assign), quote(makeActiveBinding))

#' @param call A call to test.
#' @keywords internal
#' @rdname parse_export_specs
is_static_assign_call = function (call) {
    any(map_lgl(identical, static_assign_calls, call))
}

#' @keywords internal
#' @rdname parse_export_specs
is_assign_call = function (call) {
    any(map_lgl(identical, assign_calls, call))
}

#' @param block A roxygen2 block to inspect.
#' @keywords internal
#' @rdname parse_export_specs
block_is_assign = function (block) {
    is_assign_call(attr(block, 'call')[[1L]])
}

#' @keywords internal
#' @rdname parse_export_specs
block_is_use_call = function (block) {
    attr(block, 'call')[[1L]] %==% use_call
}

#' @keywords internal
#' @rdname parse_export_specs
block_is_exported = function (block) {
    'export' %in% names(block)
}

#' @keywords internal
#' @rdname parse_export_specs
block_name = function (block) {
    attr(block, 'object')$alias
}

#' Extract comment tags from Roxygen block comments
#'
#' @param exprs The unevaluated expressions to parse.
#' @return \code{parse_export_tags} returns a list of \code{roxy_block}s for all
#' exported declarations.
#' @note The following code performs the same function as roxygen2 with a custom
#' \code{@} tag roclet. Unfortunately roxygen2 itself pulls in many
#' dependencies, making it less suitable for an infrastructure package such as
#' this one. Furthermore, the code license of roxygen2 is incompatible with
#' ours, so we cannot simply copy and paste the relevant code out. Luckily the
#' logic is straightforward to reimplement.
#' @keywords internal
parse_export_tags = function (info, exprs, mod_ns) {
    refs = utils::getSrcref(exprs)
    comment_refs = add_comments(refs)
    is_exported = map_lgl(has_export_tag, comment_refs)
    map(create_export_block, exprs[is_exported], refs[is_exported], list(info), list(mod_ns))
}

#' Collect export tag information
#'
#' @param expr The unevaluated expression represented by the tag.
#' @param ref The code reference \code{srcref} represented by the tag.
#' @return \code{create_export_block} returns an object of type
#' \code{roxy_block} represents an exported declaration expression, along with
#' its source code location.
#' @note This could be represented much simpler but we keep compatibility with
#' roxygen2 — at least for the time being — to make integration with the
#' roxygen2 API easier, should it become necessary.
#' @keywords internal
create_export_block = function (expr, ref, info, mod_ns) {
    structure(
        list(export = ''),
        filename = attr(ref, 'srcfile')$filename,
        location = as.vector(ref),
        call = expr,
        object = parse_object(info, expr, mod_ns),
        class = 'roxy_block'
    )
}

#' @keywords internal
#' @rdname create_export_block
parse_object = function (info, expr, mod_ns) {
    if (is.character(expr)) {
        if (expr %==% '_PACKAGE') {
            value = list(path = info$source_path)
            roxygen2_object(expr, value, 'package')
        } else {
            roxygen2_object(expr, expr, 'data')
        }
    } else if (is.call(expr)) {
        if (expr[[1L]] %==% use_call) {
            roxygen2_object('use', list(pkg = 'box', fun = 'use'), 'import')
        } else {
            # Attempt to extract a name from the LHS: recurse until the
            # left-most token is a name or character literal (this is necessary
            # because `"a" = 1` is valid R; but also to allow `assign` and
            # `makeActiveBinding` calls).
            while (is.call(expr) && length(expr) > 1L) {
                if (is.name(expr[[2L]]) || is.character(expr[[2L]])) {
                    name = if (is_static_assign_call(expr[[1L]])) {
                        as.character(expr[[2L]])
                    } else {
                        # `assign(…)`, `makeActiveBinding(…)` and similar.
                        eval(expr[[2L]], mod_ns)
                    }

                    obj = if (bindingIsActive(name, mod_ns)) {
                        roxygen2_object(name, activeBindingFunction(name, mod_ns), 'active')
                    } else {
                        value = get(name, mod_ns)
                        if (is.function(value)) {
                            # FIXME: Add handling for S3
                            roxygen2_object(name, value, 'function')
                        } else {
                            roxygen2_object(name, value, NULL)
                        }
                    }
                    return(obj)
                }
                expr = expr[[2L]]
            }
            # We do not care about the rest.
            NULL
        }
    } else {
        NULL
    }
}

#' @param alias The object name.
#' @param value The object value.
#' @param type The object type.
#' @keywords internal
#' @rdname create_export_block
roxygen2_object = function (alias, value, type) {
    structure(
        list(alias = alias, value = value, methods = NULL, topic = alias),
        class = paste0('roxygen2_', c(type, 'object'))
    )
}

#' Extend code regions to include leading comments and whitespace
#'
#' @param refs a list of the code region \code{srcref}s to extend.
#' @return \code{add_comment} returns a list of \code{srcref}s corresponding to
#' \code{srcref}, but extended to include the preceding comment block.
#' @keywords internal
add_comments = function (refs) {
    if (length(refs) == 0L) return(list())

    block_end_lines = map_int(`[[`, refs, 3L)
    block_end_bytes = map_int(`[[`, refs, 4L)
    block_start_lines = c(1L, block_end_lines[-length(block_end_lines)] + 1L)
    srcfiles = map(attr, refs, 'srcfile')
    llocs = map(c, block_start_lines, 1L, block_end_lines, block_end_bytes)
    map(srcref, srcfiles, llocs)
}

#' Find \code{@export} tags in code regions
#'
#' @param ref The code region \code{srcref} to search.
#' @return \code{TRUE} if the given region is annotated with a \code{@export}
#' tag, \code{FALSE} otherwise.
#' @keywords internal
has_export_tag = function (ref) {
    self = environment()

    next_char = function () {
        self$pos = pos + 1L
        substr(line, pos, pos)
    }

    consume_char = function (chars) {
        matched = next_char() %in% chars
        if (! matched) self$pos = pos - 1L
        matched
    }

    consume_chars = function (chars) {
        while (pos != nchar(line) && consume_char(chars)) {}
    }

    consume_whitespace = function () {
        consume_chars(c(' ', '\t'))
    }

    is_roxygen_comment = function () {
        prev = pos
        consume_whitespace()
        if (! consume_char('#')) return(FALSE)
        consume_chars('#')
        if (! consume_char("'")) {
            self$pos = prev
            return(FALSE)
        }
        consume_whitespace()
        TRUE
    }

    is_empty_or_comment = function () {
        consume_whitespace()
        pos == nchar(line) || consume_char('#')
    }

    is_export = function () {
        self$pos = pos + 1L
        len = nchar('@export')
        substr(line, pos, pos + len) == '@export' &&
            (pos + len > nchar(line) || substr(line, pos + len, pos + len) %in% c(' ', '\t'))
    }

    block = as.character(ref)
    for (line in block) {
        pos = 0L
        if (! is_roxygen_comment()) {
            if (is_empty_or_comment()) next else return(FALSE)
        }
        if (is_export()) return(TRUE)
    }
    FALSE
}

context('format')

test_that('literal strings are unchanged', {
    expect_equal(fmt('this is a test'), 'this is a test')
    expect_equal(fmt('this', 'is', 'a', 'test'), 'thisisatest')
    expect_equal(fmt(c('this', 'is'), 'a', 'test'), 'thisisatest')
})

test_that('escaping works', {
    expect_equal(fmt('a {{test}}'), 'a {test}')
    expect_equal(fmt('a {{{{test}}'), 'a {{test}')
})

test_that('interpolation works', {
    expect_equal(fmt('a {"test"}'), 'a test')
    expect_equal(fmt('a {{{"test"}}}'), 'a {test}')
    x = 'test'
    expect_equal(fmt('a {x}'), 'a test')
    expect_equal(fmt('a {x}', x = 42L), 'a 42')
})

# test_that('escaping works inside interpolation', {
#     expect_equal(fmt('{"{{test"}'), '{test')
#     expect_equal(fmt('{"test}}"}'), 'test}')
# })

test_that('format modifiers work', {
    expect_equal(fmt('a {"test";"}'), sprintf('a %s', dQuote('test')))
    expect_equal(fmt("a {'test';'}"), sprintf('a %s', sQuote('test')))
    expect_equal(fmt('pi = {pi;.2f}'), sprintf('pi = %.2f', pi))
    x = 1 : 3
    expect_equal(fmt('x = {x}'), 'x = 1, 2, 3')
    expect_equal(fmt('{x;"}'), toString(dQuote(x)))
})

test_that('invalid formats raise an error', {
    expect_box_error(
        fmt('{12;1?}'),
        sprintf('unrecognized format modifier %s', dQuote('1\\?'))
    )
})

test_that('deparsed expression forms a single string', {
    expr = quote(long_call(with + some, 'arguments in it', that_total, over, 60L - ...chars))
    actual = fmt('{expr;"}')

    expect_equal(actual, dQuote(deparse1(expr)))
})

test_that('real use-cases from this package implementation work', {
    old_opts = options(useFancyQuotes = FALSE)
    on.exit(options(old_opts))

    # errors.r

    call = call('foo', 'bar', x = 1L)
    error = simpleError('test', call)
    expect_equal(
        fmt(
            '{msg}\n(inside {calls})',
            msg = conditionMessage(error),
            calls = paste(dQuote(deparse(conditionCall(error))), collapse = '\n')
        ),
        'test\n(inside "foo("bar", x = 1L)")'
    )

    fmt = '?'
    expect_equal(
        fmt('unrecognized format modifier {fmt;"}'),
        'unrecognized format modifier "?"'
    )

    # export.r

    expect_equal(
        fmt('{"export";"} can only be called from inside a module'),
        '"export" can only be called from inside a module'
    )

    name = quote(test)
    expect_equal(
        fmt(
            '{"export()";"} requires a list of unquoted names; ',
            '{name;"} is not a name'
        ),
        '"export()" requires a list of unquoted names; "test" is not a name',
    )

    expect_equal(
        fmt(
            'Invalid attempt to export names from an incompletely ',
            'loaded, cyclic import (module {name;"}) in line {line}.',
            name = 'foo/test',
            line = 42L
        ),
        'Invalid attempt to export names from an incompletely loaded, cyclic import (module "foo/test") in line 42.'
    )

    expect_equal(
        fmt(
            'The {"@export";"} tag may only be applied to assignments or ',
            '{"use";"} declarations.\n',
            'Used incorrectly in line {location}:\n',
            '    {paste(code, collapse = \'\n    \')}',
            code = c('foo', 'bar'),
            location = 42L
        ),
        'The "@export" tag may only be applied to assignments or "use" declarations.\nUsed incorrectly in line 42:\n    foo\n    bar'
    )

    # help.r

    topic = quote(foo$bar)
    expect_equal(
        fmt('{topic;"} is not a valid module help topic'),
        '"foo$bar" is not a valid module help topic'
    )

    expect_equal(
        fmt('Displaying documentation requires {"roxygen2";\'} installed'),
        'Displaying documentation requires \'roxygen2\' installed'
    )

    mod_name = 'foo'
    expect_equal(
        fmt('No documentation available for {mod_name;"}'),
        'No documentation available for "foo"'
    )

    subject = 'bar'
    expect_equal(
        fmt('No documentation available for {subject;"} in module {mod_name;"}'),
        'No documentation available for "bar" in module "foo"'
    )

    # info.r

    mod_name = 'foo/bar'
    base_paths = c('foo', 'bar', 'baz')
    expect_equal(
        fmt(
            'Unable to load module {mod_name;"}; ',
            'not found in {base_paths;"}'
        ),
        'Unable to load module "foo/bar"; not found in "foo", "bar", "baz"',
    )

    # use.r

    missing = 'foo'
    expect_equal(
        fmt(
            'Name{s} {missing;"} not exported by {mod_name;"}',
            s = if (length(missing) > 1L) 's' else ''
        ),
        'Name "foo" not exported by "foo/bar"',
    )

    missing = c('foo', 'bar')
    expect_equal(
        fmt(
            'Name{s} {missing;"} not exported by {mod_name;"}',
            s = if (length(missing) > 1L) 's' else ''
        ),
        'Names "foo", "bar" not exported by "foo/bar"'
    )
})

#' Find the full paths of files in modules
#'
#' @usage \special{box::file(\dots)}
#' @usage \special{box::file(\dots, module)}
#' @param \dots character vectors of files or subdirectories inside a module; if
#'  none is given, return the root directory of the module
#' @param module a module environment
#' @return A character vector containing the absolute paths to the files
#'  specified in \code{\dots}.
#'
#' @note If called from outside a module, the current working directory is used.
#'
#' This function is similar to \code{system.file} for packages. Its semantics
#' differ in the presence of non-existent files: \code{box::file} always returns
#' the requested paths, even for non-existent files; whereas \code{system.file}
#' returns empty strings for non-existent files, or fails (if requested via the
#' argument \code{mustWork = TRUE}).
#' @seealso \code{\link[base]{system.file}}
#' @export
file = function (..., module) {
    check_dots_unnamed()
    if (missing(module)) module = current_mod()
    file.path(base_path(module), ...)
}

context('RStudio')

with_mock_rstudio = function (expr) {
    Sys.setenv(RSTUDIO = '1')

    old_gui = .Platform$GUI
    unlockBinding('.Platform', .BaseNamespaceEnv)
    .BaseNamespaceEnv$.Platform$GUI = 'RStudio'
    Sys.unsetenv('TESTTHAT')

    on.exit({
        Sys.unsetenv('RSTUDIO')
        .BaseNamespaceEnv$.Platform$GUI = old_gui
        lockBinding('.Platform', .BaseNamespaceEnv)
        Sys.setenv(TESTTHAT = 'true')
    })

    expr
}

with_mock_rstudio_tools_path = function (path, expr) {
    rstudio_tools_env = new.env()
    local(envir = rstudio_tools_env, {
        .rs.api.getActiveDocumentContext = function () { # nolint
            list(
                path = path,
                contents = '',
                selection = list(list(range = rep(1L, 4L), text = ''))
            )
        }
        .rs.api.versionInfo = function () list()
    })
    on.exit(detach('tools:rstudio'))
    attach(rstudio_tools_env, name = 'tools:rstudio')

    expr
}

test_that('Path of active file in RStudio is found without ‘rstudioapi’', {
    skip_on_ci()
    skip_on_cran()
    # Assume that we cannot edit package library path on other systems.
    skip_outside_source_repos()

    pkg_path = dirname(dirname(attr(suppressWarnings(packageDescription('rstudioapi')), 'file')))
    tmp_path = paste0(pkg_path, '-x')

    unloadNamespace('rstudioapi')
    expect_true(file.rename(pkg_path, tmp_path))
    on.exit(file.rename(tmp_path, pkg_path))

    file_path = with_mock_rstudio({
        with_mock_rstudio_tools_path('/rstudio/test.r', box::file())
    })

    expect_paths_equal(file_path, '/rstudio')
    expect_false(isNamespaceLoaded('rstudioapi'))
})

test_that('Path of active file in RStudio is found with ‘rstudioapi’', {
    skip_on_cran()
    skip_if_not_installed('rstudioapi')

    file_path = with_mock_rstudio({
        with_mock_rstudio_tools_path('/rstudio/test.r', box::file())
    })

    expect_paths_equal(file_path, '/rstudio')
    expect_true(isNamespaceLoaded('rstudioapi'))
})

#' Apply function to elements in list
#'
#' \code{map} applies a function to lists of arguments, similar to \code{Map} in
#' base R, with the argument \code{USE.NAMES} set to \code{FALSE}.
#' \code{flatmap} performs a recursive map: the return type is always a vector
#' of some type given by the \code{.default}, and if the return value of calling
#' \code{.f} is a vector, it is flattened into the enclosing vector (see
#' \sQuote{Examples}).
#' \code{transpose} is a special \code{map} application that concatenates its
#' inputs to compute a transposed list.
#' @param .f an n-ary function where n is the number of further arguments given
#' @param \dots lists of arguments to map over in parallel
#' @param .default the default value returned by \code{flatmap} for an empty
#' input
#' @return \code{map} returns a (potentially nested) list of values resulting
#' from applying \code{.f} to the arguments.
#' @section Examples:
#' \preformatted{
#' flatmap_chr(identity, NULL)
#' # character(0)
#'
#' flatmap_chr(identity, c('a', 'b'))
#' # [1] "a" "b"
#'
#' flatmap_chr(identity, list(c('a', 'b'), 'c'))
#' # [1] "a" "b" "c"
#'
#' transpose(1 : 2, 3 : 4)
#' # [[1]]
#' # [1] 1 3
#' #
#' # [[2]]
#' # [1] 2 4
#' }
#' @keywords internal
map = function (.f, ...) {
    if (length(..1) == 0L) list() else Map(.f, ..., USE.NAMES = FALSE)
}

#' @return \code{flatmap} returns a vector with type given by \code{.default},
#' or \code{.default}, if the input is empty.
#' @rdname map
flatmap = function (.f, ..., .default) {
    Reduce(c, map(.f, ...), .default)
}

#' @rdname map
flatmap_chr = function (.f, ...) {
    flatmap(.f, ..., .default = character(0L))
}

#' @rdname map
vmap = function (.f, .x, ..., .default) {
    # FIXME: Find a more efficient implementation.
    fun_value = eval(call(class(.default), 1L))
    vapply(X = .x, FUN = .f, FUN.VALUE = fun_value, ..., USE.NAMES = FALSE)
}

#' @rdname map
map_int = function (.f, ...) {
    vmap(.f = .f, ..., .default = integer(0L))
}

#' @rdname map
map_lgl = function (.f, ...) {
    vmap(.f = .f, ..., .default = logical(0L))
}

#' @rdname map
map_chr = function (.f, ...) {
    vmap(.f = .f, ..., .default = character(0L))
}

#' @return \code{transpose} returns a list of the element-wise concatenated
#' input vectors; that is, a \dQuote{transposed list} of those elements.
#' @rdname map
transpose = function (...) {
    map(c, ...)
}

context('help')

help_test_cases = c(quote(a), quote(a$b), quote(a$b$c), quote(a$b$c$d))
help_test_env = new.env()

local({
    box::use(mod/help/a)
}, envir = help_test_env)

test_that('help topic targets are correctly resolved', {
    # Necessary to allow ‘knitr’ tests to unload ‘knitr’, which in turn requires
    # ‘roxygen2’.
    on.exit(unloadNamespace('roxygen2'))
    a = help_test_env$a
    expected_mods = c(a, a, a$b, a$b$c)
    expected_names = c('.__module__.', 'b', 'c', 'd')
    actual_targets = box:::map(box:::help_topic_target, help_test_cases, list(help_test_env))
    box:::map(expect_identical, box:::map(`[[`, actual_targets, list(1L)), expected_mods)
    box:::map(expect_identical, box:::map(`[[`, actual_targets, list(2L)), expected_names)
})

test_that('documentation can be parsed', {
    # Necessary to allow ‘knitr’ tests to unload ‘knitr’, which in turn requires
    # ‘roxygen2’.
    on.exit(unloadNamespace('roxygen2'))
    a = help_test_env$a
    doc = parse_documentation(attr(a, 'info'), attr(a, 'namespace'))
    expect_length(doc, 1L)
    expect_named(doc, '.__module__.')

    c = a$b$c
    doc = parse_documentation(attr(c, 'info'), attr(c, 'namespace'))
    expect_length(doc, 2L)
    expect_named(doc, c('.__module__.', 'd'))
})

# Set up local module environment to test against.
# Note that we override the normal path here.

options(box.path = getwd())

expect_not_equal = function (object, expected, info = NULL, label = NULL, expected.label = NULL) {
    act = testthat::quasi_label(rlang::enquo(object), label, arg = 'object')
    exp = testthat::quasi_label(rlang::enquo(expected), expected.label, arg = 'expected')
    cmp = testthat::compare(act$val, exp$val)
    val = deparse(act$val)

    testthat::expect(
        ! cmp$equal,
        sprintf('%s is equal to %s.\n%s == %s', act$lab, exp$lab, val, val),
        info = info
    )
    invisible(act$value)
}

expect_not_identical = function (object, expected, info = NULL, label = NULL, expected.label = NULL) {
    act = testthat::quasi_label(rlang::enquo(object), label, arg = 'object')
    exp = testthat::quasi_label(rlang::enquo(expected), expected.label, arg = 'expected')
    ident = identical(act$val, exp$val)

    testthat::expect(
        ! ident,
        sprintf('%s identical to %s', act$lab, exp$lab),
        info = info,
    )
    invisible(act$val)
}

expect_in = function (object, list) {
    act = testthat::quasi_label(rlang::enquo(list), arg = 'list')
    testthat::expect(
        object %in% act$val,
        sprintf('%s is not in %s.', deparse(object), act$lab)
    )
    invisible(act$val)
}

expect_not_in = function (object, list) {
    act = testthat::quasi_label(rlang::enquo(list), arg = 'list')
    testthat::expect(
        ! object %in% act$val,
        sprintf('%s is in %s.', deparse(object), act$lab)
    )
    invisible(act$val)
}

expect_not_null = function (object, info = NULL, label = NULL) {
    act = testthat::quasi_label(rlang::enquo(object), label, arg = 'object')
    testthat::expect_false(is.null(object), info = info, label = act$lab)
}

expect_box_error = function (object, regexp = NULL, class = NULL, ..., info = NULL, label = NULL) {
    expect_error(
        object = object,
        regexp = regexp,
        class = c(class, 'box_error'),
        ...,
        info = info,
        label = label
    )
}

expect_messages = function (object, has = NULL, has_not = NULL, info = NULL, label = NULL) {
    self = environment()
    messages = character(0L)

    act = withCallingHandlers(
        testthat::quasi_label(rlang::enquo(object), label, arg = 'object'),
        message = function (m) {
            self$messages = c(self$messages, sub('\\n$', '', conditionMessage(m)))
            invokeRestart('muffleMessage')
        }
    )

    pretty_messages = function (which) {
        paste('*', vapply(messages[which], deparse, character(1L)), collapse = '\n')
    }

    if (! is.null(has) && length(has) != length(messages)) {
        testthat::expect(
            FALSE,
            sprintf(
                '%s did not produce %s message(s). It produced:\n%s\n\nExpected:\n%s',
                act$lab,
                length(messages),
                pretty_messages(TRUE),
                paste('*', vapply(has[TRUE], deparse, character(1L)), collapse = '\n')
            ),
            info = info
        )
    }

    expected = unlist(box:::map(grepl, has, messages))

    if (! all(expected)) {
        # We can’t use `testthat::expect(all(expected), …)` here, since that will cause the subsequent assertion to
        # be ignored inside a nested assertion, such as when used inside `expect_failure`. This caused the test of
        # this helper itself to fail.
        testthat::expect(
            FALSE,
            sprintf(
                '%s did not produce the expected message(s). It produced:\n%s\n\nExpected:\n%s',
                act$lab,
                pretty_messages(! expected),
                paste('*', vapply(has[! expected], deparse, character(1L)), collapse = '\n')
            ),
            info = info
        )
    }

    unexpected = vapply(
        messages,
        function (m) any(vapply(has_not, grepl, logical(1L), m)),
        logical(1L)
    )

    if (any(unexpected)) {
        testthat::expect(
            FALSE,
            sprintf(
                '%s produced unwanted message(s):\n%s',
                act$lab,
                pretty_messages(unexpected)
            ),
            info = info
        )
    }
}

in_globalenv = function (expr) {
    old_ls = ls(.GlobalEnv, all.names = TRUE)
    on.exit({
        new_ls = ls(.GlobalEnv, all.names = TRUE)
        to_delete = setdiff(new_ls, old_ls)
        rm(list = to_delete, envir = .GlobalEnv)
    })
    eval.parent(substitute(eval(quote(expr), .GlobalEnv)))
}

in_source_repo = local({
    in_tests = grepl('tests/testthat$', getwd())
    basedir = if (in_tests) dirname(dirname((getwd()))) else getwd()
    file.exists(file.path(basedir, 'DESCRIPTION'))
})

skip_outside_source_repos = function () {
    skip_if(! in_source_repo, 'Outside source code repository')
}

#' Environment of loaded modules
#'
#' Each module is stored as an environment inside \code{loaded_mods} with the
#' module’s code location path as its identifier. The path rather than the
#' module name is used because module names are not unique: two modules called
#' \code{a} can exist nested inside modules \code{b} and \code{c}, respectively.
#' Yet these may be loaded at the same time and need to be distinguished.
#'
#' \code{is_mod_loaded} tests whether a module is already loaded.
#'
#' \code{register_mod} caches a module namespace and marks the module as loaded.
#'
#' \code{deregister_mod} removes a module namespace from the cache, unloading
#' the module from memory.
#'
#' \code{loaded_mod} retrieves a loaded module namespace given its info.
#'
#' \code{is_mod_still_loading} tests whether a module is still being loaded.
#'
#' \code{mod_loading_finished} signals that a module has been completely loaded.
#'
#' @format \code{loaded_mods} is an environment of the loaded module and package
#' namespaces.
#'
#' @keywords internal
#' @name loaded
loaded_mods = new.env(parent = emptyenv())

#' @param info the mod info of a module
#' @rdname loaded
is_mod_loaded = function (info) {
    info$source_path %in% names(loaded_mods)
}

#' @param mod_ns module namespace environment
#' @rdname loaded
register_mod = function (info, mod_ns) {
    loaded_mods[[info$source_path]] = mod_ns
    attr(loaded_mods[[info$source_path]], 'loading') = TRUE
}

#' @rdname loaded
deregister_mod = function (info) {
    # May already have been removed if called by `unload_mod_recursive`, where
    # a given module is imported by multiple submodules.
    if (exists(info$source_path, envir = loaded_mods)) {
        rm(list = info$source_path, envir = loaded_mods)
    }
}

#' @rdname loaded
loaded_mod = function (info) {
    loaded_mods[[info$source_path]]
}

#' @note \code{is_mod_still_loading} and \code{mod_loading_finished} are used to
#' break cycles during the loading of modules with cyclic dependencies.
#' @rdname loaded
is_mod_still_loading = function (info) {
    # pkg_info has no `source_path` but already finished loading anyway.
    ! is.null(info$source_path) && attr(loaded_mods[[info$source_path]], 'loading')
}

#' @rdname loaded
mod_loading_finished = function (info, mod_ns) {
    attr(loaded_mods[[info$source_path]], 'loading') = FALSE
}

# The following defines a function which has a function inside its body. This is
# *different* from a nested function; that is, `f` is different from
#
#   f1 = function () { identity }
#
# … because `f1` contains a *name* in its body. Likewise, it is different from
#
#   f2 = function () { function (x) x }
#
# … because `f2` contains a *call expression* which, when executed, *defines* a
# function, in its body. That is:
#
#   class(body(f)[[2L]])  == 'function'
#   class(body(f1)[[2L]]) == 'name'
#   class(body(f2)[[2L]]) == 'call'
f = function () { NULL }
body(f)[[2L]] = identity

#' `g` clearly isn’t a generic even though `UseMethod` is used inside a nested
#' function in its body.
#' @export
g = function (x) {
    nested = function () UseMethod('nested')
    nested()
}

#' … nor is `h`.
#' @export
h = function () { NULL }
body(h)[[2L]] = function () UseMethod('foo')

#' Module namespace handling
#'
#' \code{make_namespace} creates a new module namespace.
#' @param info the module info.
#' @return \code{make_namespace} returns the newly created module namespace for
#' the module described by \code{info}.
#' @details
#' The namespace contains a module’s content. This schema is very much like R
#' package organisation. A good resource for this is:
#' <http://obeautifulcode.com/R/How-R-Searches-And-Finds-Stuff/>
#' @note Module namespaces aren’t actual R package namespaces. This is
#' intentional, since R makes strong assumptions about package namespaces that
#' are violated here. In particular, such namespaces would have to be registered
#' in R’s internal namespace registry, and their (de)serialisation is handled by
#' R code which assumes that they belong to actual packges that can be loaded
#' via `loadNamespace`.
#' @name namespace
#' @keywords internal
make_namespace = function (info) {
    # Packages use `baseenv()` instead of `emptyenv()` for the parent
    # environment of `.__NAMESPACE__.`. I don’t know why: there should never be
    # any need for inherited name lookup. We’re only using an environment for
    # `.__module__.` to get a mutable value store.
    ns_attr = new.env(parent = emptyenv())
    ns_attr$info = info
    ns_env = new.env(parent = make_imports_env(info))
    ns_env$.__module__. = ns_attr
    # TODO: Set exports here!
    enable_s3_lookup(ns_env, info)
    structure(ns_env, name = paste0('namespace:', info$name), class = 'box$ns')
}

enable_s3_lookup = function (ns_env, info) {
    ns_env$.packageName = info$name
    # TODO: Create S3 methods table
}

import_decl = function (ns, spec, info) {
    structure(list(ns = ns, spec = spec, info = info), class = 'box$import_decl')
}

make_imports_env = function (info) {
    structure(
        new.env(parent = import_env_parent),
        name = paste0('imports:', info$name),
        class = 'box$imports'
    )
}

legacy_warn_msg = c(
    'Using {call;"} inside a module may cause issues; see the FAQ at ',
    '`{call("vignette", "faq", package = "box")}` for details.'
)

box_library = function (...) {
    warning(fmt(legacy_warn_msg, call = 'library'))
    eval.parent(`[[<-`(match.call(), 1L, library))
}

box_require = function (...) {
    warning(fmt(legacy_warn_msg, call = 'require'))
    eval.parent(`[[<-`(match.call(), 1L, require))
}

box_source = function (file, local = FALSE, ...) {
    if (is.logical(local) && ! local) {
        warning(fmt(legacy_warn_msg, call = 'source'))
    }
    eval.parent(`[[<-`(match.call(), 1L, source))
}

legacy_intercept_env = list2env(
    list(
        library = box_library,
        require = box_require,
        source = box_source
    ),
    parent = baseenv()
)

#' \code{is_namespace} checks whether a given environment corresponds to a
#' module namespace.
#' @param env an environment that may be a module namespace.
#' @rdname namespace
is_namespace = function (env) {
    exists('.__module__.', env, mode = 'environment', inherits = FALSE)
}

#' @param ns the module namespace environment.
#' @param which the key (as a length 1 character string) of the info to get/set.
#' @param default default value to use if the key is not set.
#' @rdname namespace
namespace_info = function (ns, which, default = NULL) {
    get0(which, ns$.__module__., inherits = FALSE, ifnotfound = default)
}

#' @param value the value to assign to the specified key.
#' @rdname namespace
`namespace_info<-` = function (ns, which, value) {
    assign(which, value, envir = ns$.__module__.)
    ns
}

#' Get a module’s name
#'
#' @usage \special{box::name()}
#' @return \code{box::name} returns a character string containing the name of
#' the module, or \code{NULL} if called from outside a module.
#' @note Because this function returns \code{NULL} if not invoked inside a
#' module, the function can be used to check whether a code is being imported as
#' a module or called directly.
#' @export
name = function () {
    mod_ns = current_mod()
    if (is_namespace(mod_ns)) namespace_info(mod_ns, 'info')$name
}

#' Get a module’s namespace environment
#'
#' Called inside a module, \code{box::topenv()} returns the module namespace
#' environment. Otherwise, it behaves similarly to \code{\link[base]{topenv}}.
#' @usage \special{box::topenv()}
#' @usage \special{box::topenv(env)}
#' @param module a module environment
#' @return \code{box::topenv()} returns the top-level module environment of the
#' module it is called from, or the nearest top-level non-module environment
#' otherwise; this is usually \code{.GlobalEnv}.
#'
#' \code{box::topenv(env)} returns the nearest top-level environment that is a
#' direct or indirect parent of \code{env}.
#' @export
topenv = function (module) {
    if (missing(module)) module = current_mod()
    if (inherits(module, 'box$mod')) attr(module, 'namespace')
    else mod_topenv(module)
}

current_mod = function (env = parent.frame(2L)) {
    mod_topenv(env)
}

#' \code{mod_topenv} is the same as \code{topenv} for module namespaces.
#' @name namespace
mod_topenv = function (env = parent.frame()) {
    while (! is_mod_topenv(env)) env = parent.env(env)
    env
}

#' \code{is_mod_topenv} returns \code{TRUE} if \code{env} is a top level
#' environment.
#' @name namespace
is_mod_topenv = function (env) {
    is_namespace(env) || env %==% base::topenv(env) || env %==% emptyenv()
}

#' @keywords internal
make_export_env = function (info, spec, ns) {
    structure(
        new.env(parent = emptyenv()),
        name = paste0('mod:', spec_name(spec)),
        class = 'box$mod',
        spec = spec,
        info = info,
        namespace = ns
    )
}

strict_extract = function (e1, e2) {
    # Implemented in C since this function is called very frequently and needs
    # to be fast, and the C implementation is about 270% faster than an R
    # implementation based on `get`, and provides more readable error messages.
    # In fact, the fastest code that manages to provide a readable error message
    # that contains the actual call ("foo$bar") rather than only mentioning the
    # `get` function call, is more than 350% slower.
    .Call(c_strict_extract, e1, e2)
}

#' @export
`$.box$mod` = strict_extract

#' @export
`$.box$ns` = strict_extract

#' @export
`print.box$mod` = function (x, ...) {
    spec = attr(x, 'spec')
    type = if (inherits(spec, 'pkg_spec')) 'package' else 'module'
    cat(fmt('<{type}: {spec_name(spec)}>\n'))
    invisible(x)
}

find_import_env = function (x, spec, info, mod_ns) {
    UseMethod('find_import_env')
}

`find_import_env.box$ns` = function (x, spec, info, mod_ns) {
    parent.env(x)
}

`find_import_env.box$mod` = function (x, spec, info, mod_ns) {
    x
}

find_import_env.environment = function (x, spec, info, mod_ns) {
    env = if (x %==% .GlobalEnv) {
        # We need to use `attach` here: attempting to set
        # `parent.env(.GlobalEnv)` causes R to segfault.
        box_attach(NULL, name = paste0('mod:', spec_name(spec)))
    } else {
        parent.env(x) = new.env(parent = parent.env(x))
    }
    structure(env, class = 'box$mod', spec = spec, info = info, namespace = mod_ns)
}

import_into_env = function (to_env, to_names, from_env, from_names) {
    for (i in seq_along(to_names)) {
        if (
            exists(from_names[i], from_env, inherits = FALSE)
            && bindingIsActive(from_names[i], from_env)
            && ! inherits((fun = activeBindingFunction(from_names[i], from_env)), 'box$placeholder')
        ) {
            makeActiveBinding(to_names[i], fun, to_env)
        } else {
            assign(to_names[i], env_get(from_env, from_names[i]), envir = to_env)
        }
    }
}

env_get = function (env, name) {
    UseMethod('env_get')
}

# Method for package namespace environments. This distinction is necessary since
# lazydata in packages can’t be loaded via `get`.
env_get.environment = function (env, name) {
    getExportedValue(env, name)
}

`env_get.box$mod` =
`env_get.box$ns` = function (env, name) {
    # Explicitly allow inherited values, which is used to support re-exporting
    # imports in modules.
    get(name, envir = env)
}

#' Wrap \dQuote{unsafe calls} functions
#'
#' \code{wrap_unsafe_function} declares a function wrapper to a function that
#' causes an \command{R CMD check} NOTE when called directly. We should usually
#' not call these functions, but we need some of them because we want to
#' explicitly support features they provide.
#' @param ns The namespace of the unsafe function.
#' @param name The name of the unsafe function.
#' @return \code{wrap_unsafe_calls} returns a wrapper function with the same
#' argument as the wrapped function that can be called without causing a NOTE.
#' @note Using an implementation that simply aliases \code{getExportedValue}
#' does not work, since \command{R CMD check} sees right through this
#' \dQuote{ruse}.
#' @keywords internal
wrap_unsafe_function = function (ns, name) {
    f = getExportedValue(ns, name)
    wrapper = function (...) eval.parent(`[[<-`(match.call(), 1L, f))
    formals(wrapper) = formals(f)
    wrapper
}

box_attach = wrap_unsafe_function(.BaseNamespaceEnv, 'attach')

box_unlock_binding = wrap_unsafe_function(.BaseNamespaceEnv, 'unlockBinding')

context('environments')

test_that('`topenv` works inside modules', {
    box::use(mod/env)

    expected = attr(env, 'namespace')

    expect_identical(env$e1, expected)
    expect_identical(env$e2(), expected)
    expect_identical(env$e3(), expected)
    expect_identical(env$e4()(), expected)
})

test_that('`topenv` works on module environments', {
    box::use(mod/env)
    env_ns = attr(env, 'namespace')
    expect_identical(box::topenv(env), env_ns)

    box::use(stats)
    stats_ns = asNamespace('stats')
    expect_identical(box::topenv(stats), stats_ns)
})

test_that('`topenv` works on module namespaces', {
    box::use(mod/env)
    env_ns = attr(env, 'namespace')
    expect_identical(box::topenv(env_ns), env_ns)

    box::use(stats)
    stats_ns = asNamespace('stats')
    expect_identical(box::topenv(stats_ns), stats_ns)
})

test_that('`topenv` works on non-module environments', {
    expect_identical(in_globalenv(box::topenv()), .GlobalEnv)

    e = new.env(parent = .GlobalEnv)
    expect_identical(local(box::topenv(), envir = e), .GlobalEnv)

    stats_env = as.environment('package:stats')
    expect_identical(box::topenv(stats_env), stats_env)
})

context('packages')

# Helper function to check for existence of an attached name (which is expected
# to be found in the parent environment of the calling environment).
local({
    pls = function () {
        ls(envir = parent.env(parent.frame()))
    }
}, .GlobalEnv)

test_that('R core packages can be used', {
    expect_not_in('methods', ls())
    box::use(methods)
    expect_true(isNamespaceLoaded('methods'))
    expect_in('methods', ls())
    expect_in('as', ls(methods))
    expect_identical(methods$as, methods::as)
})

# Use ‘devtools’ as a test package since it’s suggested by testthat and required
# for ‘box’/ development, so highly likely to be installed, but hopefully not
# loaded.

test_that('previously loaded user packages can be used', {
    loadNamespace('devtools')
    expect_not_in('devtools', ls())
    expect_true(isNamespaceLoaded('devtools'))
    box::use(devtools)
    expect_in('devtools', ls())
    expect_in('load_all', ls(devtools))
    expect_identical(devtools$load_all, devtools::load_all)
})

test_that('unloaded user packages can be used', {
    unloadNamespace('devtools')
    expect_not_in('devtools', ls())
    expect_false(isNamespaceLoaded('devtools'))
    box::use(devtools)
    expect_true(isNamespaceLoaded('devtools'))
    expect_in('devtools', ls())
    expect_in('load_all', ls(devtools))
    expect_identical(devtools$load_all, devtools::load_all)
})

test_that('packages can be aliased', {
    expect_not_in('methods', ls())
    expect_not_in('m', ls())
    box::use(m = methods)
    expect_not_in('methods', ls())
    expect_in('m', ls())
    expect_in('as', ls(m))
    expect_identical(m$as, methods::as)
})

test_that('packages can be used with aliases', {
    expect_not_in('dev', ls())
    box::use(dev = devtools)
    expect_in('dev', ls())
    expect_identical(dev$load_all, devtools::load_all)
})

test_that('things can be attached locally', {
    expect_not_in('load_all', pls())
    box::use(devtools[load_all])
    expect_in('load_all', pls())

    expect_not_in('unload', pls())
    expect_not_in('reload', pls())
    box::use(devtools[unload, reload])
    expect_in('unload', pls())
    expect_in('reload', pls())
})

test_that('all things can be attached locally', {
    devtools_exports = getNamespaceExports('devtools')
    expect_gt(length(devtools_exports), 0L) # Sanity check …
    box::use(devtools[...])
    expect_false(any(is.na(match(pls(), devtools_exports))))
})

test_that('things can be attached globally', {
    # Custom assertions are not attached, and are made available differently in
    # `devtools::test()` and `R CMD check`. In particular, the latter doesn’t
    # find them in the ‘box’ namespace either. That’s why the following code
    # computes all values in individual .GlobalEnv wrappers, rather than
    # wrapping the whole unit test in `in_globalenv`.

    objs = ls(envir = .GlobalEnv)
    on.exit(rm(list = setdiff(ls(envir = .GlobalEnv), objs), envir = .GlobalEnv))

    gpls = function () local(pls(), envir = .GlobalEnv)

    expect_not_in('load_all', gpls())
    local(box::use(devtools[load_all]), envir = .GlobalEnv)
    expect_in('load_all', gpls())
    expect_not_in('unload', gpls())
    expect_not_in('reload', gpls())
    local(box::use(devtools[unload, reload]), envir = .GlobalEnv)
    expect_in('unload', gpls())
    expect_in('reload', gpls())
})

test_that('all things can be attached globally', {
    in_globalenv({
        devtools_exports = getNamespaceExports('devtools')
        expect_gt(length(devtools_exports), 0L) # Sanity check …
        box::use(devtools[...])
        expect_false(any(is.na(match(pls(), devtools_exports))))
    })
})

test_that('attachments can be aliased', {
    expect_not_in('u', pls())
    expect_not_in('reload', pls())
    expect_not_in('la', pls())
    box::use(devtools[u = unload, reload, la = load_all])
    expect_in('u', pls())
    expect_in('reload', pls())
    expect_in('la', pls())
    expect_not_in('unload', pls())
    expect_not_in('load_all', pls())
})

test_that('wildcard attachments can contain aliases', {
    # Attach everything, and give some names aliases
    devtools_exports = getNamespaceExports('devtools')
    expected = c(
        setdiff(devtools_exports, c('test', 'r_env_vars')),
        'test_alias', 'ev'
    )
    box::use(devtools[..., test_alias = test, ev = r_env_vars])
    expect_equal(length(pls()), length(expected))
    expect_false(any(is.na(match(pls(), expected))))
})

test_that('non-existent aliases raise error', {
    expect_box_error(box::use(devtools[foobar123]))
    expect_box_error(box::use(devtools[test = foobar123]))
    expect_box_error(box::use(devtools[..., test = foobar123]))
})

test_that('only exported things can be attached', {
    expect_in('yesno', ls(getNamespace('devtools')))
    expect_box_error(box::use(devtools[yesno]), 'not exported')
})

test_that('packages that attach things are not aliased', {
    box::use(devtools[load_all])
    expect_not_in('devtools', ls())
    expect_in('load_all', pls())
})

test_that('packages that attach things are can be aliased', {
    expect_not_in('dev', ls())
    box::use(dev = devtools[load_all])
    expect_in('dev', ls())
    expect_in('load_all', pls())
})

test_that('S3 lookup works for partial exports', {
    # ‘devtools’ has no S3 exports at the time of writing but ‘roxygen2’ does,
    # and is also needed to compile this package — and hence likely available
    # when running these tests.
    box::use(roxygen2[parse = roxy_tag_parse])
    x = structure(list(raw = '{}'), class = 'roxy_tag_usage')
    actual = parse(x)
    expected = roxygen2::roxy_tag_parse(x)
    expect_equal(actual, expected)
})

test_that('packages with lazydata can be loaded', {
    # Data inside ‘datasets’ is available via `get`.
    expect_error(box::use(datasets), NA)
    expect_error(get('mtcars', asNamespace('datasets')), NA)

    # ‘stringr’ uses lazydata; `get` doesn’t work.
    expect_error(box::use(stringr), NA)
    expect_error(get('fruit', asNamespace('stringr')))
    expect_in('fruit', ls(stringr))
    expect_type(stringr$fruit, 'character')

    # Test that attach lists work
    expect_error(box::use(stringr[ff = fruit, ...]), NA)
    expect_equal(ff, stringr$fruit)
    expect_equal(words, stringr$words)
})

context('reexport')

test_that('module reexport exposes the correct names', {
    box::use(x = mod/reexport)

    xns = attr(x, 'namespace')
    exports = box:::namespace_info(xns, 'exports')
    direct_exports = c('a', 'c')
    sub_exports = c('d', 'e')
    all_expected_exports = c(direct_exports, sub_exports, 'sub')
    expect_setequal(exports, all_expected_exports)
})

test_that('module reexports the correct names', {
    box::use(x = mod/reexport)

    exported_names = c('a', 'c', 'sub', 'd', 'e')
    expect_in('a', ls(x))
    expect_not_in('b', ls(x))
    expect_in('c', ls(x))
    expect_in('sub', ls(x))
    expect_in('d', ls(x))
})

test_that('module reexport names refer to correct objects', {
    box::use(x = mod/reexport)

    expect_equal(x$sub$a, 'sub$a')
    expect_equal(x$sub$b, 'sub$b')
    expect_equal(x$c, 'reexport$c')
    expect_equal(x$d, 'sub$c')
    expect_equal(x$e, 'sub$b')
})

context('basic')

is_module_loaded = function (mod) {
    box:::is_mod_loaded(attr(mod, 'info'))
}

test_teardown(clear_mods())

module_source_path = function (mod) {
    attr(mod, 'info')$source_path
}

test_that('module can be imported', {
    box::use(mod/a)
    expect_true(is_module_loaded(a))
    expect_in('double', ls(a))
})

test_that('import works in global namespace', {
    in_globalenv({
        box::use(mod/a)
        expect_true(box:::is_mod_loaded(attr(a, 'info')))
        # `expect_in` isn’t found here.
        expect_true('double' %in% ls(a))
    })
})

test_that('module is uniquely identified by path', {
    box::use(mod/a)
    box::use(ba = mod/b/a)
    expect_true(is_module_loaded(a))
    expect_true(is_module_loaded(ba))
    expect_not_identical(module_source_path(a), module_source_path(ba))
    expect_in('double', ls(a))
    expect_not_in('double', ls(ba))
})

test_that('can use imported function', {
    box::use(mod/a)
    expect_equal(a$double(42), 42 * 2)
})

test_that('modules export all objects', {
    box::use(mod/a)
    expect_gt(length(lsf.str(a)), 0L)
    expect_gt(length(ls(a)), length(lsf.str(a)))
    a_namespace = environment(a$double)
    expect_equal(a$get_counter(), 1L)
})

test_that('module can modify its variables', {
    box::use(mod/a)
    counter = a$get_counter()
    a$inc()
    expect_equal(a$get_counter(), counter + 1L)
})

test_that('hidden objects are not exported', {
    box::use(mod/a)
    ns = environment(a$get_counter)
    expect_true(exists('modname', envir = a))
    expect_true(exists('make_counter', envir = ns))
    expect_false(exists('make_counter', envir = a))
    expect_true(exists('private_modname', envir = ns))
    expect_false(exists('private_modname', envir = a))
})

test_that('module bindings are locked', {
    box::use(mod/a)

    ns = attr(a, 'namespace')

    expect_true(environmentIsLocked(a))
    expect_true(environmentIsLocked(ns))
    expect_true(bindingIsLocked('get_counter', a))
    expect_true(bindingIsLocked('modname', a))
    expect_true(bindingIsLocked('get_counter', ns))
    expect_true(bindingIsLocked('modname', ns))

    err = try({a$counter = 2L}, silent = TRUE)
    expect_s3_class(err, 'try-error')
})

test_that('modules don’t need exports', {
    expect_equal(ls(), character(0L))
    expect_error(box::use(mod/no_exports), NA)
    expect_error(capture.output(box::use(mod/no_names)), NA)
    expect_setequal(ls(), c('no_exports', 'no_names'))
})

test_that('modules can be empty', {
    expect_error(box::use(mod/empty), NA)
    expect_error(box::use(mod/export_empty), NA)
})

test_that('global scope is not leaking into modules', {
    in_globalenv({
        x = 1L
        expect_error(box::use(mod/issue151), 'object .* not found')
    })
})

test_that('package exports do not leak into modules', {
    expect_true('package:stats' %in% search())
    box::use(mod/a)
    ns_a = attr(a, 'namespace')
    # First, ensure this is run in the right environment:
    expect_equal(get0('modname', envir = ns_a, inherits = FALSE), 'a')
    expect_true(get0('T', envir = ns_a))
    expect_null(get0('t.test', envir = ns_a))
    expect_not_null(get0('t.test', envir = .GlobalEnv))
})

test_that('partial name causes error', {
    box::use(mod/a)
    expect_error(a$double, NA)
    expect_error(a$doubl, "name 'doubl' not found in 'a'")

    # Check whether error call is correct:
    error = tryCatch(a$doubl, error = identity)
    expect_s3_class(error, 'simpleError')
    expect_identical(error$call, quote(a$doubl))
})

test_that('trailing comma is accepted', {
    expect_error(box::use(mod/a, ), NA)
    expect_error(box::use(mod/a, mod/b, ), NA)
    expect_error(box::use(mod/a[modname, double, ]), NA)
})

test_that('nested module can use parent', {
    box::use(mod/b/b)
    expect_true(exists('z', b))
    expect_equal(b$z, 1)
})

test_that('using legacy functions raises warning', {
    on.exit({
        box::unload(library)
        box::unload(require)
        box::unload(source)
    })

    expect_warning(box::use(mod/legacy/library), '.+library.+ inside a module')
    expect_warning(box::use(mod/legacy/require), '.+require.+ inside a module')

    expect_false(exists('source_test', envir = .GlobalEnv))
    on.exit(rm(source_test, envir = .GlobalEnv))
    expect_warning(box::use(mod/legacy/source), '.+source.+ inside a module')
    expect_true(exists('source_test', envir = .GlobalEnv))
})

test_that('legacy function warning can be silenced', {
    old_opts = options(box.warn.legacy = FALSE)
    box:::set_import_env_parent()
    on.exit({
        options(old_opts)
        box:::set_import_env_parent()
    })

    expect_warning(box::use(mod/legacy/library), NA)
    expect_warning(box::use(mod/legacy/require), NA)

    expect_false(exists('source_test', envir = .GlobalEnv))
    on.exit(rm(source_test, envir = .GlobalEnv), add = TRUE)
    expect_warning(box::use(mod/legacy/source), NA)
    expect_true(exists('source_test', envir = .GlobalEnv))
})

test_that('r/core can be imported', {
    # All the test case logic is inside the `core_test` module.
    box::use(mod/core_test)

    # Ensure tests were actually run:
    expect_gt(core_test$tests_run, 0L)
})

test_that('modules can be imported and exported by different local names', {
    expect_error(box::use(mod/issue211), NA)
})

test_that('legacy modules export all names', {
    is_legacy = function (mod) {
        box:::namespace_info(attr(mod, 'namespace'), 'legacy', default = FALSE)
    }

    box::use(
        mod/a,
        mod/legacy,
        mod/no_exports
    )

    expect_false(is_legacy(a))
    expect_true(is_legacy(legacy))
    expect_false(is_legacy(no_exports))

    expect_setequal(ls(legacy), c('a', 'b'))
    expect_setequal(ls(no_exports), character())
})

test_that('modules can specify explicit exports', {
    box::use(ex = mod/explicit_exports)
    expect_setequal(ls(ex), c('a', 'double', 'modname'))
    expect_identical(ex$double, attr(ex, 'namespace')$double)
    expect_equal(ex$modname, ex$a$modname)
})

test_that('`box::export()` checks its arguments', {
    # Create a test module namespace since `box::export()` can only be called inside a module.
    test_spec = box:::mod_spec(list(mod = 'test', name = 'test', explicit = FALSE))
    test_info = box:::mod_info(test_spec, '.')
    test_ns = box:::make_namespace(test_info)
    test_ns$a = 1

    expect_error(local(box::export(a), envir = test_ns), NA)
    expect_error(
        local(box::export(a = a), envir = test_ns),
        'unexpected named argument'
    )
})

test_that('legcay modules do not call hooks', {
    # Ensure module is first unloaded:
    box::use(mod/legacy)
    box::unload(legacy)

    expect_messages(
        box::use(mod/legacy),
        has = 'legacy module loaded',
        has_not = '\\.on_load is not called'
    )
})

#' String formatting helpers
#'
#' Interpolate expressions in a string
#' @param \dots one or more unnamed character string arguments, followed
#' optionally by named arguments
#' @param x an object to convert
#' @param a a character vector of length \code{n}
#' @param b a character vector of length \code{n - 1}
#' @return \code{fmt(\dots)} concatenates any unnamed arguments, and
#' interpolates all embedded expressions as explained in the \sQuote{Details}.
#' Named arguments are treated as locally defined variables, and are added to
#' (and override, in case of name reuse) names defined in the calling scope.
#' @details
#' \code{fmt} interpolates embedded expressions in a string.
#' \code{chr} converts a value to a character vector; unlike
#' \code{as.character}, it correctly deparses unevaluated names and expressions.
#' \code{interleave} is a helper that interleaves two vectors \code{a = c(a[1],
#' \dots, a[n])} and \code{b = c(b[1], \dots, b[n - 1])}.
#'
#' The general format of an interpolation expression inside a \code{fmt} string
#' is: \code{\{\dots\}} interpolates the expression \code{\dots}. To insert
#' literal braces, double them (i.e. \code{\{\{}, \code{\}\}}). Interpolated
#' expressions can optionally be followed by a \emph{format modifier}: if
#' present, it is specified via the syntax \code{\{\dots;modifier\}}. The
#' following modifiers are supported:
#' \describe{
#'  \item{\code{\"}}{like \code{dQuote(\dots)}}
#'  \item{\code{\'}}{like \code{sQuote(\dots)}}
#'  \item{\code{‹fmt›f}}{like \code{sprintf('\%‹fmt›f', \dots)}}
#' }
#' Vectors of length > 1 will be concatenated as if using
#' \code{\link[base]{toString}} before interpolation.
#' @keywords internal
fmt = function (..., envir = parent.frame()) {
    dots = list(...)
    named = nzchar(names(dots))
    str = paste(unlist(dots[(! named) %||% TRUE]), collapse = '')
    vars = dots[named]

    matches = gregexpr(
        '(?<op>\\{\\{)|(?<cp>\\}\\})|\\{(?<expr>[^};]+)(;(?<mod>[^}]+))?\\}',
        str,
        perl = TRUE
    )[[1L]]

    if (matches[[1L]] == -1L) return(str)

    glue = regmatches(str, list(matches), invert = TRUE)[[1L]]
    pos = attr(matches, 'capture.start')
    len = attr(matches, 'capture.length')
    what = attr(matches, 'capture.names')[apply(pos, 1L, Position, f = identity)]

    interp = map_chr(function (row) {
        switch(
            what[row],
            op = '{',
            cp = '}',
            {
                p = Find(identity, pos[row, ])
                l = Find(identity, len[row, ])
                expr = substr(str, p, p + l - 1L)
                val = eval(str2lang(expr), envir = vars, enclos = envir)

                mod = pos[row, 'mod']
                res = if (mod == 0L) {
                    chr(val)
                } else {
                    fmt = substr(str, mod, mod + len[row, 'mod'] - 1L)
                    switch(
                        substr(fmt, nchar(fmt), nchar(fmt)),
                        `"` = dQuote(chr(val)),
                        `'` = sQuote(chr(val)),
                        f = sprintf(paste0('%', fmt), val),
                        throw('unrecognized format modifier {fmt;"}')
                    )
                }
                paste(res, collapse = ', ')
            }
        )
    }, seq_along(what))

    paste(interleave(glue, interp), collapse = '')
}

#' @return \code{chr(x)} returns a string representation of a value or
#' unevaluated expression \code{x}.
#' @rdname fmt
chr = function (x) {
    UseMethod('chr')
}

chr.default = function (x) {
    as.character(x)
}

chr.call =
chr.for =
chr.if =
chr.while =
`chr.(` =
`chr.{` =
`chr.=` = function (x) {
    deparse1(x)
}

# The following is required because of a bug in `R CMD CHECK` which erroneously
# flags up a regular declaration of `chr.<-` because it thinks the function is a
# *replacement function*, which requires a `value` argument.
# Adding a dummy `value` argument as a workaround to silence this warning, which
# used to work before R 4.3.0, now fails as well because `R CMD CHECK` rightly
# complains that this function has a different signature than the generic.
# See <https://stackoverflow.com/q/69674485/1968> for more information.
# Alternatively we could export the definitions and add the following:
#
# #' @rawNamespace S3method(chr, "<-", "chr.=")
#
# … however, these are internal functions that should not be exported.
registerS3method('chr', '<-', `chr.=`, environment())

chr.expression = function (x) {
    chr(x[[1L]])
}

chr.name = function (x) {
    deparse1(x, backtick = TRUE)
}

#' @return \code{html_escape(x)} returns the HTML-escaped version of \code{x}.
#' @rdname fmt
html_escape = function (x) {
    from = c('&', '<', '>', '"', "'")
    to = c('&amp;', '&lt;', '&gt;', '&quot;', '&apos;')
    substitutions = transpose(from, to)
    Reduce(function (x, r) gsub(r[1L], r[2L], x), substitutions, x)
}

#' @return \code{interleave(a, b)} returns a vector \code{c(a[1], b[1], a[2],
#' b[2], \dots, a[n - 1], b[n - 1], a[n])}.
#' @rdname fmt
interleave = function (a, b) {
    index = order(c(seq_along(a), seq_along(b)))
    c(a, b)[index]
}

#' Display module documentation
#'
#' \code{box::help} displays help on a module’s objects and functions in much
#' the same way \code{\link[utils]{help}} does for package contents.
#'
#' @usage \special{box::help(topic, help_type = getOption("help_type", "text"))}
#' @param topic either the fully-qualified name of the object or function to get
#'  help for, in the format \code{module$function}; or a name that was exported
#'  and attached from an imported module or package.
#' @param help_type character string specifying the output format; currently,
#'  only \code{'text'} is supported.
#' @return \code{box::help} is called for its side effect when called directly
#' from the command prompt.
#' @details
#' See the vignette at \code{vignette('box', 'box')} for more information about
#' displaying help for modules.
#' @export
help = function (topic, help_type = getOption('help_type', 'text')) {
    topic = substitute(topic)
    target = help_topic_target(topic, parent.frame())
    target_mod = target[[1L]]
    subject = target[[2L]]

    if (! inherits(target_mod, 'box$mod')) {
        throw('{topic;"} is not a valid module help topic')
    }

    if (subject != '.__module__.') {
        obj = if (
            exists(subject, target_mod, inherits = FALSE)
            && ! bindingIsActive(subject, target_mod)
        ) get(subject, envir = target_mod, inherits = FALSE)

        if (inherits(obj, 'box$mod')) {
            target_mod = obj
            subject = '.__module__.'
        }
    }

    info = attr(target_mod, 'info')
    mod_name = strsplit(attr(target_mod, 'name'), ':')[[1L]][2L]

    if (inherits(info, 'box$pkg_info')) {
        help_call = if (subject == '.__module__.') {
            bquote(help(.(as.name(mod_name))))
        } else {
            bquote(help(topic = .(subject), package = .(mod_name)))
        }
        return(call_help(help_call, parent.frame()))
    }

    if (! requireNamespace('roxygen2')) {
        throw('displaying documentation requires {"roxygen2";\'} installed')
    }

    mod_ns = attr(target_mod, 'namespace')
    all_docs = namespace_info(mod_ns, 'doc')

    if (is.null(all_docs)) {
        all_docs = parse_documentation(info, mod_ns)
        namespace_info(mod_ns, 'doc') = all_docs
    }

    doc = all_docs[[subject]]

    if (is.null(doc)) {
        if (subject == '.__module__.') {
            throw('no documentation available for {mod_name;"}')
        } else {
            throw('no documentation available for {subject;"} in module {mod_name;"}')
        }
    }

    display_help(doc, mod_name, help_type)
}

#' Parse a module’s documentation
#'
#' @param info The module info.
#' @param mod_ns The module namespace.
#' @return \code{parse_documentation} returns a list of character strings with
#' the Rd documentation source code for each documented name in a module.
#' @keywords internal
parse_documentation = function (info, mod_ns) {
    rdfiles = parse_roxygen_tags(info, mod_ns)
    # Due to aliases, documentation entries may have more than one name.
    aliases = map(function (rd) unique(rd$get_value('alias')), rdfiles)
    names = rep(names(rdfiles), lengths(aliases))
    docs = patch_mod_doc(stats::setNames(rdfiles[names], unlist(aliases)))

    lapply(docs, format, wrap = FALSE)
}

#' @keywords internal
#' @rdname parse_documentation
parse_roxygen_tags = function (info, mod_ns) {
    mod_path = info$source_path
    blocks = roxygen2::parse_file(mod_path, mod_ns)
    roxygen2::roclet_process(
        roxygen2::rd_roclet(),
        blocks,
        mod_ns,
        dirname(mod_path)
    )
}

#' @param docs the list of \pkg{roxygen2} documentation objects.
#' @keywords internal
#' @rdname parse_documentation
patch_mod_doc = function (docs) {
    if ('.__module__.' %in% names(docs)) {
        mod_doc = docs[['.__module__.']]
        mod_doc$sections$docType$value = 'package'
        mod_doc$sections$usage = NULL
        mod_doc$sections$format = NULL
        mod_doc$sections$name$value = 'module'
    }

    docs
}

#' Helper functions for the help functionality
#'
#' \code{help_topic_target} parses the expression being passed to the
#' \code{help} function call to find the innermost module subset expression in
#' it.
#' \code{find_env} acts similarly to \code{\link[utils]{find}}, except that it
#' looks in the current environment’s parents rather than in the global
#' environment search list, it returns only one hit (or zero), and it returns
#' the environment rather than a character string.
#' \code{call_help} invokes a \code{help()} call expression for a package help
#' topic, finding the first \code{help} function definition, ignoring the one
#' from this package.
#'
#' @param topic the unevaluated expression passed to \code{help}.
#' @param caller the environment from which \code{help} was called.
#' @return \code{help_topic_target} returns a list of two elements containing
#' the innermost module of the \code{help} call, as well as the name of the
#' object that’s the subject of the \code{help} call. For \code{help(a$b$c$d)},
#' it returns \code{list(c, quote(d))}.
#' @name help-internal
#' @keywords internal
help_topic_target = function (topic, caller) {
    inner_mod = function (mod, expr) {
        name = if (is.name(expr)) {
            as.character(expr)
        } else if (is.call(expr) && expr[[1L]] %==% quote(`$`)) {
            mod = Recall(mod, expr[[2L]])
            as.character(expr[[3L]])
        } else {
            throw('{topic;"} is not a valid module help topic', call = call)
        }
        get(name, envir = mod)
    }

    call = sys.call(-1L)

    if (is.name(topic)) {
        obj = inner_mod(caller, topic)

        if (inherits(obj, 'box$mod')) {
            list(obj, '.__module__.')
        } else {
            name = as.character(topic)
            list(find_env(name, caller), name)
        }
    } else {
        list(inner_mod(caller, topic[[2L]]), as.character(topic[[3L]]))
    }
}

#' @param name the name to look for.
#' @name help-internal
#' @keywords internal
find_env = function (name, caller) {
    while (caller %!=% emptyenv()) {
        if (exists(name, envir = caller, inherits = FALSE)) return(caller)
        caller = parent.env(caller)
    }
    NULL
}

#' @param call the patched \code{help} call expression.
#' @name help-internal
call_help = function (call, caller) {
    # Search for `help` function in caller scope. This is intended to find the
    # first help call which is either `utils::help` or potentially from another
    # environment, such as `devtools_shims`.
    # Unfortunately during testing (and during development) the current package
    # *is* attached, so we can’t just use `get` in the global/caller’s
    # environment — it would recurse indefinitely to this package’s `help`
    # function. To fix this, we need to manually find the first hit that isn’t
    # inside this package.
    candidates = utils::getAnywhere('help')
    envs = map(environment, candidates$objs)
    valid = candidates$visible & map_lgl(is.function, candidates$objs)
    other_helps = candidates$objs[valid & ! map_lgl(identical, envs, base::topenv())]

    call[[1L]] = other_helps[[1L]]
    eval(call, envir = caller)
}

#' @export
test = function (x) UseMethod('test')

test.default = function (x) 'test.default'

test.character = function (x) 'test.character'

print.test = function (x) 's3$print.test'

box::register_S3_method('print', 'test', print.test)

print.test2 = function (x) 's3$print.test2'

box::register_S3_method('print', 'test2')

#' @export
se = function (...) UseMethod('se')

se.default = function (...) 's3$se.default'

se.contrast.test = function (...) 's3$se.contrast.test'

# Adding a method to a generic defined in another package requires importing it.
box::use(stats[se.contrast])

box::register_S3_method('se.contrast', 'test', se.contrast.test)

#' A test module
#'
#' With a description
'.__module__.'

#' Double a number
#'
#' Add a number to itself by the cunning use of arithmetic.
#'
#' @param x a number
#' @return \code{x * 2}
#' @export
double = function (x) x * 2

#' @export
modname = box::name()

private_modname = box::name()

# Variables at namespace scope are locked so mutable variables need to be
# wrapped inside a closure.
make_counter = function () {
    self = environment()
    counter = 1L

    list(
        get = function () counter,
        inc = function () self$counter = counter + 1L
    )
}

counter = make_counter()

#' The module’s name
#' @export
get_modname = function () private_modname

#' The module’s name, via a function
#' @name get_modname
#' @export
get_modname2 = function () box::name()

#' Read the counter
#' @export
get_counter = counter$get

#' Increment the counter
#' @export
inc = counter$inc

#' Use \code{a} if it exists, else \code{b}
#' @export
`%or%` = function (a, b)
    if (length(a) > 0L) a else b

#' String concatenation
#' @export
`+.string` = function (a, b)
    paste(a, b, sep = '')

#' @export
which = function () '/a'

#' @export
encoding_test = function () '☃' # U+2603: SNOWMAN

#' @export
.hidden = 1L

# Test cases for the issue 42. The semantics of the functions don’t matter so I
# am using whimsical place-holders. What matters is whether or not the functions
# get exported as operators.

#' @export
`%.%` = function (f, g) function (x) f(g(x))

#' @export
`%x.%` = function (x, f) f(x)

#' @export
`%.x%` = function (f, x) f(x)

#' @export
`%x.x%` = function (x, y) function (f) f(x, y)

#' @export
`%foo.bar` = function (x) message(x)

#' @export
`%%.%%` = function (a, b) list(a, b)

#' @export
`%a%.class%` = function (a, b) list(a, b)

context('encoding')

test_that('source is read as UTF-8', {
    box::use(mod/a)
    expected_bytes = as.raw(c(0xE2, 0x98, 0x83))
    expect_equal(nchar(a$encoding_test(), 'bytes'), 3L)
    expect_equal(Encoding(a$encoding_test()), 'UTF-8')
    expect_equal(charToRaw(a$encoding_test()), expected_bytes)
})

define_backports = function (ns) {
    if (getRversion() < '4.0.0') {
        ns$deparse1 = function (expr, collapse = ' ', width.cutoff = 500L, ...) {
            paste(deparse(expr, width.cutoff, ...), collapse = collapse)
        }

        ns$activeBindingFunction = function (sym, env) {
            class = class(env)
            on.exit({class(env) = class})
            as.list(`class<-`(env, NULL), all.names = TRUE)[[sym]]
        }

        ns$tryInvokeRestart = function (r, ...) {
            if (! isRestart(r)) r = findRestart(r)
            if (! is.null(r)) invokeRestart(r, ...)
        }
    }
}

f = function() {
    box::use(./a[...])
    helper()
}

g = function() {
    box::use(./a)
    helper()
}

h = function() {
    box::use(./a[`%.%`])
    helper()
}

ns = environment()

#' @export
helper_var = 0L

helper = function() { ns$helper_var = helper_var + 1L }

f()
g()
h()

#' Register S3 methods
#'
#' \code{box::register_S3_method} makes an S3 method for a given generic and
#' class known inside a module.
#'
#' @usage \special{box::register_S3_method(name, class, method)}
#' @param name the name of the generic as a character string.
#' @param class the class name as a character string.
#' @param method the function to register as a method (optional).
#' @return \code{box::register_S3_method} is called for its side effect.
#'
#' @details If \code{method} is missing, it defaults to a function named
#' \code{name.class} in the calling module. If no such function exists, an error
#' is raised.
#'
#' Methods for generics defined in the same module do not need to be registered
#' explicitly, and indeed \emph{should not} be registered. However, if the user
#' wants to add a method for a known generic (defined outside the module, e.g.
#' \code{\link{print}}), then this needs to be made known explicitly.
#'
#' See the vignette at \code{vignette('box', 'box')} for more information about
#' defining S3 methods inside modules.
#'
#' @note \strong{Do not} call \code{\link[base]{registerS3method}} inside a
#' module, only use \code{box::register_S3_method}. This is important for the
#' module’s own book-keeping.
#' @export
register_S3_method = function (name, class, method) {
    if (missing(method)) {
        module = current_mod()
        method = rethrow_on_error(
            get(paste0(name, '.', class), envir = module, mode = 'function', inherits = FALSE)
        )
    } else {
        module = environment(method)
    }
    attr(module, 'S3') = c(attr(module, 'S3'), paste(name, class, sep = '.'))
    registerS3method(name, class, method, module)
}

#' Internal S3 infrastructure helpers
#'
#' The following are internal S3 infrastructure helper functions.
#'
#' \code{is_S3_user_generic} checks whether a function given by name is a
#' user-defined generic. A user-defined generic is any function which, at some
#' point, calls \code{UseMethod}.
#'
#' \code{make_S3_methods_known} finds and registers S3 methods inside a module.
#' @param function_name function name as character string.
#' @param envir the environment this function is invoked from.
#' @return \code{is_S3_user_generic} returns \code{TRUE} if the specified
#' function is a user-defined S3 generic, \code{FALSE} otherwise.
#' @keywords internal
#' @name s3
is_S3_user_generic = function (function_name, envir = parent.frame()) {
    ! bindingIsActive(function_name, envir) &&
        is_S3(body(get(function_name, envir = envir, mode = 'function')))
}

is_S3 = function (expr) {
    if (length(expr) == 0L) {
        FALSE
    } else if (is.function(expr)) {
        FALSE
    } else if (is.call(expr)) {
        fun = expr[[1L]]
        if (is.name(fun)) {
            # NB: this is relying purely on static analysis. We do not test
            # whether these calls actually refer to the expected base R
            # functions since that would require evaluating the function body in
            # the general case (namely, the function body itself could redefine
            # them).
            if (fun %==% quote(UseMethod)) return(TRUE)

            # Make sure nested function definitions are *not* getting
            # traversed: `UseMethod` inside a nested function does not make
            # the containing function a generic.
            if (fun %==% quote(`function`)) return(FALSE)

            # Without `as.list`, missing arguments in call expressions cause
            # missing values in our code. Rather than handle these as special
            # cases, we use `as.list` to flatten those into empty expressions.
            Recall(as.list(expr)[-1L])
        } else {
            Recall(fun) || Recall(as.list(expr)[-1L])
        }
    } else if (is.recursive(expr)) {
        Recall(expr[[1L]]) || Recall(expr[-1L])
    } else {
        FALSE
    }
}

#' @param module the module object for which to register S3 methods
#' @rdname s3
make_S3_methods_known = function (module) {
    # S3 methods are found by first finding all generics, and then searching
    # function names with the generic’s name as a prefix. This may however
    # fail for cases where a found name actually corresponds to a method for a
    # different, “known” generic with a dot in its name (an example of this is
    # `se.contrast`). We therefore test whether a method was already
    # registered manually via `register_S3_method` before registering it
    # automatically here. To avoid such ambiguities it is highly recommended
    # not to use dots in function and class names. Use underscores instead.

    functions = lsf(module)
    generics = Filter(function (f) is_S3_user_generic(f, module), functions)

    find_methods = function (generic) grep(fmt('^{generic}\\.[^.]'), functions, value = TRUE)

    methods = map(find_methods, generics)

    register_method = function (name, generic) {
        # Ensure we don’t register functions which have already been
        # registered explicitly. This guards against ambiguous cases.
        if (name %in% attr(module, 'S3')) return()

        # + 1 for dot, + 1 for position after that.
        class = substr(name, nchar(generic) + 2L, nchar(name))
        registerS3method(generic, class, module[[name]], module)
    }

    register_methods = function (methods, generic_name)
        map(register_method, methods, list(generic_name))

    map(register_methods, methods, generics)
    invisible()
}

#' Return a list of function names in an environment
#'
#' @param envir the environment to search in.
#' @return \code{lsf} returns a vector of function names in the given environment.
#' @keywords internal
lsf = function (envir) {
    # We cannot use `eapply` here since that will try to evaluate active
    # bindings starting with R 4.1. ‘box’ uses active bindings to for delayed
    # cyclic loading, and these should not be evaluated at this point.
    is_function = function (name) {
        ! bindingIsActive(name, envir) &&
            ! is.null(get0(name, envir, inherits = FALSE, mode = 'function'))
    }
    Filter(is_function, names(envir))
}

context('S3')

test_that('S3 generics are recognized', {
    foo = function (x) UseMethod('foo')
    bar = function (x) print('UseMethod')
    baz = function (x) {
        x = 42
        UseMethod('baz')
    }
    qux = function (x) {
        UseMethod('print')
        a = 12
    }
    quz = function (x)
        foo(bar(sum(1, UseMethod('quz'))))

    expect_true(is_S3_user_generic('foo'))
    expect_false(is_S3_user_generic('bar'))
    expect_true(
        is_S3_user_generic('baz'),
        'Multi-statement method not recognized'
    )
    expect_true(
        is_S3_user_generic('qux'),
        'Method cannot dispatch to generic of different name'
    )
    expect_true(
        is_S3_user_generic('quz'),
        '`UseMethod` can be nested in other calls'
    )
})

test_that('S3 methods are found', {
    box::use(mod/s3)
    s3ns = environment(s3$test)
    test = getS3method('test', 'character', envir = s3)
    test_dot_character = s3ns$test.character
    expect_identical(test, test_dot_character)

    # NOT executed locally!
    print = getS3method('print', 'test')
    print_dot_test = s3ns$print.test
    expect_identical(print, print_dot_test)
})

test_that('S3 methods can be called without attaching', {
    box::use(mod/s3)
    expect_equal(s3$test(1), 'test.default')
    expect_equal(s3$test('a'), 'test.character')

    foo = structure(42, class = 'test')
    expect_equal(print(foo), 's3$print.test')
})

test_that('`box::register_S3_method()` correctly infers method', {
    box::use(mod/s3)
    foo = structure(42, class = 'test2')
    expect_equal(print(foo), 's3$print.test2')
})

test_that('S3 methods are not registered twice', {
    box::use(mod/s3)

    result = s3$se(structure(1, class = 'contrast.test'))
    expect_equal(
        result, 's3$se.default',
        'Generic does not call `se.contrast.test`'
    )

    result = se.contrast(structure(1, class = 'test'))
    expect_equal(
        result, 's3$se.contrast.test',
        'Known generics are still callable'
    )
})

test_that('Forwarded S3 genetics without methods work', {
    box::use(mod/s3_b)
    expect_equal(s3_b$test(1), 'test.default')
    expect_equal(s3_b$test('a'), 'test.character')
})

test_that('`is_S3_user_generic` can deal with substituted functions', {
    expect_error(box::use(mod/issue125), NA)
})

test_that('nested functions are parsed correctly', {
    expect_error(box::use(mod/issue203), NA)
    expect_false(box:::is_S3_user_generic('g', issue203))
    expect_false(box:::is_S3_user_generic('h', issue203))
})

test_that('functions with missing arguments are parsed correctly', {
    expect_error(is_S3(quote(tag$span('foo', ))), NA)
    expect_error(is_S3(quote(base$quote(expr =))), NA)
    expect_error(is_S3(quote((quote)(expr =))), NA)
})

#' Information about a physical module or package
#'
#' A \code{mod_info} represents an existing, installed module and its runtime
#' physical location (usually in the file system).
#' @param spec a \code{mod_spec} or \code{pkg_spec} (for \code{mod_info} and
#' \code{pkg_info}, respectively)
#' @param source_path character string full path to the physical module
#' location.
#' @return \code{mod_info} and \code{pkg_info} return a structure representing
#' the module/package information for the given specification/source location.
#' @keywords internal
#' @name info
mod_info = function (spec, source_path) {
    structure(
        list(name = spec$name, source_path = source_path),
        class = c('box$mod_info', 'box$info')
    )
}

#' A \code{pkg_info} represents an existing, installed package.
#' @keywords internal
#' @name info
pkg_info = function (spec) {
    structure(list(name = spec$name), class = c('box$pkg_info', 'box$info'))
}

#' @export
`print.box$info` = function (x, ...) {
    cat(as.character(x, ...), '\n', sep = '')
    invisible(x)
}

#' @export
`as.character.box$mod_info` = function (x, ...) {
    fmt('<mod_info: \x1B[33m{x$name}\x1B[0m at \x1B[33m{x$source_path}\x1B[0m>')
}

#' @export
`as.character.box$pkg_info` = function (x, ...) {
    path = getNamespaceInfo(x$name, 'path')
    fmt('<mod_info: \x1B[33m{path}\x1B[0m>')
}

is_absolute = function (spec) {
    spec$prefix[1L] %in% c('.', '..')
}

find_mod = function (spec, caller) {
    UseMethod('find_mod')
}

`find_mod.box$mod_spec` = function (spec, caller) {
    if (is_absolute(spec)) find_local_mod(spec, caller) else find_global_mod(spec, caller)
}

`find_mod.box$pkg_spec` = function (spec, caller) {
    pkg_info(spec)
}

find_local_mod = function (spec, caller) {
    find_in_path(spec, calling_mod_path(caller))
}

find_global_mod = function (spec, caller) {
    # In the future, this may be augmented by pluggable ways of loading modules.
    find_in_path(spec, mod_search_path(caller))
}

#' Find a module’s source location
#'
#' @param spec a \code{mod_spec}.
#' @param base_paths a character vector of paths to search the module in, in
#' order of preference.
#' @return \code{find_in_path} returns a \code{mod_info} that specifies the
#' module source location.
#' @details
#' A module is physically represented in the file system either by
#' \file{‹spec_name(spec)›.r} or by \file{‹spec_name(spec)›/__init__.r}, in that
#' order of preference in case both exist. File extensions are case insensitive
#' to allow for R’s obsession with capital-R extensions (but lower-case are
#' given preference, and upper-case file extensions are discouraged).
#' @keywords internal
find_in_path = function (spec, base_paths) {
    candidates = mod_file_candidates(spec, base_paths)
    hits = map(file.exists, candidates)
    which_base = which(map_lgl(any, hits))[1L]

    if (is.na(which_base)) {
        throw(
            'unable to load module {spec_name(spec);"}; ',
            'not found in {base_paths;"}'
        )
    }

    path = candidates[[which_base]][hits[[which_base]]][1L]
    base_path = base_paths[which_base]
    mod_info(spec, normalizePath(path))
}

mod_file_candidates = function (spec, base_paths) {
    mod_path_prefix = merge_path(spec$prefix)
    ext = c('.r', '.R')
    simple_mod = file.path(mod_path_prefix, paste0(spec$name, ext))
    nested_mod = file.path(mod_path_prefix, spec$name, paste0('__init__', ext))
    map(file.path, base_paths, list(c(simple_mod, nested_mod)))
}

#' Import a module or package
#'
#' \code{box::use} imports one or more modules and/or packages, and makes them
#' available in the calling environment.
#'
#' @usage \special{box::use(prefix/mod, \dots)}
#' @usage \special{box::use(pkg, \dots)}
#' @usage \special{box::use(alias = prefix/mod, \dots)}
#' @usage \special{box::use(alias = pkg, \dots)}
#' @usage \special{box::use(prefix/mod[attach_list], \dots)}
#' @usage \special{box::use(pkg[attach_list], \dots)}
#'
#' @param prefix/mod a qualified module name
#' @param pkg a package name
#' @param alias an alias name
#' @param attach_list a list of names to attached, optionally with aliases of
#' the form \code{alias = name}; or the special placeholder name \code{\dots}
#' @param \dots further import declarations
#' @return \code{box::use} has no return value. It is called for its
#' side effect.
#'
#' @details
#' \code{box::use(\dots)} specifies a list of one or more import declarations,
#' given as individual arguments to \code{box::use}, separated by comma.
#' \code{box::use} permits using a trailing comma after the last import
#' declaration. Each import declaration takes one of the following forms:
#'
#' \describe{
#' \item{\code{\var{prefix}/\var{mod}}:}{
#'      Import a module given the qualified module name
#'      \code{\var{prefix}/\var{mod}} and make it available locally using the
#'      name \code{\var{mod}}. The \code{\var{prefix}} itself can be a nested
#'      name to allow importing specific submodules. \emph{Local imports} can be
#'      specified via the prefixes starting with \code{.} and \code{..}, to
#'      override the search path and use the local path instead. See the
#'      \sQuote{Search path} below for details.
#' }
#' \item{\code{\var{pkg}}:}{
#'      Import a package \code{\var{pkg}} and make it available locally using its own
#'      package name.
#' }
#' \item{\code{\var{alias} = \var{prefix}/\var{mod}} or \code{\var{alias} = \var{pkg}}:}{
#'      Import a module or package, and make it available locally using the name
#'      \code{\var{alias}} instead of its regular module or package name.
#' }
#' \item{\code{\var{prefix}/\var{mod}[\var{attach_list}]} or \code{\var{pkg}[\var{attach_list}]}:}{
#'      Import a module or package and attach the exported symbols listed in
#'      \code{\var{attach_list}} locally. This declaration does \emph{not} make
#'      the module/package itself available locally. To override this, provide
#'      an alias, that is, use \code{\var{alias} =
#'      \var{prefix}/\var{mod}[\var{attach_list}]} or \code{\var{alias} =
#'      \var{pkg}[\var{attach_list}]}.
#'
#'      The \code{\var{attach_list}} is a comma-separated list of names,
#'      optionally with aliases assigned via \code{alias = name}. The list can
#'      also contain the special symbol \code{\dots}, which causes \emph{all}
#'      exported names of the module/package to be imported.
#' }
#' }
#'
#' See the vignette at \code{vignette('box', 'box')} for detailed examples of
#' the different types of use declarations listed above.
#'
#' @section Import semantics:
#' Modules and packages are loaded into dedicated namespace environments. Names
#' from a module or package can be selectively attached to the current scope as
#' shown above.
#'
#' Unlike with \code{\link[base]{library}}, attaching happens \emph{locally},
#' i.e. in the caller’s environment: if \code{box::use} is executed in the
#' global environment, the effect is the same. Otherwise, the effect of
#' importing and attaching a module or package is limited to the caller’s local
#' scope (its \code{environment()}). When used \emph{inside a module} at module
#' scope, the newly imported module is only available inside the module’s scope,
#' not outside it (nor in other modules which might be loaded).
#'
#' Member access of (non-attached) exported names of modules and packages
#' happens via the \code{$} operator. This operator does not perform partial
#' argument matching, in contrast with the behavior of the \code{$} operator in
#' base \R, which matches partial names.
#'
#' \strong{Note} that replacement functions (i.e. functions of the form
#' \code{fun<-}) must be \emph{attached} to be usable, because \R syntactically
#' does not allow assignment calls where the left-hand side of the assignment
#' contains \code{$}.
#'
#' @section Export specification:
#'
#' Names defined in modules can be marked as \emph{exported} by prefixing them
#' with an \code{@export} tag comment; that is, the name needs to be immediately
#' prefixed by a comment that reads, verbatim, \code{#' @export}. That line may
#' optionally be part of a \pkg{roxygen2} documentation for that name.
#'
#' Alternatively, exports may be specified via the
#' \code{\link[=export]{box::export}} function, but using declarative
#' \code{@export} tags is generally preferred.
#'
#' A module which has not declared any exports is treated as a \emph{legacy
#' module} and exports \emph{all} default-visible names (that is, all names that
#' do not start with a dot (\code{.})). This usage is present only for backwards
#' compatibility with plain \R scripts, and its usage is \emph{not recommended}
#' when writing new modules.
#'
#' To define a module that exports no names, call \code{box::export()} without
#' arguments. This prevents the module from being treated as a legacy module.
#'
#' @section Search path:
#' Modules are searched in the module search path, given by
#' \code{getOption('box.path')}. This is a character vector of paths to search,
#' from the highest to the lowest priority. The current directory is always
#' considered last. That is, if a file \file{a/b.r} exists both locally in the
#' current directory and in a module search path, the local file \file{./a/b.r}
#' will \emph{not} be loaded, unless the import is explicitly declared as
#' \code{box::use(./a/b)}.
#'
#' Modules in the module search path \emph{must be organised in subfolders}, and
#' must be imported fully qualified. Keep in mind that \code{box::use(name)}
#' will \emph{never} attempt to load a module; it always attempts to load a
#' package. A common module organisation is by project, company or user name;
#' for instance, fully qualified module names could mirror repository names on
#' source code sharing websites (such as GitHub).
#'
#' Given a declaration \code{box::use(a/b)} and a search path \file{\var{p}}, if
#' the file \file{\var{p}/a/b.r} does not exist, \pkg{box} alternatively looks
#' for a nested file \file{\var{p}/a/b/__init__r} to load. Module path names are
#' \emph{case sensitive} (even on case insensitive file systems), but the file
#' extension can be spelled as either \file{.r} or \file{.R} (if both exist,
#' \code{.r} is given preference).
#'
#' The module search path can be overridden by the environment variable
#' \env{R_BOX_PATH}. If set, it may consist of one or more search paths,
#' separated by the platform’s path separator (i.e. \code{;} on Windows, and
#' \code{:} on most other platforms).
#'
#' \strong{Deprecation warning:} in the next major version, \pkg{box} will read
#' environment variables only \emph{once}, at package load time. Modifying the
#' value of \env{R_BOX_PATH} afterwards will have no effect, unless the package
#' is unloaded and reloaded.
#'
#' The \emph{current directory} is context-dependent: inside a module, the
#' directory corresponds to the module’s directory. Inside an \R code file
#' invoked from the command line, it corresponds to the directory containing
#' that file. If the code is running inside a \pkg{Shiny} application or a
#' \pkg{knitr} document, the directory of the execution is used. Otherwise (e.g.
#' in an interactive \R session), the current working directory as given by
#' \code{getwd()} is used.
#'
#' Local import declarations (that is, module prefixes that start with \code{./}
#' or \code{../}) never use the search path to find the module. Instead,
#' only the current module’s directory (for \code{./}) or the parent module’s
#' directory (for \code{../}) is looked at. \code{../} can be nested:
#' \code{../../} denotes the grandparent module, etc.
#'
#' @section S3 support:
#' Modules can contain S3 generics and methods. To override known generics
#' (= those defined outside the module), methods inside a module need to be
#' registered using \code{\link[=register_S3_method]{box::register_S3_method}}.
#' See the documentation there for details.
#'
#' @section Module names:
#' A module’s full name consists of one or more \R names separated by \code{/}.
#' Since \code{box::use} declarations contain \R expressions, the names need to
#' be valid \R names. Non-syntactic names need to be wrapped in backticks; see
#' \link[base]{Quotes}.
#'
#' Furthermore, since module names usually correspond to file or folder names,
#' they should consist only of valid path name characters to ensure portability.
#'
#' @section Encoding:
#' All module source code files are assumed to be UTF-8 encoded.
#'
#' @examples
#' # Set the module search path for the example module.
#' old_opts = options(box.path = system.file(package = 'box'))
#' old_env = Sys.getenv('R_BOX_PATH', NA)
#' Sys.unsetenv('R_BOX_PATH')
#'
#' # Basic usage
#' # The file `mod/hello_world.r` exports the functions `hello` and `bye`.
#' box::use(mod/hello_world)
#' hello_world$hello('Robert')
#' hello_world$bye('Robert')
#'
#' # Using an alias
#' box::use(world = mod/hello_world)
#' world$hello('John')
#'
#' # Attaching exported names
#' box::use(mod/hello_world[hello])
#' hello('Jenny')
#' # Exported but not attached, thus access fails:
#' try(bye('Jenny'))
#'
#' # Attach everything, give `hello` an alias:
#' box::use(mod/hello_world[hi = hello, ...])
#' hi('Eve')
#' bye('Eve')
#'
#' # Reset the module search path
#' on.exit({
#'   options(old_opts)
#'   if (! is.na(old_env)) Sys.setenv(R_BOX_PATH = old_env)
#' })
#'
#' \dontrun{
#' # The following code illustrates different import declaration syntaxes
#' # inside a single `box::use` declaration:
#'
#' box::use(
#'     global/mod,
#'     mod2 = ./local/mod,
#'     purrr,
#'     tbl = tibble,
#'     dplyr = dplyr[filter, select],
#'     stats[st_filter = filter, ...],
#' )
#'
#' # This declaration makes the following names available in the caller’s scope:
#' #
#' # 1. `mod`, which refers to the module environment for  `global/mod`
#' # 2. `mod2`, which refers to the module environment for `./local/mod`
#' # 3. `purrr`, which refers to the package environment for ‘purrr’
#' # 4. `tbl`, which refers to the package environment for ‘tibble’
#' # 5. `dplyr`, which refers to the package environment for ‘dplyr’
#' # 6. `filter` and `select`, which refer to the names exported by ‘dplyr’
#' # 7. `st_filter`, which refers to `stats::filter`
#' # 8. all other exported names from the ‘stats’ package
#' }
#' @seealso
#' \code{\link[=name]{box::name}} and \code{\link[=file]{box::file}} give
#' information about loaded modules.
#' \code{\link[=help]{box::help}} displays help for a module’s exported names.
#' \code{\link[=unload]{box::unload}} and \code{\link[=reload]{box::reload}} aid
#' during module development by performing dynamic unloading and reloading of
#' modules in a running \R session.
#' \code{\link[=export]{box::export}} can be used as an alternative to
#' \code{@export} comments inside a module to declare module exports.
#' @export
use = function (...) {
    caller = parent.frame()
    call = match.call()
    imports = call[-1L]
    aliases = names(imports) %||% character(length(imports))
    map(use_one, imports, aliases, list(caller), use_call = list(sys.call()))
    invisible()
}

#' Import a module or package
#'
#' Actual implementation of the import process
#'
#' @details
#' \code{use_one} performs the actual import. It is invoked by \code{use} given
#' the calling context and unevaluated expressions as arguments, and only uses
#' standard evaluation.
#'
#' \code{load_and_register} performs the loading, attaching and exporting of a
#' module identified by its spec and info.
#'
#' \code{register_as_import} registers a \code{use} declaration in the calling
#' module so that it can be found later on, if the declaration is reexported by
#' the calling module.
#'
#' \code{defer_import_finalization} is called by \code{load_and_register} to
#' earmark a module for deferred initialization if it hasn’t been fully loaded
#' yet.
#'
#' \code{finalize_deferred} exports and attaches names from a module use
#' declaration which has been deferred due to being part of a cyclic loading
#' chain.
#'
#' \code{export_and_attach} exports and attaches names from a given module use
#' declaration.
#'
#' \code{load_from_source} loads a module source file into its newly created,
#' empty module namespace.
#'
#' \code{load_mod} tests whether a module or package was already loaded and, if
#' not, loads it.
#'
#' \code{mod_exports} returns an export environment containing a copy of the
#' module’s exported objects.
#'
#' \code{attach_to_caller} attaches the listed names of an attach specification
#' for a given use declaration to the calling environment.
#'
#' \code{assign_alias} creates a module/package object in calling environment,
#' unless it contains an attach declaration, and no explicit alias is given.
#'
#' \code{assign_temp_alias} creates a placeholder object for the module in the
#' calling environment, to be replaced by the actual module export environment
#' once the module is completely loaded (which happens in the case of cyclic
#' imports).
#' @param declaration an unevaluated use declaration expression without the
#' surrounding \code{use} call
#' @param alias the use alias, if given, otherwise \code{NULL}
#' @param caller the client’s calling environment (parent frame)
#' @param use_call the \code{use} call which is invoking this code
#' @return \code{use_one} does not currently return a value. — This might change
#' in the future.
#' @note If a module is still being loaded (because it is part of a cyclic
#' import chain), \code{load_and_register} earmarks the module for deferred
#' registration and holds off on attaching and exporting for now, since not all
#' its names are available yet.
#' @keywords internal
#' @name importing
use_one = function (declaration, alias, caller, use_call) {
    # Permit empty expression resulting from trailing comma.
    if (declaration %==% quote(expr =) && alias %==% '') return()

    rethrow_on_error({
        spec = parse_spec(declaration, alias)
        info = find_mod(spec, caller)
        load_and_register(spec, info, caller)
    }, call = use_call)
}

#' @param spec a module use declaration specification
#' @param info the physical module information
#' @rdname importing
load_and_register = function (spec, info, caller) {
    ret = load_mod(info)
    mod_ns = ret$mod_ns
    is_apex = ret$is_apex
    on.exit(lock_all_environments(is_apex))
    register_as_import(spec, info, mod_ns, caller)

    if (is_mod_still_loading(info)) {
        # Module was cached but hasn’t fully loaded yet. This happens for
        # cyclic imports (1. A -> 2. B -> 3. A) in step (3). To proceed, we
        # take note of the issue and wait until we bounce back to step (1) to
        # perform deferred finalization.

        assign_temp_alias(spec, caller)
        defer_import_finalization(spec, info, mod_ns, caller)
        return()
    }

    export_and_attach(spec, info, mod_ns, caller)
}

#' @param mod_ns the module namespace environment of the newly loaded module
#' @rdname importing
register_as_import = function (spec, info, mod_ns, caller) {
    if (is_namespace(caller)) {
        # Import declarations are stored in a list in the module metadata
        # dictionary. They are looked up by their spec.
        import = import_decl(ns = mod_ns, spec = spec, info = info)
        existing_imports = namespace_info(caller, 'imports', list())
        namespace_info(caller, 'imports') = c(existing_imports, list(import))
    }
}

#' @rdname importing
defer_import_finalization = function (spec, info, mod_ns, caller) {
    defer_args = as.list(environment())
    existing_deferred = attr(loaded_mods[[info$source_path]], 'deferred')
    attr(loaded_mods[[info$source_path]], 'deferred') =
        c(existing_deferred, list(defer_args))
}

#' @rdname importing
finalize_deferred = function (info) {
    UseMethod('finalize_deferred')
}

`finalize_deferred.box$mod_info` = function (info) {
    deferred = attr(loaded_mods[[info$source_path]], 'deferred')
    if (is.null(deferred)) return()

    attr(loaded_mods[[info$source_path]], 'deferred') = NULL

    for (defer_args in deferred) {
        do.call(export_and_attach, defer_args)
    }
}

`finalize_deferred.box$pkg_info` = function (info) {}

#' @rdname importing
export_and_attach = function (spec, info, mod_ns, caller) {
    finalize_deferred(info)

    mod_exports = mod_exports(info, spec, mod_ns)
    defer_locking(mod_exports)

    assign_alias(spec, mod_exports, caller)
    attach_to_caller(spec, info, mod_exports, mod_ns, caller)
}

#' @rdname importing
load_from_source = function (info, mod_ns) {
    # R, Windows and Unicode don’t play together. `source` does not work here.
    # See http://developer.r-project.org/Encodings_and_R.html and
    # http://stackoverflow.com/q/5031630/1968 for a discussion of this.
    exprs = parse(info$source_path, keep.source = TRUE, encoding = 'UTF-8')
    eval(exprs, mod_ns)

    if (is.null(namespace_info(mod_ns, 'exports'))) {
        exports = parse_export_specs(info, exprs, mod_ns)
        is_legacy = length(exports) == 0L
        namespace_info(mod_ns, 'legacy') = is_legacy
        namespace_info(mod_ns, 'exports') = if (is_legacy) ls(mod_ns) else exports
    }

    make_S3_methods_known(mod_ns)
}

#' @return \code{load_mod} returns a named \code{list(mod_ns, is_apex)}
#' containing the module or package namespace environment of the specified
#' module or package info, as well as a flag specifying whether the loaded
#' module is an “apex” module, i.e. not loaded from within another module. This
#' information is then used to lock all environments that were created during
#' the loading. — This must happen after nested modules are fully loaded, since
#' deferred finalization otherwise cannot modify these environments during
#' cyclic imports.
#' @rdname importing
load_mod = function (info) {
    UseMethod('load_mod')
}

`load_mod.box$mod_info` = function (info) {
    if (is_mod_loaded(info)) {
        return(list(mod_ns = loaded_mod(info), is_apex = is_at_apex()))
    }

    # Load module/package and dependencies; register the module now, to allow
    # cyclic imports without recursing indefinitely — but deregister upon
    # failure to load.

    on.exit({
        deregister_mod(info)
        lock_all_environments(is_apex)
    })

    mod_ns = make_namespace(info)
    is_apex = defer_locking(mod_ns)
    register_mod(info, mod_ns)
    load_from_source(info, mod_ns)
    mod_loading_finished(info, mod_ns)

    # Call `.on_load` hook just after loading is finished but before exporting
    # symbols, so that `.on_load` can modify these symbols.
    call_hook(mod_ns, '.on_load', mod_ns)

    on.exit()
    list(mod_ns = mod_ns, is_apex = is_apex)
}

`load_mod.box$pkg_info` = function (info) {
    pkg = info$name
    list(mod_ns = base::.getNamespace(pkg) %||% loadNamespace(pkg), is_apex = is_at_apex())
}

#' @return \code{mod_exports} returns an export environment containing the
#' exported names of a given module.
#' @rdname importing
mod_exports = function (info, spec, mod_ns) {
    exports = mod_export_names(info, mod_ns)
    env = make_export_env(info, spec, mod_ns)
    import_into_env(env, exports, mod_ns, exports)
    env
}

#' @return \code{mode_export_names} returns a vector containing the same names as
#' \code{names(mod_exports(info, spec, mod_ns))} but does not create an export
#' environment.
#' @rdname importing
mod_export_names = function (info, mod_ns) {
    UseMethod('mod_export_names')
}

`mod_export_names.box$mod_info` = function (info, mod_ns) {
    namespace_info(mod_ns, 'exports')
}

`mod_export_names.box$pkg_info` = function (info, mod_ns) {
    c(getNamespaceExports(mod_ns), ls(getNamespaceInfo(mod_ns, 'lazydata')))
}

#' @rdname importing
attach_to_caller = function (spec, info, mod_exports, mod_ns, caller) {
    attach_list = attach_list(spec, names(mod_exports))
    if (is.null(attach_list)) return()

    import_env = find_import_env(caller, spec, info, mod_ns)
    attr(mod_exports, 'attached') = environmentName(import_env)
    import_into_env(import_env, names(attach_list), mod_exports, attach_list)
}

#' @return \code{attach_list} returns a named character vector of the names in
#' an attach specification. The vector’s names are the aliases, if provided, or
#' the attach specification names themselves otherwise.
#' @rdname importing
attach_list = function (spec, exports) {
    if (is.null(spec$attach)) return()
    is_wildcard = is.na(spec$attach)
    name_spec = spec$attach[! is_wildcard]

    if (! all(is_wildcard) && any((what = ! name_spec %in% exports))) {
        missing = name_spec[what]
        throw(
            'name{s} {missing;"} not exported by {spec_name(spec);"}',
            s = if (length(missing) > 1L) 's' else ''
        )
    }

    if (any(is_wildcard)) {
        if (length(exports) == 0L) return()

        aliases = exports
        if (length(spec$attach) > 1L) {
            # Substitute aliased names in export list
            to_replace = match(name_spec, exports)
            aliases[to_replace] = names(name_spec)
        }

        stats::setNames(exports, aliases)
    } else {
        name_spec
    }
}

#' @rdname importing
assign_alias = function (spec, mod_exports, caller) {
    create_mod_alias = is.null(spec$attach) || spec$explicit
    if (! create_mod_alias) return()

    if (exists(spec$alias, caller, inherits = FALSE) && bindingIsLocked(spec$alias, caller)) {
        box_unlock_binding(spec$alias, caller)
    }
    assign(spec$alias, mod_exports, envir = caller)
}

#' @rdname importing
assign_temp_alias = function (spec, caller) {
    create_mod_alias = is.null(spec$attach) || spec$explicit
    if (! create_mod_alias) return()

    self = environment()
    callers = list()

    binding = function (mod_exports) {
        if (missing(mod_exports)) {
            # Find from where I’m called, and infer the target of the export.
            mod_exports_frame_index = utils::tail(which(map_lgl(
                function (call) call[[1L]] %==% quote(mod_exports),
                sys.calls()
            )), 1L)
            frame = sys.frame(mod_exports_frame_index)
            self$callers = c(callers, frame$env)

            # FIXME: Do we need to create transitive placeholder active bindings?
            structure(list(), class = 'box$placeholder')
        } else {
            # Resolve assignments
            for (env in callers) {
                assign(spec$alias, mod_exports, envir = env)
                defer_locking(env)
            }

            # Replace myself
            rm(list = spec$alias, envir = caller)
            assign(spec$alias, mod_exports, envir = caller)
        }
    }

    makeActiveBinding(spec$alias, structure(binding, class = 'box$placeholder'), caller)
}

lock_info = new.env(parent = emptyenv())
lock_info$envs = list()

defer_locking = function (ns) {
    is_apex = is_at_apex()
    lock_info$envs = c(lock_info$envs, list(ns))
    is_apex
}

is_at_apex = function () {
    length(lock_info$envs) == 0L
}

lock_all_environments = function (is_apex) {
    if (! is_apex) {
        return()
    }

    for (ns in lock_info$envs) {
        lockEnvironment(ns, bindings = TRUE)
    }

    lock_info$envs = list()
}

context('circular dependencies')

test_that('cyclic dependencies load in finite time', {
    box::use(mod/cyclic_a)
    ns = attr(cyclic_a, 'namespace')
    expect_true(environmentIsLocked(cyclic_a))
    expect_true(bindingIsLocked('name', cyclic_a))
    expect_true(environmentIsLocked(ns))
    expect_true(bindingIsLocked('name', ns))
})

test_that('cyclic import fully loads dependencies', {
    box::use(a = mod/cyclic_a)
    box::use(b = mod/cyclic_b)
    ns_a = attr(a, 'namespace')
    ns_b = attr(b, 'namespace')

    expect_equal(a$name, 'a')
    expect_equal(b$name, 'b')
    expect_equal(a$b_name(), 'b')
    expect_equal(b$a_name(), 'a')
    expect_equal(a$b$name, 'b')
    expect_equal(b$a$name, 'a')
    expect_equal(a$b$a$b_name(), 'b')
    expect_equal(a$b$a$name, 'a')
    expect_equal(b$a$b$a_name(), 'a')
    expect_equal(b$a$b$name, 'b')

    expect_true(environmentIsLocked(a))
    expect_true(bindingIsLocked('name', a))
    expect_true(environmentIsLocked(b))
    expect_true(bindingIsLocked('name', b))
    expect_true(environmentIsLocked(ns_a))
    expect_true(bindingIsLocked('name', ns_a))
    expect_true(environmentIsLocked(ns_b))
    expect_true(bindingIsLocked('name', ns_b))
})

box::use(testthat[...])

.on_load = function (ns) {
    run = 0L
    core_pkgs = c('methods', 'stats', 'graphics', 'grDevices', 'utils')

    # Find an exported name from each of these core packages to use for testing.
    names = vapply(core_pkgs, function (p) getNamespaceExports(p)[1L], character(1L))

    find_in_parents = function (name) {
        found = c()
        env = parent.env(parent.frame())
        while (! identical(env, emptyenv())) {
            if (name %in% names(env)) found = c(found, environmentName(env))
            env = parent.env(env)
        }
        found
    }

    name_found = function (name, pkg) {
        where = paste0('package:', pkg)
        where %in% eval.parent(bquote(find_in_parents(.(name))))
    }

    # Ensure that names from default packages are not found prior to import.

    for (i in seq_along(core_pkgs)) {
        info = sprintf('%s::%s is attached', core_pkgs[i], names[i])
        expect_false(name_found(names[i], core_pkgs[i]), info = info)
        run = run + 1L
    }

    # Now attach R default packages and ensure their names can be found.

    box::use(r/core[...])

    imports = ls(parent.env(environment()), all.names = TRUE)

    for (name in names) {
        label = sprintf("'%s' %%in%% imports", name)
        expect_true(name %in% imports, label = label)
        run = run + 1L
    }

    ns$tests_run = run
}

#' @export
tests_run = 0L

context('submodules')

test_that('submodules can be loaded one by one', {
    on.exit(clear_mods())

    result = capture.output(box::use(mod/nested/a/b))
    expect_equal(result, 'a/b/__init__.r')

    result = capture.output(box::use(mod/nested/a/b/c))
    expect_equal(result, 'a/b/c/__init__.r')

    result = capture.output(box::use(mod/nested/a/b/d))
    expect_equal(result, 'a/b/d/__init__.r')
})

test_that('module can export nested submodules', {
    box::use(mod/b)
    expect_equal(b$answer, 42L)
})

context('spec parser')

test_use = function (...) {
    call = match.call()
    parse_spec(call[[2L]], names(call)[[2L]] %||% '')
}

is_mod_spec = function (x) {
    inherits(x, 'box$mod_spec')
}

is_pkg_spec = function (x) {
    inherits(x, 'box$pkg_spec')
}

test_that('modules without attaching can be parsed', {
    m = test_use(foo/bar)
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'bar')
    expect_false(m$explicit)
    expect_null(m$attach)
})

test_that('parse errors are correctly handled', {
    expect_box_error(
        test_use(foo/bar[]),
        '^expected at least one name in attach list'
    )
})

test_that('fully qualified names can be nested', {
    m = test_use(foo/bar/baz)
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'baz')
    expect_equal(m$prefix, c('foo', 'bar'))
    expect_equal(m$alias, 'baz')
    expect_false(m$explicit)
    expect_null(m$attach)
})

test_that('imports can be relative', {
    m = test_use(./foo)
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$prefix, '.')
    expect_null(m$attach)

    m = test_use(./foo[bar])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$prefix, '.')
    expect_equal(m$attach, c(bar = 'bar'))

    m = test_use(..)
    expect_true(is_mod_spec(m))
    expect_equal(m$name, '.')
    expect_equal(m$prefix, '..')
    expect_null(m$attach)

    m = test_use(..[foo])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, '.')
    expect_equal(m$prefix, '..')
    expect_equal(m$attach, c(foo = 'foo'))

    m = test_use(../foo)
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$prefix, '..')
    expect_null(m$attach)

    m = test_use(../foo[bar])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$prefix, '..')
    expect_equal(m$attach, c(bar = 'bar'))
})

test_that('`./` can only be used as a prefix', {
    expect_box_error(test_use(a/./b), 'can only be used as a prefix')
    expect_box_error(test_use(././b), 'can only be used as a prefix')
    expect_box_error(test_use(a/b/.), 'can only be used as a prefix')
    expect_box_error(test_use(a/./b[foo]), 'can only be used as a prefix')
    expect_box_error(test_use(././b[foo]), 'can only be used as a prefix')
    expect_box_error(test_use(a/b/.[foo]), 'can only be used as a prefix')

    expect_error(test_use(../../b), NA)
    expect_box_error(test_use(.././b), 'can only be used as a prefix')
    expect_box_error(test_use(../.), 'can only be used as a prefix')
    expect_error(test_use(../../b[foo]), NA)
    expect_box_error(test_use(.././b[foo]), 'can only be used as a prefix')
    expect_box_error(test_use(../.[foo]), 'can only be used as a prefix')

    expect_box_error(test_use(a/../b), 'can only be used as a prefix')
    expect_box_error(test_use(a/b/..), 'can only be used as a prefix')
    expect_box_error(test_use(a/../b[foo]), 'can only be used as a prefix')
    expect_box_error(test_use(a/b/..[foo]), 'can only be used as a prefix')

    expect_box_error(test_use(./../b), 'can only be used as a prefix')
    expect_box_error(test_use(./..), 'can only be used as a prefix')
    expect_box_error(test_use(./../b[foo]), 'can only be used as a prefix')
    expect_box_error(test_use(./..[foo]), 'can only be used as a prefix')
})

test_that('modules can have explicit aliases', {
    m = test_use(qux = foo/bar)
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'qux')
    expect_true(m$explicit)
    expect_null(m$attach)
})

test_that('modules can have specific exports', {
    m = test_use(foo/bar[sym1, sym2])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'bar')
    expect_false(m$explicit)
    expect_equal(m$attach, c(sym1 = 'sym1', sym2 = 'sym2'))
})

test_that('modules with alias can have specific exports', {
    m = test_use(baz = foo/bar[sym1, sym2])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'baz')
    expect_true(m$explicit)
    expect_equal(m$attach, c(sym1 = 'sym1', sym2 = 'sym2'))
})

test_that('modules can export everything', {
    m = test_use(foo/bar[...])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'bar')
    expect_false(m$explicit)
    expect_true(is.na(m$attach))
})

test_that('attached names can have aliases', {
    m = test_use(foo/bar[alias1 = sym1, sym2])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'bar')
    expect_false(m$explicit)
    expect_equal(m$attach, c(alias1 = 'sym1', sym2 = 'sym2'))
})

test_that('exports and aliases can’t be duplicated', {
    expect_box_error(test_use(foo/bar[a, b, a]), 'duplicate names')
    expect_box_error(test_use(foo/bar[x = a, y = b, x = c]), 'duplicate names')
    expect_box_error(test_use(foo/bar[..., ...]), 'duplicate names')

    m = test_use(foo/bar[x = a, a = b, b])
    expect_true(is_mod_spec(m))
    expect_equal(m$attach, c(x = 'a', a = 'b', b = 'b'))
})

test_that('wildcards can be mixed with aliases', {
    m = test_use(foo/bar[x = a, y = b, ...])
    expect_true(is_mod_spec(m))
    expect_equal(m$name, 'bar')
    expect_equal(m$prefix, 'foo')
    expect_equal(m$alias, 'bar')
    expect_false(m$explicit)
    expect_equal(m$attach, c(x = 'a', y = 'b', ... = NA_character_))
})

test_that('wildcard can’t have alias', {
    expect_box_error(test_use(foo/bar[x = ...]), 'cannot be aliased')
    expect_box_error(test_use(foo/bar[x = a, y = ...]), 'cannot be aliased')
})

# … the same for packages.

test_that('packages without attaching can be parsed', {
    m = test_use(foo)
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'foo')
    expect_false(m$explicit)
    expect_null(m$attach)
})

test_that('packages can have explicit alias', {
    m = test_use(bar = foo)
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'bar')
    expect_true(m$explicit)
    expect_null(m$attach)
})

test_that('packages can have specific exports', {
    m = test_use(foo[sym1, sym2])
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'foo')
    expect_false(m$explicit)
    expect_equal(m$attach, c(sym1 = 'sym1', sym2 = 'sym2'))
})

test_that('packages with alias can have specific exports', {
    m = test_use(bar = foo[sym1, sym2])
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'bar')
    expect_true(m$explicit)
    expect_equal(m$attach, c(sym1 = 'sym1', sym2 = 'sym2'))
})

test_that('packages can export everything', {
    m = test_use(foo[...])
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'foo')
    expect_false(m$explicit)
    expect_true(is.na(m$attach))
})

test_that('attached names in packages can have aliases', {
    m = test_use(foo[alias1 = sym1, sym2])
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'foo')
    expect_false(m$explicit)
    expect_equal(m$attach, c(alias1 = 'sym1', sym2 = 'sym2'))
})

test_that('exports and aliases can’t be duplicated', {
    expect_box_error(test_use(foo[a, b, a]), 'duplicate names')
    expect_box_error(test_use(foo[x = a, y = b, x = c]), 'duplicate names')
    expect_box_error(test_use(foo[..., ...]), 'duplicate names')

    m = test_use(foo[x = a, a = b, b])
    expect_true(is_pkg_spec(m))
    expect_equal(m$attach, c(x = 'a', a = 'b', b = 'b'))
})

test_that('wildcards can be mixed with aliases', {
    m = test_use(foo[x = a, y = b, ...])
    expect_true(is_pkg_spec(m))
    expect_equal(m$name, 'foo')
    expect_equal(m$alias, 'foo')
    expect_false(m$explicit)
    expect_equal(m$attach, c(x = 'a', y = 'b', ... = NA_character_))
})

test_that('wildcard can’t have alias', {
    expect_box_error(test_use(foo[x = ...]), 'cannot be aliased')
    expect_box_error(test_use(foo[x = a, y = ...]), 'cannot be aliased')
})

test_that('trailing comma is accepted', {
    expect_error(test_use(mod/a, ), NA)
    expect_error(test_use(mod/a, mod/b, ), NA)
    expect_error(test_use(mod/a[modname, double, ]), NA)
})

test_that('aliases need a name', {
    expect_box_error(test_use(alias =), 'alias without a name provided in use declaration')
    expect_box_error(box::use(mod/a, alias =), 'alias without a name provided in use declaration')
    expect_box_error(test_use(foo/bar[alias =, y]), 'alias without a name provided in attach list')
    expect_box_error(test_use(foo/bar[x, alias =]), 'alias without a name provided in attach list')
})

context('partial attaching')

test_that('partial attach works locally', {
    a = box::use(mod/a[double])
    expect_setequal(ls(parent.env(environment())), 'double')
})

test_that('partial attach works globally', {
    exports = c('inc', 'get_counter')
    local(box::use(a = mod/a[inc, get_counter]), envir = .GlobalEnv)
    expect_equal(search()[2L], environmentName(a))
    on.exit(detach(), add = TRUE)
    expect_setequal(ls(2L), c('get_counter', 'inc'))
})

test_that('Invalid attach specifier raises error', {
    expect_box_error(
        box::use(mod/a[foo, bar]),
        regexp = 'names .* not exported by'
    )
})

context('operators')

test_that('S3 operators work', {
    box::use(a = mod/a[...])
    on.exit(box::unload(a))

    expect_equal(1L + 2L, 3L)
    s = structure('foo', class = 'string')
    expect_equal(s + 'bar', 'foobar')
})

test_that('dot operators work', {
    # Test cases for the fix of issue 42.
    expect_false(exists('%.%'))
    expect_false(exists('%x.%'))
    expect_false(exists('%.x%'))
    expect_false(exists('%x.x%'))
    # S3 generic `%%`
    expect_false(exists('%%.%%'))
    expect_false(exists('%a%.class%'))

    box::use(a = mod/a[...])
    on.exit(unload(a))

    expect_true(exists('%.%'))
    expect_true(exists('%x.%'))
    expect_true(exists('%.x%'))
    expect_true(exists('%x.x%'))
    expect_true(exists('%%.%%'))
    expect_true(exists('%a%.class%'))
    # S3 method (not an operator)
    expect_true(exists('%foo.bar', envir = a))
})

help_format = function (format) {
    structure(list(), class = c(paste0(format, '_help_format'), 'help_format'))
}

compile_help = function (x, rd) UseMethod('compile_help')

patch_topic_name = function (x, file, topic) UseMethod('patch_topic_name')

display_help_file = function (x, file, topic) UseMethod('display_help_file')

mock_package_name = '\1PKG\1'

compile_help.text_help_format = function (x, rd) {
    tools::Rd2txt(rd, out = tempfile('Rtxt'), package = mock_package_name)
}

compile_help.html_help_format = function (x, rd) {
    withCallingHandlers(
        tools::Rd2HTML(rd, out = tempfile('Rtxt'), package = c(mock_package_name, NA_character_)),
        message = function (.) tryInvokeRestart('muffleMessage'),
        warning = function (.) tryInvokeRestart('muffleWarning')
    )
}

patch_topic_name.text_help_format = function (x, file, topic) {
    doc_text = readLines(file)
    old_topic = paste0('package:', mock_package_name)
    padded_old_topic = sprintf('%-*s', nchar(topic), old_topic)
    padded_new_topic = sprintf('%-*s', nchar(old_topic), topic)
    doc_text[1L] = sub(padded_old_topic, padded_new_topic, doc_text[1L])
    writeLines(doc_text, file)
}

patch_topic_name.html_help_format = function (x, file, topic) {
    doc_text = readLines(file)
    writeLines(gsub(mock_package_name, html_escape(topic), doc_text), file)
}

display_help_file.text_help_format = function (x, file, topic) {
    file.show(
        file, title = gettextf('R Help on %s', dQuote(topic)),
        delete.file = TRUE
    )
}

display_help_file.html_help_format = function (x, file, topic) {
    topic = sanitize_path_fragment(topic)
    port = tools::startDynamicHelp(NA)
    html_path = file.path(tempdir(), sprintf('.R/doc/html/%s.html', topic))
    dir.create(dirname(html_path), recursive = TRUE, showWarnings = FALSE)
    on.exit(unlink(file))
    file.copy(file, html_path, overwrite = TRUE)
    utils::browseURL(sprintf('http://127.0.0.1:%s/doc/html/%s.html', port, topic))
}

display_help = function (help_text, title, format = c('text', 'html')) {
    format = help_format(match.arg(format))
    rd = tools::parse_Rd(textConnection(help_text))
    tempfile = compile_help(format, rd)
    patch_topic_name(format, tempfile, title)
    display_help_file(format, tempfile, title)
}

context('meta')

# Test that the custom expectations behave as expected

test_that('`expect_not_equal` works', {
    expect_not_equal(1, 2)
    expect_not_equal('a', 'A')
    expect_failure(expect_not_equal(1, 1))

})

test_that('`expect_not_identical` works', {
    expect_not_identical(1, 2)
    expect_not_identical(1, 1L)
    expect_not_identical(new.env(), new.env())

    expect_failure(expect_not_identical(1, 1))
    expect_failure(expect_not_identical(list('a'), list('a')))
    expect_failure(expect_not_identical(asNamespace('base'), .BaseNamespaceEnv))
})

test_that('`expect_in` works', {
    expect_in(1, c(1, 2, 3))
    expect_in(2, c(1, 2, 3))
    expect_in('A', LETTERS)

    expect_failure(expect_in(1, c()))
    expect_failure(expect_in(1, c(2, 3)))
    expect_failure(expect_in('A', letters))
})

test_that('`expect_not_in` works', {
    expect_not_in(1, c())
    expect_not_in(1, c(2, 3))
    expect_not_in('A', letters)

    expect_failure(expect_not_in(1, c(1, 2, 3)))
    expect_failure(expect_not_in(2, c(1, 2, 3)))
    expect_failure(expect_not_in('A', LETTERS))
})

test_that('`expect_messages` works', {
    expect_messages(
        {
            message('foo')
            message('bar')
        },
        c('foo', 'bar')
    )

    expect_failure(
        expect_messages(
            {
                message('foo')
                message('bar')
            },
            c('foo', 'baz')
        ),
        'did not produce the expected message'
    )

    expect_failure(
        expect_messages(
            {
                message('foo')
                message('bar')
            },
            has_not = 'foo'
        ),
        'produced unwanted message\\(s\\):\n\\* "foo"'
    )
})

context('interactive')

skip_outside_source_repos()

play_back_results = function (record_events, onto) {
    for (event in record_events) {
        switch(
            event$type,
            start_test = onto$start_test(event$context, event$test),
            add_result = onto$add_result(event$context, event$test, event$result),
            end_test = onto$end_test(event$context, event$test)
        )
    }
}

interactive_r(code = {
    library(testthat)
    devtools::load_all()

    RecordReporter = R6::R6Class(
        'RecordReporter',
        inherit = Reporter,
        public = list(
            events = NULL,

            initialize = function () {
                super$initialize()
                self$events = testthat:::Stack$new()
            },

            start_test = function (context, test) {
                self$events$push(list(type = 'start_test', context = context, test = test))
            },

            add_result = function (context, test, result) {
                self$events$push(list(type = 'add_result', context = context, test = test, result = result))
            },

            end_test = function (context, test) {
                self$events$push(list(type = 'end_test', context = context, test = test))
            },

            get_events = function () {
                self$events$as_list()
            }
        )
    )

    record = RecordReporter$new()
    tryCatch(
        test_file('test-basic.r', reporter = record),
        finally = saveRDS(record$get_events(), 'test_results.rds')
    )
})

record_events = readRDS('test_results.rds')
unlink('test_results.rds')
play_back_results(record_events, get_reporter())

#' @export
this_module_name = box::name()

#' @export
function_module_name = function () box::name()

box::use(./a[...])

# Is this changed by the previous import/attach?
#' @export
this_module_name2 = box::name()

#' @export
after_module_attach = function () {
    # Muffle message
    silently = function (expr) {
        on.exit({sink(); close(file)})
        file = textConnection('out', 'w', local = TRUE)
        sink(file)
        expr
    }
    silently(box::use(a = ./nested/a[...]))
    on.exit(box::unload(a))
    box::name()
}

#' @export
after_package_attach = function () {
    box::use(datasets[...])
    box::name()
}

#' @export
nested_module_name = function () {
    local({
        box::use(datasets[...])
        box::name()
    })
}

context('names')

test_that('the global namespace has no module name', {
    expect_null(box::name())
})

test_that('modules have a name', {
    box::use(mod/a)
    expect_equal(a$get_modname(), 'a')
})

test_that('module names can be read inside functions', {
    box::use(mod/a)
    expect_equal(a$get_modname2(), 'a')
})

test_that('module_name works after attaching modules', {
    # Test that #66 is fixed and that there are no regressions.

    box::use(a = mod/a[...])
    expect_null(box::name())

    in_globalenv({
        box::use(a = mod/a[...])
        on.exit(box::unload(a))
        expect_null(box::name())
    })

    box::use(x = mod/mod_name)

    expect_equal(x$this_module_name, 'mod_name')
    expect_equal(x$function_module_name(), 'mod_name')
    expect_equal(x$this_module_name2, 'mod_name')
    expect_equal(x$after_module_attach(), 'mod_name')
    expect_equal(x$after_package_attach(), 'mod_name')
    expect_equal(x$nested_module_name(), 'mod_name')
})

context('file paths')

test_that('box::file works in global namespace', {
    expect_paths_equal(box::file(), getwd())
    this_file = (function () getSrcFilename(sys.call(sys.nframe())))()
    expect_true(nzchar(this_file)) # Just to make sure.
    expect_true(nzchar(box::file(this_file)))
})

test_that('box::file works for module', {
    box::use(mod/a)
    expect_true(grepl('/b$', box::file('b', module = a)))
    expect_true(grepl('/c\\.r$', box::file('c.r', module = a)))
    expect_length(box::file(c('b', 'c.r'), module = a), 2L)
})

test_that('box::base_path works', {
    skip_outside_source_repos()
    # On earlier versions of “devtools”, this test reproducibly segfaulted due
    # to the call to `load_all` from within a script. This seems to be fixed now
    # with version 1.9.1.9000.
    script_path = 'support/devtools_segfault.r'

    rcmd_result = rcmd(script_path)
    expect_paths_equal(rcmd_result, file.path(getwd(), 'support'))

    rscript_result = rscript(script_path)
    expect_paths_equal(rscript_result, file.path(getwd(), 'support'))
})

test_that('box::file works after attaching modules', {
    # R CMD CHECK resets the working directory AFTER executing the test helpers.
    # This throws off the subsequent tests, so we need to re-set the path here
    # although this shouldn’t be necessary.
    old_opts = options(box.path = getwd())
    on.exit(options(old_opts))

    # Test that #66 is fixed and that there are no regressions.

    expected_module_file = box::file()
    box::use(mod/a[...])
    expect_paths_equal(box::file(), expected_module_file)

    modfile = in_globalenv({
        expected_module_file = box::file()
        box::use(a = mod/a[...])
        on.exit(box::unload(a))
        list(actual = box::file(), expected = expected_module_file)
    })

    expect_paths_equal(modfile$actual, modfile$expected)

    box::use(x = mod/mod_file)
    expected_module_file = file.path(getwd(), 'mod')
    expect_paths_equal(x$this_module_file, expected_module_file)
    expect_paths_equal(x$function_module_file(), expected_module_file)
    expect_paths_equal(x$this_module_file2, expected_module_file)
    expect_paths_equal(x$after_module_attach(), expected_module_file)
    expect_paths_equal(x$after_package_attach(), expected_module_file)
    expect_paths_equal(x$nested_module_file(), expected_module_file)
})

test_that('regression #76 is fixed', {
    box::use(x = mod/issue76)
    expect_equal(x$helper_var, 3L)
})

test_that('regression #79 is fixed', {
    skip_outside_source_repos()
    script_path = 'support/issue79.r'
    result = tail(interactive_r(script_path), 3L)

    expect_equal(result[1L], '> before; after')
    expect_equal(result[2L], 'NULL')
    # The following assertion in particular should not fail.
    expect_equal(result[3L], 'NULL')
})

test_that('common split_path operations are working', {
    expect_correct_path_split('foo')
    expect_correct_path_split('foo/')
    expect_correct_path_split('./foo')
    expect_correct_path_split('./foo/')
    expect_correct_path_split('foo/bar')
    expect_correct_path_split('foo/bar/')
    expect_is_cwd('.')
    expect_is_cwd('./')
    expect_is_cwd('./.')
    expect_correct_path_split('~')
    expect_correct_path_split('~/foo')
})

test_that('split_path is working on Unix', {
    skip_on_os('windows')

    expect_correct_path_split('/foo/bar')
    expect_correct_path_split('/foo/bar/')
    expect_correct_path_split('/.')
})

test_that('split_path is working on Windows', {
    if (.Platform$OS.type != 'windows') skip('Only run on Windows')

    # TODO: Standard paths
    # TODO: UNC paths
})

test_that('`box::file()` checks its arguments', {
    expect_error(box::file('a', mustWork = FALSE), 'unexpected named argument')
})

#' An alternative module system for R
#'
#' Use \code{box::use(prefix/mod)} to import a module, or \code{box::use(pkg)}
#' to import a package. Fully qualified names are supported for nested modules,
#' reminiscent of module systems in many other modern languages.
#'
#' @section Using modules & packages:
#'
#' \itemize{
#'  \item \code{\link[=use]{box::use}}
#' }
#'
#' @section Writing modules:
#'
#' Infrastructure and utility functions that are mainly used inside modules.
#'
#' \itemize{
#   \item \code{\link[=export]{box::export}}
#'  \item \code{\link[=file]{box::file}}
#'  \item \code{\link[=name]{box::name}}
#'  \item \code{\link[=register_S3_method]{box::register_S3_method}}
#'  \item \link{mod-hooks}
#' }
#'
#' @section Interactive use:
#'
#' Functions for use in interactive sessions and for testing.
#'
#' \itemize{
#'  \item \code{\link[=help]{box::help}}
#'  \item \code{\link[=unload]{box::unload}},
#'      \code{\link[=reload]{box::reload}},
#'      \code{\link[=unload]{box::purge_cache}}
#'  \item \code{\link[=set_script_path]{box::set_script_path}}
#'  \item \code{\link[=script_path]{box::script_path}},
#'      \code{\link[=script_path]{box::set_script_path}}
#' }
#'
#' @useDynLib box, .registration = TRUE
#' @name box
#' @docType package
#' @keywords internal
'_PACKAGE'

.onLoad = function (libname, pkgname) {
    ns = base::topenv()
    ns$system_mod_path = system.file('mod', package = pkgname)

    define_backports(ns)

    set_import_env_parent()
}

.onUnload = function (libpath) {
    purge_cache()
}

.onAttach = function (libname, pkgname) {
    # Do not permit attaching ‘box’, except during build/check/CI.
    if (
        called_from_devtools()
        || called_from_pkgdown()
        || called_from_ci()
        # `utils::example` also attaches the package.
        || called_from_example()
    ) return()

    is_bad_call = function (call) {
        as.character(call[[1L]]) %in% c('library', 'require')
    }

    # `unname` prevents the name from being displayed as `c(name = "box")`.
    pkgname = unname(pkgname)
    # Deparsed to silence spurious `R CMD check` warning.
    default = call('library', as.name(pkgname))
    bad_call = Filter(is_bad_call, sys.calls())[1L][[1L]] %||% default
    throw(
        'the {pkgname;\'} package is not supposed to be attached!\n\n',
        'Please consult the user guide at `{vignette}`.',
        vignette = call('vignette', pkgname, package = pkgname),
        call = bad_call,
        subclass = 'box_attach_error'
    )
}

called_from_devtools = function () {
    is_devtools_ns = function (x) x %==% getNamespace('devtools')

    isNamespaceLoaded('devtools') &&
        ! nzchar(Sys.getenv('R_BOX_TEST_ALLOW_DEVTOOLS')) &&
        any(map_lgl(is_devtools_ns, lapply(sys.frames(), base::topenv)))
}

called_from_pkgdown = function () {
    isNamespaceLoaded('pkgdown') && {
        is_pkgdown_ns = function (x) x %==% getNamespace('pkgdown')
        any(map_lgl(is_pkgdown_ns, lapply(sys.frames(), base::topenv)))
    }
}

called_from_ci = function () {
    any(Sys.getenv(c('R_INSTALL_PKG', '_R_CHECK_PACKAGE_NAME_', 'R_TESTS')) != '')
}

called_from_example = function () {
    utils_ns = getNamespace('utils')
    example = quote(example)
    frames = sys.frame()
    # N.B.: This only handles direct, unqualified calls, i.e.
    #   example(…)
    # it fails with other forms, such as
    #   utils::example(…)
    #   get('example')(…)
    # etc.
    is_example_call = function (i)
        sys.call(i)[[1L]] %==% example &&
            base::topenv(sys.frame(i)) %==% utils_ns
    any(map_lgl(is_example_call, seq_len(sys.nframe())))
}

import_env_parent = NULL

# Separate function for unit testing.
set_import_env_parent = function () {
    utils::assignInMyNamespace(
        'import_env_parent',
        if (getOption('box.warn.legacy', TRUE)) {
            legacy_intercept_env
        } else {
            baseenv()
        }
    )
}

#' @export
a = 'reexport$a'

box::use(./reexport_sub[a, b, c, d])

b = 'reexport$b'

#' @export
c = 'reexport$c'

#' @export
box::use(
    sub = ./reexport_sub[d = c, e = b]
    , # Keep trailing comma to test regression of fix for #263.
)

#' Unload or reload modules
#'
#' Given a module which has been previously loaded and is assigned to an alias
#' \code{mod}, \code{box::unload(mod)} unloads it; \code{box::reload(mod)}
#' unloads and reloads it from its source. \code{box::purge_cache()} marks all
#' modules as unloaded.
#' @usage \special{box::unload(mod)}
#' @param mod a module object to be unloaded or reloaded
#' @return These functions are called for their side effect. They do not return
#' anything.
#'
#' @details
#' Unloading a module causes it to be removed from the internal cache such that
#' the next subsequent \code{box::use} declaration will reload the module from
#' its source. \code{box::reload} unloads and reloads the specified modules and
#' all its transitive module dependencies. \code{box::reload} is \emph{not}
#' merely a shortcut for calling \code{box::unload} followed by \code{box::use},
#' because \code{box::unload} only unloads the specified module itself, not any
#' dependent modules.
#'
#' @note Any other references to the loaded modules remain unchanged, and will
#' (usually) still work. Unloading and reloading modules is primarily useful for
#' testing during development, and \emph{should not be used in production code:}
#' in particular, unloading may break other module references if the
#' \code{.on_unload} hook unloaded any binary shared libraries which are still
#' referenced.
#'
#' These functions come with a few restrictions.
#' \code{box::unload} attempts to detach names attached by the corresponding
#' \code{box::use} call.
#' \code{box::reload} attempts to re-attach these same names. This only works if
#' the corresponding \code{box::use} declaration is located in the same scope.
#' \code{box::purge_cache} only removes the internal cache of modules, it does
#' not actually invalidate any module references or names attached from loaded
#' modules.
#'
#' \code{box::unload} will execute the \code{.on_unload} hook of the module, if
#' it exists.
#' \code{box::reload} will re-execute the \code{.on_load} hook of the module and
#' of all dependent modules during loading (after executing the corresponding
#' \code{.on_unload} hooks during unloading).
#' \code{box::purge_cache} will execute any existing \code{.on_unload} hooks in
#' all loaded modules.
#' @seealso \code{\link[=use]{box::use}}, \link[=mod-hooks]{module hooks}
#' @export
unload = function (mod) {
    modname = substitute(mod)
    expect(
        is.name(modname),
        '{"unload";"} expects a module object, got {modname;"}, ',
        'which is not a module alias variable'
    )

    mod_ref = as.character(modname)
    expect(
        exists(mod_ref, envir = parent.frame()),
        'object {mod_ref;"} not found'
    )
    expect(
        inherits(mod, 'box$mod'),
        '{"unload";"} expects a module object, got {modname;"}, ',
        'which is of type {class(mod);"} instead'
    )

    mod_ns = attr(mod, 'namespace')
    attached = attr(mod, 'attached')

    if (! is.null(attached)) {
        if (attached %==% '') {
            caller = parent.frame()
            parent.env(caller) = parent.env(parent.env(caller))
        } else {
            attached_pos = match(attached, search())
            if (! is.na(attached_pos)) {
                detach(attached, character.only = TRUE)
            }
        }
    }

    unload_mod(mod_ns, attr(mod, 'info'))

    # Unset the mod reference in its scope, i.e. the caller’s environment or
    # some parent thereof.
    rm(list = mod_ref, envir = parent.frame(), inherits = TRUE)
}

#' @usage \special{box::reload(mod)}
#' @name unload
#' @export
reload = function (mod) {
    modname = substitute(mod)
    expect(
        is.name(modname),
        '{"reload";"} expects a module object, got {modname;"}, ',
        'which is not a module alias variable'
    )

    mod_ref = as.character(modname)
    expect(
        exists(mod_ref, envir = parent.frame()),
        'object {mod_ref;"} not found'
    )
    expect(
        inherits(mod, 'box$mod'),
        '{"reload";"} expects a module object, got {modname;"}, ',
        'which is of type {class(mod);"} instead'
    )

    caller = parent.frame()
    spec = attr(mod, 'spec')
    info = attr(mod, 'info')
    mod_ns = attr(mod, 'namespace')
    attached = attr(mod, 'attached')

    unload_mod_recursive(mod_ns, info)

    on.exit({
        warning(fmt('Reloading module {modname;"} failed, attempting to restore the old instance.'))
        register_mod(info, mod_ns)
    })

    if (! is.null(attached)) {
        if (attached %==% '') {
            attached_env = parent.env(caller)
            parent.env(caller) = parent.env(attached_env)
            on.exit((parent.env(caller) = attached_env), add = TRUE)
        } else {
            attached_pos = match(attached, search())
            if (! is.na(attached_pos)) {
                detach(attached, character.only = TRUE)
            }
            on.exit(
                box_attach(attached_env, pos = attached_pos, name = attached),
                add = TRUE
            )
        }
    }

    load_and_register(spec, info, caller)
    # Loading worked, so cancel restoring the old module.
    on.exit()
}

#' @usage \special{box::purge_cache()}
#' @name unload
#' @export
purge_cache = function () {
    eapply(loaded_mods, function (mod_ns) {
        call_hook(mod_ns, '.on_unload', mod_ns)
    }, all.names = TRUE)
    rm(list = names(loaded_mods), envir = loaded_mods)
}

#' @keywords internal
unload_mod = function (mod_ns, info) {
    call_hook(mod_ns, '.on_unload', mod_ns)
    deregister_mod(info)
}

#' @keywords internal
unload_mod_recursive = function (mod_ns, info) {
    UseMethod('unload_mod_recursive')
}

`unload_mod_recursive.box$ns` = function (mod_ns, info) {
    unload_mod(mod_ns, info)

    for (import in namespace_info(mod_ns, 'imports')) {
        unload_mod_recursive(import$ns, import$info)
    }
}

# Package namespace
unload_mod_recursive.environment = function (mod_ns, info) {}

context('hooks')

test_that('on_load hook is invoked', {
    box::use(mod/hooks/a)
    expect_equal(a$on_load_called, 1L)
})

test_that('on_load hook is invoked only once', {
    local({
        box::use(mod/hooks/a)
        expect_equal(a$on_load_called, 1L) # Still 1
    })
    local({
        box::use(mod/hooks/a)
        expect_equal(a$on_load_called, 1L) # STILL 1
    })
})

test_that('on_unload hook is invoked during unloading', {
    self = environment()
    unload_called = 0L

    box::use(mod/hooks/a)
    a$register_unload_callback(function () self$unload_called = unload_called + 1L)

    expect_equal(unload_called, 0L)
    box::unload(a)
    expect_equal(unload_called, 1L)
})

test_that('hooks are invoked during reloading', {
    self = environment()
    unload_called = 0L

    box::use(mod/hooks/a)
    a$register_unload_callback(function () self$unload_called = unload_called + 1L)

    expect_equal(unload_called, 0L)
    box::reload(a)
    expect_equal(unload_called, 1L)
    expect_equal(a$on_load_called, 1L)
})

context('reloading')

is_module_loaded = function (path) {
    path %in% names(box:::loaded_mods)
}

tempfile_dir = function (...) {
    file = tempfile()
    dir.create(file)
    file
}

create_nested_test_module = function (dir) {
    mod = file.path(dir, 'mod', 'a')
    dir.create(mod, recursive = TRUE)
    writeLines("#' @export\nbox::use(./sub)", file.path(mod, '__init__.r'))
    writeLines("#' @export\nvalue = 1L", file.path(mod, 'sub.r'))
}

edit_nested_test_module = function (dir) {
    mod = file.path(dir, 'mod', 'a')
    writeLines("#' @export\nvalue = 2L", file.path(mod, 'sub.r'))
}

test_that('module can be reloaded', {
    # Required since other tests have side-effects.
    # Tear-down would be helpful here, but not supported by testthat.
    box::purge_cache()

    box::use(mod/a)
    expect_equal(length(box:::loaded_mods), 1L)
    counter = a$get_counter()
    a$inc()
    expect_equal(a$get_counter(), counter + 1L)

    box::reload(a)
    expect_true(is_module_loaded(box:::path(a)))
    expect_length(box:::loaded_mods, 1L)
    expect_equal(a$get_counter(), counter)
})

test_that('reload checks its arguments', {
    expect_error(box::reload(123))
    expect_error(box::reload(foo))
    box::use(mod/a)
    expect_error(box::reload((a)))
})

test_that('reload includes module dependencies', {
    # This test case actually edits a dependency and reloads the edit. The
    # purpose of this is to ensure that reloading doesn’t merely call `.on_load`
    # again, but actually does reload the changes from disk.
    dir = tempfile_dir()
    on.exit(unlink(dir, recursive = TRUE))

    old_path = options(box.path = dir)
    on.exit(options(old_path), add = TRUE)

    old_env = Sys.getenv('R_BOX_PATH', NA)
    if (!is.na(old_env)) {
        Sys.unsetenv('R_BOX_PATH')
        on.exit(Sys.setenv(R_BOX_PATH = old_env), add = TRUE)
    }

    create_nested_test_module(dir)

    box::use(mod/a)

    expect_equal(a$sub$value, 1L)

    edit_nested_test_module(dir)

    box::reload(a)

    expect_equal(a$sub$value, 2L)

    # To do:
    # * modules with compiled source,
    # * tricky packages loaded as modules, e.g. packages that call
    #   system.file(), and alike, and
    # * modules with S4 classes/object,
})

test_that('reload includes transitive dependencies', {
    # Unlike in the previous test, this test uses `.on_load` as an indicator of
    # reloading, to keep things simpler.
    box::use(mod/reload/a)
    expect_messages(
        box::reload(a),
        has = c('^a unloaded', '^c unloaded', '^c loaded')
    )
})

test_that('reload of transitive imports skips packages', {
    box::use(mod/reload/pkg)
    expect_error(box::reload(pkg), NA)
})

test_that('`reload` shows expected errors', {
    old_opts = options(useFancyQuotes = FALSE)
    on.exit(options(old_opts))

    expect_box_error(
        box::reload(mod/a),
        '"reload" expects a module object, got "mod/a"'
    )
    expect_box_error(
        box::reload(./a),
        '"reload" expects a module object, got "./a"'
    )
    expect_box_error(box::reload(na), 'object "na" not found')

    x = 1L
    expect_box_error(
        box::reload(x),
        '"reload" expects a module object, got "x", which is of type "integer" instead'
    )
})

test_that('reloading a module used twice does not cause a warning', {
    box::use(mod/reload/nested/c)

    expect_warning(box::reload(c), NA)
})

context('attaching')

# File name starts with `z` so that the test is executed last.

test_mod_envname = 'mod:mod/a'

test_that('‘box’ cannot be attached', {
    skip_on_cran()
    pkg_pos = match('package:box', search())
    if (! is.na(pkg_pos)) {
        pkg = as.environment(pkg_pos)
        detach(pos = pkg_pos)
    }

    # Temporarily unset environment indicating that we’re inside testing mode:
    envs = c('_R_CHECK_PACKAGE_NAME_', 'R_TESTS')
    old_env = Sys.getenv(envs)
    on.exit({
        do.call('Sys.setenv', as.list(old_env))
        Sys.unsetenv('R_BOX_TEST_ALLOW_DEVTOOLS')
        if (! is.na(pkg_pos)) {
            attach(pkg, pos = pkg_pos, name = environmentName(pkg))
        }
    })
    Sys.unsetenv(envs)
    Sys.setenv(R_BOX_TEST_ALLOW_DEVTOOLS = 'TRUE')
    expect_error(attachNamespace('box'), 'Please consult the user guide at `vignette\\("box", package = "box"\\)`')
})

test_that('attach works locally inside module', {
    box::use(mod/no_exports)
    # `no_exports` attaches `a`. So check that `a` is *not* attached here.
    expect_not_in(test_mod_envname, search())
    expect_false(exists('modname'))
})

test_that('attach works locally inside function', {
    f = function () {
        box::use(mod/a[...])
        expect_true(exists('modname'))
    }

    f()
    expect_false(exists('modname'))
})

test_that('module can be attached to global environment', {
    .GlobalEnv$searchlen = length(search())
    in_globalenv({
        box::use(a = mod/a[...])
        mod_path = box:::path(a)
        expect_length(search(), searchlen + 1L)
        expect_true(mod_path %in% names(box:::loaded_mods))
        expect_equal(search()[2L], environmentName(a))
    })
    rm(searchlen, envir = .GlobalEnv)
})

test_that('module can be detached', {
    expect_equal(search()[2L], test_mod_envname)
    parent = as.environment(3L)
    detach(test_mod_envname, character.only = TRUE)
    expect_identical(as.environment(2L), parent)
})

test_that('hidden names can be attached', {
    expect_false(exists('.hidden'))
    box::use(mod/a[...])
    expect_true(exists('.hidden'))
})

test_that('unloading a module detaches it', {
    parent = as.environment(2L)
    local(box::use(a = mod/a[...]), .GlobalEnv)
    expect_equal(search()[2L], test_mod_envname)
    expect_not_identical(as.environment(2L), parent)

    in_globalenv(box::unload(a))
    expect_identical(as.environment(2L), parent)
})

test_that('unloading a local module detaches it', {
    local({
        old_parent = parent.env(environment())
        box::use(a = mod/a[...])
        new_parent = parent.env(environment())
        expect_not_identical(old_parent, new_parent)
        expect_identical(old_parent, parent.env(new_parent))
        box::reload(a)
        expect_not_identical(old_parent, parent.env(environment()))
        expect_not_identical(new_parent, parent.env(environment()))
        expect_identical(old_parent, parent.env(parent.env(environment())))
    })
})

test_that('reloading a module reattaches it', {
    parent = as.environment(2L)
    local(box::use(a = mod/a[...]), .GlobalEnv)

    expect_equal(search()[2L], test_mod_envname)
    expect_not_identical(as.environment(2L), parent, 'Precondition')
    expect_identical(as.environment(3L), parent, 'Precondition')

    in_globalenv(box::reload(a))
    expect_not_identical(as.environment(2L), parent)
    expect_identical(as.environment(3L), parent)
    in_globalenv(box::unload(a))
})

context('unloading')

is_module_loaded = function (path) {
    path %in% names(box:::loaded_mods)
}

test_that('module can be unloaded', {
    box::use(mod/a)
    path = box:::path(a)
    expect_true(is_module_loaded(path))
    box::unload(a)
    expect_false(is_module_loaded(path))
    expect_false(exists('a', inherits = FALSE))
})

test_that('unloaded module can be reloaded', {
    box::use(mod/a)
    box::unload(a)
    box::use(mod/a)
    expect_true(is_module_loaded(box:::path(a)))
    expect_true(exists('a', inherits = FALSE))
})

test_that('unload checks its arguments', {
    expect_error(box::unload(123))
    expect_error(box::unload(foo))
    box::use(mod/a)
    expect_error(box::unload((a)))
})

test_that('unloading calls unload hook', {
    box::use(mod/reload/a)
    expect_message(box::unload(a), '^a unloaded')
})

test_that('unloading does not unload dependencies', {
    box::use(mod/reload/a)
    expect_messages(
        box::unload(a),
        has = '^a unloaded',
        has_not = '^c unloaded'
    )
})

test_that('`unload` shows expected errors', {
    old_opts = options(useFancyQuotes = FALSE)
    on.exit(options(old_opts))

    expect_box_error(
        box::unload(mod/a),
        '"unload" expects a module object, got "mod/a"'
    )
    expect_box_error(
        box::unload(./a),
        '"unload" expects a module object, got "./a"'
    )
    expect_box_error(box::unload(na), 'object "na" not found')

    x = 1L
    expect_box_error(
        box::unload(x),
        '"unload" expects a module object, got "x", which is of type "integer" instead'
    )
})

test_that('purging cache marks modules as unloaded', {
    box::use(mod/a)
    path = box:::path(a)

    expect_true(is_module_loaded(path))
    box::purge_cache()
    expect_false(is_module_loaded(path))
})

test_that('after purging the cache, modules get reloaded', {
    box::use(mod/a)
    a$inc()
    counter = a$get_counter()

    box::purge_cache()
    box::use(mod/a)
    expect_not_equal(a$get_counter(), counter)
})

test_that('purging the cache executes `.on_unload` hooks', {
    box::use(mod/reload/a)
    expect_message(box::purge_cache(), '^a unloaded')
})

#' Retrieve a value or a default
#'
#' \code{a \%||\% b} returns \code{a} unless it is empty, in which case
#' \code{b} is returned.
#' @param a the value to return if non-empty
#' @param b default value
#' @return \code{a \%||\% b} returns \code{a}, unless it is \code{NULL}, empty,
#' \code{FALSE} or \code{""}; in which case \code{b} is returned.
#' @name default
#' @keywords internal
`%||%` = function (a, b) {
    if (length(a) == 0L || a %==% FALSE || a %==% '') {
        b
    } else {
        a
    }
}

#' \code{lhs \%|\% rhs} is a vectorized version.
#' @param lhs vector with potentially missing values, or \code{NULL}
#' @param rhs vector with default values, same length as \code{lhs} unless that
#'  is \code{NULL}
#' @return \code{lhs \%|\% rhs} returns a vector of the same length as
#' \code{rhs} with all missing values in \code{lhs} replaced by the
#' corresponding values in \code{rhs}.
#' @rdname default
#' @keywords internal
`%|%` = function (lhs, rhs) {
    if (is.null(lhs)) {
        rhs
    } else {
        mapply(`%||%`, lhs, rhs, USE.NAMES = FALSE)
    }
}

#' @export
on_load_called = 0L

.on_load = function (ns) {
    ns$on_load_called = ns$on_load_called + 1L
}

#' @export
register_unload_callback = local({
    self = environment()
    unloaded = NULL

    function (callback) {
        self$unloaded = callback
    }
}, envir = (callback = new.env()))

.on_unload = function (ns) {
    if (! is.null(callback$unloaded)) {
        callback$unloaded()
    }
}

#' @keywords internal
call_hook = function (ns, hook, ...) {
    if (namespace_info(ns, 'legacy', FALSE)) return()

    hook_fn = get0(hook, ns, mode = 'function', inherits = FALSE)
    if (! is.null(hook_fn)) hook_fn(...)
}

#' Hooks for module events
#'
#' Modules can declare functions to be called when a module is first loaded.
#'
#' @param ns the module namespace environment
#' @return Any return values of the hook functions are ignored.
#'
#' @note The API for hook functions is still subject to change. In particular,
#' there might in the future be a way to subscribe to module events of other
#' modules and packages, equivalently to R package \link[base]{userhooks}.
#'
#' @details
#' To create module hooks, modules should define a function with the specified
#' name and signature. Module hooks should \emph{not} be exported.
#'
#' When \code{.on_load} is called, the unlocked module namespace environment is
#' passed to it via its parameter \code{ns}. This means that code in
#' \code{.on_load} is permitted to modify the namespace by adding names to,
#' replacing names in, or removing names from the namespace.
#'
#' \code{.on_unload} is called when modules are unloaded. The (locked) module
#' namespace is passed as an argument. It is primarily useful to clean up
#' resources used by the module. Note that, as for packages, \code{.on_unload}
#' is \emph{not} necessarily called when R is shut down.
#'
#' \emph{Legacy modules} cannot use hooks. To use hooks, the module needs to
#' contain an export specification (if the module should not export any names,
#' specify an explicit, empty export list via
#' \code{\link[=export]{box::export()}}.
#'
#' @name mod-hooks
#' @keywords utilities
#' @concept experimental
.on_load = function (ns) NULL

#' @name mod-hooks
.on_unload = function (ns) NULL

# Module/package specification expression grammar as a PEG, approximating the R
# parse tree:
#
# spec        → pkg_name (“[” attach_spec “]”)? !“/” /
#               mod
# pkg_name    → name
# mod         → mod_prefix “/” mod_name (“[” attach_spec “]”)?
# mod_prefix  → name !“/” /
#               mod_prefix “/” name
# mod_name    → name
# attach_spec → “...” /
#               attach_list
# attach_list → name (“,” name)+
# name        → ‹valid R name›
#
# Note: This is missing the alias declaration, since that is not captured in the
# AST provided by R. It is instead handled separately.

#' Parse a mod or pkg spec expression passed to \code{use}
#'
#' @param expr the mod or pkg spec expression to parse
#' @param alias the mod or pkg spec alias as a character, or \code{NULL}
#' @return \code{parse_spec} returns a named list that contains information
#' about the parsed mod specification. Currently it contains:
#' \describe{
#'  \item{\code{name}}{the module or package name}
#'  \item{\code{prefix}}{the prefix, if the spec is a module}
#'  \item{\code{attach}}{a named vector of symbols to attach, or
#'      \code{TRUE} to attach all symbols, or \code{NULL} to attach nothing}
#'  \item{\code{alias}}{the module or package alias}
#'  \item{\code{explicit}}{a logical value indicating whether the caller
#'      provided an explicit alias}
#' }
#' @keywords internal
#' @name spec
parse_spec = function (expr, alias) {
    spec = parse_spec_impl(expr)

    is_pkg = 'pkg' %in% names(spec)
    spec_type = if (is_pkg) pkg_spec else mod_spec
    name = spec[[if (is_pkg) 'pkg' else 'mod']]$name
    spec_type(spec, alias = alias %||% name, explicit = nzchar(alias))
}

#' @param spec named list of information the parser constructed from a given
#' spec expression
#' @param \dots further information about a spec, not represented by the spec
#' expression parse tree
#' @keywords internal
#' @name spec
mod_spec = function (spec, ...) {
    extra_spec = spec[setdiff(names(spec), 'mod')]
    structure(c(spec$mod, extra_spec, ...), class = c('box$mod_spec', 'box$spec'))
}

#' @keywords internal
#' @name spec
pkg_spec = function (spec, ...) {
    extra_spec = spec[setdiff(names(spec), 'pkg')]
    structure(c(spec$pkg, extra_spec, ...), class = c('box$pkg_spec', 'box$spec'))
}

#' @keywords internal
#' @name spec
spec_name = function (spec) {
    UseMethod('spec_name')
}

#' @export
`spec_name.box$mod_spec` = function (spec) {
    paste(paste(spec$prefix, collapse = '/'), spec$name, sep = '/')
}

#' @export
`spec_name.box$pkg_spec` = function (spec) {
    spec$name
}

#' @export
`print.box$spec` = function (x, ...) {
    cat(as.character(x, ...), '\n', sep = '')
    invisible(x)
}

#' @export
`as.character.box$spec` = function (x, ...) {
    r_name = function (names) {
        map_chr(
            function (n) {
                if (is.na(n)) '\u2026' else fmt('\x1b[33m{as.name(n)}\x1b[0m')
            },
            names
        )
    }

    format_attach = function (a) {
        with_alias = function (names, aliases) {
            mapply(
                function (n, a) if (n == '\u2026' || n %==% a) n else paste(a, '=', n),
                names, aliases
            )
        }

        if (is.null(a)) {
            ''
        } else {
            aliased_names = with_alias(r_name(a), r_name(names(a)))
            paste(', attach =', paste0('[', toString(aliased_names), ']'))
        }
    }

    mod_or_pkg = function (spec) {
        if (inherits(spec, 'box$mod_spec')) {
            prefix = paste(r_name(spec$prefix), collapse = '/')
            fmt('mod({prefix}/\x1b[4;33m{r_name(spec$name)}\x1b[0m)')
        } else {
            fmt('pkg(\x1b[4;33m{r_name(spec$name)}\x1b[0m)')
        }
    }

    fmt(
        'mod_spec({alias}{mod_or_pkg(x)}{format_attach(x$attach)})',
        alias = if (x$explicit) paste(r_name(x$alias), '= ') else ''
    )
}

parse_spec_impl = function (expr) {
    if (is.name(expr)) {
        if (expr %==% quote(.)) {
            list(prefix = '.', name = '__init__')
        } else if (expr %==% quote(..)) {
            list(prefix = '..', name = '.')
        } else {
            c(parse_pkg_name(expr), list(attach = NULL))
        }
    } else if (is.call(expr)) {
        if (expr[[1L]] %==% quote(`[`)) {
            if (is.name((name = expr[[2L]]))) {
                if (name %==% quote(.)) {
                    c(list(prefix = '.', name = '__init__'), parse_attach_spec(expr))
                } else if (name %==% quote(..)) {
                    # parse_mod(bquote(../.(`[[<-`(expr, 2L, quote(.)))))
                    c(list(prefix = '..', name = '.'), parse_attach_spec(expr))
                } else {
                    c(parse_pkg_name(name), parse_attach_spec(expr))
                }
            } else {
                throw('expected a name in {expr;"}, got {describe_token(name)}')
            }
        } else if (expr[[1L]] %==% quote(`/`)) {
            parse_mod(expr)
        } else {
            throw('expected {"/";"} in {expr;"}, got {expr[[1L]];"}')
        }
    } else {
        throw('expected a module or package name, got {describe_token(expr)}')
    }
}

parse_pkg_name = function (expr) {
    name = parse_name(expr)
    if (is.na(name)) {
        throw('alias without a name provided in use declaration')
    }
    list(pkg = list(name = name))
}

parse_mod = function (expr) {
    prefix = parse_mod_prefix(expr[[2L]])
    mod = expr[[3L]]

    # Runs of `..` from the start are valid; `..` in the middle is not:

    if (any(diff(which(c(TRUE, prefix$prefix == '..'))) > 1L)) {
        # At least one gap in the sequence of `..` from the start
        throw('{"..";"} can only be used as a prefix')
    }

    if ('.' %in% prefix$prefix[-1L]) {
        throw('{".";"} can only be used as a prefix')
    }

    if (is.call(mod)) {
        if (mod[[1L]] %==% quote(`[`)) {
            c(
                list(mod = c(parse_mod_name(mod[[2L]], prefix), prefix)),
                parse_attach_spec(mod)
            )
        } else {
            throw('expected a name in {expr;"}, got {describe_token(mod)}')
        }
    } else if (is.name(mod)) {
        list(mod = c(parse_mod_name(mod, prefix), prefix), attach = NULL)
    } else {
        throw('expected a name in {expr;"}, got {describe_token(mod)}')
    }
}

parse_mod_prefix = function (expr) {
    if (is.name(expr)) {
        list(prefix = deparse1(expr))
    } else if (is.call(expr) && expr[[1L]] %==% quote(`/`)) {
        if (! is.name(expr[[3L]])) {
            throw('expected a name in module prefix, got {describe_token(expr[[3L]])}')
        } else {
            suffix = deparse1(expr[[3L]])
            list(prefix = c(parse_mod_prefix(expr[[2L]])$prefix, suffix))
        }
    } else {
        throw('expected a module prefix, got {describe_token(expr)}')
    }
}

parse_mod_name = function (expr, prefix) {
    name = parse_name(expr)

    if (length((prefix = prefix$prefix)) > 0L) {
        dot_after_prefix = name == '.'
        back_inside_path = ! all(prefix == '..') && name == '..'

        if (dot_after_prefix || back_inside_path) {
            throw('{expr;"} can only be used as a prefix')
        }
    }

    list(name = name)
}

parse_attach_spec = function (expr) {
    syms = parse_attach_list(expr[-1L][-1L])
    list(attach = assign_missing_names(syms))
}

assign_missing_names = function (syms) {
    x = stats::setNames(syms, names(syms) %|% unlist(syms))
    x[x == '...'] = NA_character_

    if (any((dup = duplicated(names(x))))) {
        throw('cannot attach duplicate names, found duplicated {unique(names(x)[dup]);"}')
    }
    if (any(names(syms)[is.na(x)] != '')) {
        throw('wildcard imports cannot be aliased')
    }
    x
}

parse_attach_list = function (expr) {
    if (length(expr) == 1L && expr[[1L]] %==% quote(expr =)) {
        throw('expected at least one name in attach list')
    } else {
        names = stats::setNames(map_chr(parse_name, expr), names(expr))

        missing_names = is.na(names)
        if (any(missing_names)) {
            # Allow missing name for last, unnamed argument to allow “trailing
            # comma” syntax:
            #   box::use(./a[x, y, ])
            index = which(missing_names)
            if (
                length(index) != 1L
                || index != length(names)
                || nzchar(names(names)[index] %||% '')
            ) {
                throw('alias without a name provided in attach list')
            }

            names[-index]
        } else {
            names
        }
    }
}

parse_name = function (expr) {
    if (length(expr) != 1L || ! is.name(expr)) {
        throw('expected a name, got {describe_token(expr)}')
    } else {
        deparse1(expr) %||% NA_character_
    }
}

describe_token = function (expr) {
    fmt('{class(expr)} literal {expr;"}')
}

#' Throw informative error messages
#'
#' Helpers to generate readable and informative error messages for package
#' users.
#' @param \dots arguments to be passed to \code{fmt}
#' @param call the calling context from which the error is raised
#' @param subclass an optional subclass name for the error condition to be
#' raised
#'
#' @details
#' For \code{rethrow}, the \code{call} argument overrides the rethrown error’s
#' own stored call.
#' @keywords internal
throw = function (..., call = sys.call(sys.parent()), subclass = NULL) {
    message = fmt(..., envir = parent.frame())
    stop(box_error(message, call = call, subclass = subclass))
}

#' @param error an object of class \code{c("error", "condition")} to rethrow
#' @rdname throw
rethrow = function (error, call = sys.call(sys.parent())) {
    if (inherits(error, 'box_error')) {
        # R calls all exit handlers during stack unwind after internally setting
        # the stack traceback. We use this fact to override the traceback with a
        # more useful version that shows the actual calls causing the error,
        # instead of the detour via the `rethrow` logic.
        on.exit({
            # In non-interactive sessions, `.Traceback` might not be created.
            if (exists('.Traceback', .BaseNamespaceEnv)) {
                # In all versions of R currently supported, `tryCatch` inserts 4
                # calls into the call stack, which we excise here.
                tb = error$traceback
                start = Position(function (x) x[[1L]] %==% quote(rethrow_on_error), tb)
                tb = tb[- seq(start + 1L, start + 4L)]
                if (getRversion() < '4.0.0') {
                    # Prior to R 4.0.0, `.Traceback` contains deparsed calls.
                    tb = map(deparse1, tb)
                }
                if (getRversion() >= '4.1.0') box_unlock_binding('.Traceback', .BaseNamespaceEnv)
                .BaseNamespaceEnv$.Traceback = tb
                if (getRversion() >= '4.1.0') lockBinding('.Traceback', .BaseNamespaceEnv)
            }
        })
    }
    message = conditionMessage(error)
    subclass = setdiff(class(error), box_error_class)
    stop(box_error(message, call = call, subclass = subclass))
}

#' @param expr an expression to evaluate inside \code{tryCatch}
#' @return If it does not throw an error, \code{rethrow_on_error} returns the
#' value of evaluating \code{expr}.
#' @rdname throw
rethrow_on_error = function (expr, call = sys.call(sys.parent())) {
    tryCatch(expr, error = function (error) rethrow(error, call))
}

expect = function (condition, ..., call = sys.call(sys.parent()), subclass = NULL) {
    if (condition) return()
    message = fmt(..., envir = parent.frame())
    stop(box_error(message, call = call, subclass = subclass))
}

box_error_class = c('box_error', 'error', 'condition')

#' @param message the error message
#' @return \code{box_error} returns a new \sQuote{box} error condition object
#' with a given message and call, and optionally a given subclass type.
#' @rdname throw
box_error = function (message, call = NULL, subclass = NULL) {
    class = c(subclass, box_error_class)
    # Store real traceback, in case this is being called from inside
    # `rethrow_on_error`, which overrides the stack trace.
    traceback = sys.calls()
    traceback = traceback[seq_len(length(traceback) - 2L)]
    structure(list(message = message, call = call, traceback = traceback), class = class)
}

#' Set the base path of the script
#'
#' \code{box::set_script_path(path)} explicitly tells \pkg{box} the path of a
#' given script from which it is called; \code{box::script_path()} returns the
#' previously set path.
#' @usage \special{box::set_script_path(path)}
#' @param path character string containing the relative or absolute path to the
#' currently executing \R code file, or \code{NULL} to reset the path.
#' @return Both \code{box::script_path} and \code{box::set_script_path} return
#' the previously set script path, or \code{NULL} if none was explicitly set.
#' \code{box::set_script_path} returns its value invisibly.
#'
#' @details
#' \pkg{box} needs to know the base path of the topmost calling \R context (i.e.
#' the script) to find relative import locations. In most cases, \pkg{box} can
#' figure the path out automatically. However, in some cases third-party
#' packages load code in a way in which \pkg{box} cannot find the correct path
#' of the script any more. \code{box::set_script_path} can be used in these
#' cases to set the path of the currently executing \R script manually.
#'
#' @note
#' \pkg{box} \emph{should} be able to figure out the script path automatically.
#' Using \code{box::set_script_path} should therefore never be necessary.
#' \href{https://github.com/klmr/box/issues/new?assignees=&labels=\%E2\%9A\%A0\%EF\%B8\%8F+bug&title=\%5Bset_script_path\%5D\%20&template=bug-report.yml}{Please
#' file an issue} if you encounter a situation that necessitates using
#' \code{box::set_script_path}!
#'
#' @examples
#' box::set_script_path('scripts/my_script.r')
#' @name script_path
#' @export
set_script_path = function (path = NULL) {
    old_path = script_path()
    script_path_env$script_path = path
    script_path_env$value = if (is.null(path)) NULL else dirname(path)
    invisible(old_path)
}

#' @usage \special{box::script_path()}
#' @rdname script_path
#' @export
script_path = function () {
    script_path_env$script_path
}

#' Get a module’s path
#'
#' The following functions retrieve information about the path of the directory
#' that a module or script is running in.
#' @param mod a module environment or namespace
#' @return \code{path} returns a character string containing the module’s full
#' path.
#' @keywords internal
path = function (mod) {
    UseMethod('path')
}

#' @export
`path.box$mod` = function (mod) {
    attr(mod, 'info')$source_path
}

#' @export
`path.box$ns` = function (mod) {
    namespace_info(mod, 'info')$source_path
}

#' @return \code{base_path} returns a character string containing the module’s
#' base directory, or the current working directory if not invoked on a module.
#' @rdname path
base_path = function (mod) {
    normalizePath(module_path(mod), winslash = '/', mustWork = FALSE)
}

script_path_env = new.env(parent = emptyenv())

#' @return \code{module_path} returns a character string that contains the
#' directory in which the calling R code is run. See \sQuote{Details}.
#' @details
#' \code{module_path} takes a best guess at a script’s path, since R does not
#' provide a sure-fire way for determining the path of the currently executing
#' code. The following calling situations are covered:
#'
#' \enumerate{
#'  \item Path explicitly set via \code{set_script_path}
#'  \item Path of a running document/application (\pkg{knitr}, \pkg{Shiny})
#'  \item Path of unit test cases (\pkg{testthat})
#'  \item Path of the currently opened source code file in RStudio
#'  \item Code invoked as \command{Rscript script.r}
#'  \item Code invoked as \command{R CMD BATCH script.r}
#'  \item Code invoked as \command{R -f script.r}
#'  \item Script run interactively (use \code{getwd()})
#' }
#' @rdname path
module_path = function (mod) {
    for (test in path_tests) {
        path = test(mod)
        if (! is.null(path)) {
            # Don’t cache result, since it might change suddenly, e.g. due to
            # knitr or Shiny running in the same process.
            return(path)
        }
    }
    throw('unreachable code')
}

#' @return \code{mod_path} returns the script path associated with a \pkg{box}
#' module
#' @rdname path
mod_path = function (mod) {
    tryCatch(dirname(path(mod)), error = function (e) NULL)
}

#' @return \code{explicit_path} returns the script path explicitly set by the
#' user, if such a path was set.
#' @rdname path
explicit_path = function (...) {
    script_path_env$value
}

#' @return \code{r_path} returns the directory in which the current script is
#' run via \command{Rscript}, \command{R CMD BATCH} or \command{R -f}.
#' @rdname path
r_path = function (...) {
    args = commandArgs()
    if (length((file_arg = grep('^--file=', args))) != 0L) {
        unescape_path_arg(dirname(sub('--file=', '', args[file_arg])))
    } else if (length((f_arg = grep('^-f$', args))) != 0L) {
        unescape_path_arg(dirname(args[f_arg + 1L]))
    }
}

unescape_path_arg = if (tolower(Sys.info()[['sysname']]) == 'windows') {
    identity
} else {
    function (path) {
        # Translated from src/unix/system.c:unescape_arg
        # But only unescape spaces, not newlines, since the latter are only escaped
        # when passing code via `-e`, never in file paths.
        gsub('~+~', ' ', path, fixed = TRUE)
    }
}

#' @return \code{knitr_path} returns the directory in which the currently knit
#' document is run, or \code{NULL} if not called from within a \pkg{knitr}
#' document.
#' @rdname path
knitr_path = function (...) {
    if (! 'knitr' %in% loadedNamespaces()) return(NULL)

    knitr_input = suppressWarnings(knitr::current_input(dir = TRUE))
    if (! is.null(knitr_input)) dirname(knitr_input)
}

#' @return \code{shiny_path} returns the directory in which a \pkg{Shiny}
#' application is running, or \code{NULL} if not called from within a
#' \pkg{Shiny} application.
#' @rdname path
shiny_path = function (...) {
    if (! 'shiny' %in% loadedNamespaces()) return()
    in_shiny =
        (utils::packageVersion('shiny') < '1.6.0' && shiny::isRunning()) || {
            # `isRunning` no longer works in Shiny 1.6.0:
            # <https://github.com/rstudio/shiny/issues/3499>
            shiny_ns = getNamespace('shiny')
            any(map_lgl(identical, lapply(sys.frames(), base::topenv), shiny_ns))
        }
    if (in_shiny) getwd()
}

#' @return \code{testthat_path} returns the directory in which \pkg{testthat}
#' code is being executed, or \code{NULL} if not called from within a
#' \pkg{testthat} test case.
#' @rdname path
testthat_path = function (...) {
    if (Sys.getenv('TESTTHAT') %==% 'true') getwd()
}

#' @return \code{rstdio_path} returns the directory in which the currently
#' active RStudio script file is saved.
#' @rdname path
rstudio_path = function (...) {
    # .Platform$GUI is not yet set to "RStudio" during startup, so checking for
    # it *might* be insufficient in corner cases.
    # However, we cannot merely check whether the `RSTUDIO` environment
    # variable is set, because it is also set inside the RStudio terminal; and
    # when ‘box’ is invoked from a script that is run in the terminal, we do
    # *not* want to use RStudio’s active document, since that isn’t the script
    # from which we are called.
    # See also comments at <https://stackoverflow.com/a/35849779/1968>.
    if (.Platform$GUI %!=% 'RStudio') return(NULL)

    document_path = if (requireNamespace('rstudioapi', quietly = TRUE)) {
        rstudioapi::getActiveDocumentContext()$path
    } else {
        # ‘rstudioapi’ might not be installed. Attempt to use the internal API
        # as a fallback.
        tryCatch(
            as.environment('tools:rstudio')$.rs.api.getActiveDocumentContext()$path,
            error = function (.) {
                bug_reports_url = utils::packageDescription(.packageName)$BugReports
                warning(fmt(
                    'It looks like the code is run from inside RStudio but ',
                    '{.packageName;\'} is unable to identify the calling ',
                    'document. This should not happen. Please consider filing ',
                    'a bug report at <{bug_reports_url}>.'
                ))
                ''
            }
        )
    }

    if (document_path %==% '') {
        # The active document wasn’t saved yet, or the code is invoked from the
        # R REPL/console.
        getwd()
    } else {
        dirname(document_path)
    }
}

#' @return \code{wd_path} returns the current working directory.
#' @rdname path
wd_path = function (...) {
    # Fallback
    getwd()
}

path_test_hooks = c('mod', 'explicit', 'knitr', 'shiny', 'testthat', 'rstudio', 'r', 'wd')
path_tests = mget(paste0(path_test_hooks, '_path'))

#' Path related functions
#'
#' \code{mod_search_path} returns the character vector of paths where module
#' code can be located and will be found by \pkg{box}.
#'
#' @note The search paths are ordered from highest to lowest priority.
#' The current module’s path always has the lowest priority.
#'
#' There are two ways of modifying the module search path: by default,
#' \code{getOption('box.path')} specifies the search path as a character vector.
#' Users can override its value by separately setting the environment variable
#' \env{R_BOX_PATH} to one or more paths, separated by the platform’s path
#' separator (\dQuote{:} on UNIX-like systems, \dQuote{;} on Windows).
#' @keywords internal
#' @name paths
mod_search_path = function (caller) {
    env_value = strsplit(Sys.getenv('R_BOX_PATH'), .Platform$path.sep)[[1L]]
    c(
        env_value %||% getOption('box.path'),
        system_mod_path,
        calling_mod_path(caller)
    )
}

#' \code{calling_mod_path} determines the path of the module code that is
#' currently calling into the \pkg{box} package.
#'
#' @param caller the environment from which \code{box::use} was invoked.
#' @return \code{calling_mod_path} the path of the source module that is calling
#' \code{box::use}, or the script’s path if the calling code is not a module.
#' @rdname paths
calling_mod_path = function (caller) {
    calling_ns = mod_topenv(caller)
    # FIXME: Make work for modules imported inside package, if necessary.
    module_path(calling_ns)
}

#' \code{split_path(path)} is a platform independent and filesystem logic
#' aware alternative to \code{strsplit(path, '/')[[1L]]}.
#' @param path the path
#' @return \code{split_path} returns a character vector of path components that
#' logically represent \code{path}.
#' @rdname paths
split_path = function (path) {
    if (path %==% dirname(path)) {
        path
    } else {
        c(Recall(dirname(path)), basename(path))
    }
}

#' \code{merge_path(split_path(path))} is equivalent to \code{path}.
#' @param components character string vector of path components to merge
#' @return \code{merge_path} returns a single character string that is
#' logically equivalent to the \code{path} passed to \code{split_path}.
#' logically represent \code{path}.
#' @note \code{merge_path} is the inverse function to \code{split_path}.
#' However, this does not mean that its result will be identical to the
#' original path. Instead, it is only guaranteed that it will refer to the same
#' logical path given the same working directory.
#' @rdname paths
merge_path = function (components) {
    do.call('file.path', as.list(components))
}

#' \code{sanitize_path_fragment(path)} replaces invalid characters in the given
#' \emph{relative} path, making the result a valid Windows path.
#' @rdname paths
sanitize_path_fragment = function (path) {
    win32_reserved_path_chars = '[<>:"/\\|?*]'
    # Replace invalid chars with `-`, which is unlikely to appear in R names.
    # This isn’t particularly important, but it decreases the risk of name
    # clashes e.g. for path names of the interactive HTML help.
    gsub(win32_reserved_path_chars, '-', path)
}

context('find_mod')

test_use = function (...) {
    call = match.call()
    parse_spec(call[[2L]], names(call[-1L]))
}

test_that('"./" can only be used as a prefix', {
    expect_error(test_use(./a), NA)
    expect_error(test_use(.././a))
    expect_error(test_use(a/./b))
    expect_error(test_use(a/.))
    expect_error(test_use(a/b/./c))
})

test_that('"../" can only be used as a prefix', {
    expect_error(test_use(../a), NA)
    expect_error(test_use(../../a), NA)
    expect_error(test_use(./../a))
    expect_error(test_use(a/../b))
    expect_error(test_use(../a/../b))
    expect_error(test_use(a/..))
    expect_error(test_use(a/b/../c))
})

test_that('local path is searched globally', {
    old_opts = options(box.path = NULL)
    on.exit(options(old_opts))

    path = utils::tail(mod_search_path(environment()), 1L)
    expect_paths_equal(path, getwd())
})

test_that('local path is searched in module', {
    old_opts = options(box.path = NULL)
    on.exit(options(old_opts))

    box::use(rel = mod/nested/rel_import)
    nested_path = file.path(getwd(), 'mod', 'nested')

    expect_paths_equal(rel$global_path, nested_path)
    expect_paths_equal(rel$path_in_fun(), nested_path)
    expect_paths_equal(rel$path_in_nested_fun(), nested_path)
})

test_that('all module file candidates are found', {
    # See <https://github.com/klmr/box/issues/174>
    spec = parse_spec(quote(a/b), '')
    paths = c('x', 'y')
    candidates = mod_file_candidates(spec, paths)
    expected = c(
        'x/a/b.r',
        'x/a/b.R',
        'x/a/b/__init__.r',
        'x/a/b/__init__.R',
        'y/a/b.r',
        'y/a/b.R',
        'y/a/b/__init__.r',
        'y/a/b/__init__.R'
    )
    expect_setequal(unlist(candidates), expected)
})

test_that('script path can be set manually', {
    on.exit(box::set_script_path())

    expect_paths_equal(module_path(), getwd())

    box::set_script_path('mod/b/a.r')
    expect_equal(module_path(), 'mod/b')
})

test_that('script path can be queried', {
    path = 'some/script.r'
    box::set_script_path(path)
    expect_equal(box::script_path(), path)
    expect_equal(box::set_script_path(), path)
    expect_null(box::script_path())
})

test_that('can execute a script with spaces in path', {
    # Generate the test case dynamically since `R CMD check` complains if there
    # are paths with spaces in the package source directory.
    path = 'support/path with spaces'
    dir.create(path)
    on.exit(unlink(path, recursive = TRUE))
    writeLines(c(
        '.on_load = function (ns) cat("path with spaces\\n")',
        'box::export()'
    ), file.path(path, 'a.r'))
    writeLines('box::use(./a)', file.path(path, 'script.r'))

    rscript_out = rscript(file.path(path, 'script.r'))
    expect_equal(rscript_out, 'path with spaces')
})

test_that('modules are found during Shiny startup', {
    script_path = rscript('support/run-shiny.r')
    expect_paths_equal(script_path, 'support/shiny-app')
})

#' Compute normalized logical paths
#'
#' \code{realpath(path)} will return the normalized logical path for
#' \code{path}, similar to \code{normalizePath} but working correctly for
#' nonexistent paths on Unix systems.
#' @param path a character vector of paths
#' @note This function doesn’t work with paths containing newlines; this is a
#' restriction of R because it truncates nul characters and thus cannot pass
#' the \code{-z} flag to the \code{realpath} shell function.
realpath = function (path) {
    if (.Platform$OS.type == 'unix') {
        normalize_path(path)
    } else {
        sub('\\\\$', '', normalizePath(path, mustWork = FALSE))
    }
}

#' Replacement for \code{normalizePath} that works with non-existent paths on
#' Unix
#'
#' @param path vector of paths to normalize.
#' @details
#' \code{normalize_path} works like \code{\link[base]{normalizePath}}, but also
#' works on non-existent paths. \code{\link[base]{normalizePath}} has an option
#' \code{mustWork = FALSE} for this; however, that option only works on Windows.
#' On Unix systems, it causes paths to be left untouched. However, and as a
#' consequence, \code{normalize_path} does \emph{not} resolve symlinks.
normalize_path = function (path) {
    path = path.expand(path)

    # Make absolute.
    relative_paths = grep('^/', path, invert = TRUE)
    path[relative_paths] = file.path(getwd(), path[relative_paths])

    # In the following, we have to be careful not to remove `.` or `..` when
    # they are part of a path component name, e.g. `.bashrc`.

    path = gsub('(?<=/)\\.(/|$)', '', path, perl = TRUE)

    # The following needs to be in a loop rather than a global search to
    # correctly handle subsequent occurrences of `..`, e.g. `foo/../..`.
    while (any(grepl('/\\.{2}(/|$)', path))) {
        # Remove a single `..`
        path = sub('/([^/]|\\\\/)+/\\.{2}(/|$)', '/', path)
    }

    # Collapse slashes
    path = gsub('//+', '/', path)

    # Remove trailing slashes
    sub('(.+)/$', '\\1', path)
}

expect_paths_equal = function (actual, expected) {
    actual_norm = realpath(box:::merge_path(actual))
    expected_norm = realpath(expected)
    testthat::expect_equal(
        actual_norm, expected_norm,
        label = deparse(substitute(actual)),
        expected.label = deparse(substitute(expected))
    )
}

expect_correct_path_split = function (actual) {
    expect_paths_equal(box:::split_path(actual), actual)
}

expect_is_cwd = function (actual) {
    eval.parent(bquote(expect_paths_equal(.(actual), '.')))
}

context('knitr')

check_knitr = function () skip_if_not_installed('knitr')

test_that('modules are found when knitr is not loaded', {
    check_knitr()
    # Ensure knitr isn’t loaded
    unloadNamespace('knitr')
    expect_paths_equal(module_path(), getwd())
})

test_that('modules are found when knitr is loaded', {
    check_knitr()
    loadNamespace('knitr')
    on.exit(unloadNamespace('knitr'))
    expect_paths_equal(module_path(), getwd())
})

test_that('modules are found inside a knitr document', {
    check_knitr()
    on.exit(unloadNamespace('knitr'))

    input = 'support/knitr/doc.rmd'
    # Ensure that a different working directory is used.
    knitr::opts_knit$set(root.dir = getwd())
    output = knitr::knit(input, quiet = TRUE)
    on.exit(unlink(output), add = TRUE)

    expected = '```\n## knitr/a\n```'
    expect_match(paste(readLines(output), collapse = '\n'), expected)
})

# FIXME: Make the following more robust, and make it work on Windows.
# This would probably require something like `withr::local_envvar`, deferred to
# the caller, which is incredibly tedious to reimplement (requires native code).

r_cmdline = function (cmd, ...) {
    skip_on_os('windows')
    # Some of the script tests fail on CRAN and, lacking stack traces, it’s
    # simply impossible to debug this. So we disable them.
    skip_on_cran()

    rprofile = normalizePath(test_path('support/rprofile.r'))
    args = c('--no-save', '--no-restore', ...)
    # Unset `TESTTHAT` since otherwise ‘box’ thinks that tests are being run
    # from inside ‘testthat’, and fudges the local search path.
    sprintf(
        'TESTTHAT= R_ORIGINAL_PROFILE_USER="$R_PROFILE_USER" R_PROFILE_USER=%s %s %s',
        shQuote(rprofile),
        shQuote(file.path(R.home('bin'), cmd)),
        paste(shQuote(args), collapse = ' ')
    )
}

rcmd = function (script_path) {
    output_file = tempfile(fileext = '.rout')
    on.exit(unlink(output_file))
    cmd = r_cmdline(
        'R', 'CMD', 'BATCH',
        '--slave',
        '--no-timing',
        script_path,
        output_file
    )
    system(cmd)
    readLines(output_file)
}

rscript = function (script_path, wd = NULL) {
    cmd = r_cmdline('Rscript', '--slave', script_path)
    if (! is.null(wd)) {
        old_wd = setwd(wd)
        on.exit(setwd(old_wd))
    }
    p = pipe(cmd)
    on.exit(close(p), add = TRUE)
    readLines(p)
}

interactive_r = function (script_path, text, code) {
    cmd = r_cmdline('R', '--interactive')
    output_file = tempfile(fileext = '.rout')
    on.exit(unlink(output_file))

    text = if (! missing(script_path)) {
        readLines(script_path)
    } else if (! missing(code)) {
        deparse(substitute(code), backtick = TRUE)
    } else if (! missing(text)) {
        text
    } else {
        stop('Missing argument')
    }

    local({
        p = pipe(paste(cmd, '>', output_file), 'w')
        on.exit(close(p))
        writeLines(text, p)
        writeLines('interactive()', p)
    })

    result = readLines(output_file)

    strip_ansi_escapes = function (str) {
        # Only support CSI Select Graphic Rendition for now. This is necessary
        # to guard against R packages such as ‹colorout›.
        gsub('\033\\[(\\d+(;\\d+)*)?m', '', str)
    }

    check_line = function (which, expected) {
        # Separate check to generate only one assertion, and only if needed.
        if (! identical(strip_ansi_escapes(result[which]), expected)) {
            expect_identical(
                strip_ansi_escapes(result[which]), expected,
                label = sprintf('"%s"', result[which]),
                info = 'interactive_r'
            )
        }
    }

    # Ensure that code was actually run interactively.
    end = length(result)
    check_line(end - 2L, '> interactive()')
    check_line(end - 1L, '[1] TRUE')
    check_line(end, '> ')
    result[1 : (end - 3L)]
}

local({
    in_tests = grepl('tests/testthat$', getwd())
    basedir = if (in_tests) dirname(dirname((getwd()))) else getwd()
    Sys.setenv(BOX_TESTING_BASEDIR = basedir)
})

clear_mods = function () {
    rm(list = names(box:::loaded_mods), envir = box:::loaded_mods)
}

.setup_fun = NULL
.teardown_fun = NULL

context = function (desc) {
    assign('.setup_fun', NULL, envir = parent.env(environment()))
    assign('teardown_funs', NULL, envir = parent.env(environment()))
    testthat::context(desc)
}

test_setup = function (code, env = parent.frame()) {
    assign(
        '.setup_fun',
        rlang::new_function(list(), rlang::enexpr(code), env = env),
        envir = parent.env(environment())
    )
}

test_teardown = function (code, env = parent.frame()) {
    assign(
        '.teardown_fun',
        rlang::new_function(list(), rlang::enexpr(code), env = env),
        envir = parent.env(environment())
    )
}

run_setup = function () {
    if (! is.null(.setup_fun)) {
        # Note that bquote(.(f)()) does NOT perform substitution for whatever
        # reason.
        eval.parent(substitute(f(), list(f = .setup_fun)))
    }
}

run_teardown = function () {
    if (! is.null(.teardown_fun)) {
        # Note that bquote(.(f)()) does NOT perform substitution for whatever
        # reason.
        eval.parent(substitute(f(), list(f = .teardown_fun)))
    }
}

test_that = function (desc, code) {
    eval.parent(bquote({
        run_setup()
        on.exit(run_teardown())
        testthat::test_that(.(desc), .(substitute(code)))
    }))
}

context('vignettes')

expect_snapshot_vignette = function (out_dir, vignette) {
    name = paste0(tools::file_path_sans_ext(vignette), '.md')
    output_path = file.path(out_dir, name)

    announce_snapshot_file(name = name)
    expect_snapshot_file(output_path, name, compare = compare_file_vignette)
}

compare_file_vignette = function (old, new) {
    old = readLines(old)
    new = readLines(new)

    mask_env_ptrs = function (x) {
        gsub('## <environment: 0x[0-9a-f]+>', '## <environment: X>', x)
    }

    # Mask out non-deterministic values in code output, in particular pointer
    # values in the output of R environments.
    identical(mask_env_ptrs(old), mask_env_ptrs(new))
}

test_that('vignettes don’t change', {
    local_edition(3L)
    pkgdir = '../..'
    out_dir = file.path(pkgdir, 'doc')
    vignettes = dir(file.path(pkgdir, 'vignettes'), pattern = '\\.rmd$')

    skip_on_cran()
    skip_on_ci()
    skip_if(! dir.exists(out_dir))

    rendered_vignettes = dir(out_dir, pattern = '\\.md$')
    missing_vignettes = setdiff(vignettes, sub('\\.md$', '.rmd', rendered_vignettes))
    if (length(missing_vignettes) > 0L) {
        fail(paste('Rendered vignettes for', toString(missing_vignettes), 'were missing. Run `make knit-all`!'))
        return()
    }

    for (vignette in vignettes) {
        expect_snapshot_vignette(out_dir, vignette)
    }
})

---
title: "Using compiled code"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Using compiled code}
    %\VignetteEncoding{UTF-8}
---
## Current status

The ‘box’ package doesn’t have a built-in foreign function interface yet
but it is possible to integrate compiled code via R’s `SHLIB` mechanism
for building shared libraries. In particular, this also works with
packages such as [Rcpp](https://cran.r-project.org/package=Rcpp).

For the time being, the following steps unfortunately require manual
work:

1.  Compile code when a module is installed or loaded for the first
    time.
2.  Load the compiled shared library when a module is loaded.
3.  Interface with compiled code via function calls.

This guide aims to describe all steps in sufficient detail to make them
easy. In the long run, the plan is to automate all these steps.

## Example

To demonstrate these steps, we will use an example module named `c` that
uses compiled code written in C. Here is the C code that we want to make
usable in an R module, which is saved in the file `c/hello.c`:

``` c
#include "R.h"
#include "Rdefines.h"

#include <stdio.h>

SEXP hello_world(SEXP name) {
    char const msg_template[] = "Hello from C, %s!";
    char const *const c_name = CHAR(asChar(name));
    char *const msg_buf = R_alloc(sizeof msg_template - 2 + strlen(c_name) + 1, 1);
    sprintf(msg_buf, msg_template, c_name);
    return mkString(msg_buf);
}
```

In addition, compiled code often includes specific compilation
instructions. While this is unnecessary for this simple example, it’s
included anyway for completeness. For R, these compilation instructions
are contained in a file called
[`Makevars`](https://cran.r-project.org/doc/manuals/r-release/R-exts.html#Using-Makevars):

``` c/makevars
PKG_CFLAGS = -pedantic -Wall -Wextra -Werror
```

## Compile code

To make code loadable and callable by R, it should be compiled via the
`R CMD SHLIB` mechanism. This only needs to happen *once* for each
module, when it is first loaded. By convention, such code should go into
a submodule called `__setup__`. This convention makes it clear that this
is a “special” module, and not intended for direct consumption of the
module user.

The module specifies which object files to compile, and invokes
`R CMD SHLIB`:

```{r eval = FALSE}
build_shared_lib = function (object_names) {
    # Change working directory so R finds the Makevars.
    old_dir = setwd(box::file())
    on.exit(setwd(old_dir))
    rbin = file.path(R.home('bin'), 'R')
    exitcode = system2(rbin, c('CMD', 'SHLIB', paste0(object_names, '.c')))
    stopifnot(exitcode == 0L)
}

build_shared_lib('hello')
```

In principle, only the last line in this file should need to be changed
for other C projects.

With this in place, we can invoke the compilation by loading the
`c/__setup__` submodule:

```{r eval = FALSE}
box::use(./c/`__setup__`)
```

The result of the compilation will be a single file, `hello.so` (on Unix
and macOS) or `hello.dll` (on Windows) which represents a *shared
library file*, and which we can load and use inside R.

## Loading compiled code

Compiled code from a shared library is loaded in R using the `dyn.load`
command. This will happen inside the module that uses and/or exposes the
compiled code.

Since the name of the shared library file is platform dependent, we need
a helper function that gives us this name:

```{r eval = FALSE}
libname = function (name) {
    box::file(paste0(name, .Platform$dynlib.ext))
}
```

Now our module can load the compiled code; since this code needs to be
executed every time the module is loaded, it goes into the `.on_load`
hook:

```{r eval = FALSE}
.on_load = function (ns) {
    ns$dll = dyn.load(libname('hello'))
}
```

… and don’t forget to unload the dynamic library when the module is
unloaded:

```{r eval = FALSE}
.on_unload = function (ns) {
    dyn.unload(libname('hello'))
}
```

## Interacting with compiled code

Finally, our module needs a way of calling the compiled code. This is
done via the R primitive `.Call`:

```{r eval = FALSE}
#' @export
hello_world = function (name) {
    .Call(dll$hello_world, name)
}
```

To use the code, we load the module and call the `hello_world` function:

```{r eval = FALSE}
box::use(./c)
c$hello_world('Rthur')
```

    ## [1] "Hello from C, Rthur!"

Note that using `dll$hello_world` causes a somewhat costly call to
`getNativeSymbolInfo` every time the function is invoked. If this is
undesired, the value of `dll$hello_world` should be stored in a variable
when loading the module.

---
title: "Migration guide"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Migration guide}
    %\VignetteEncoding{UTF-8}
---
## ‘modules’ “v1.0” == ‘box’

‘box’ is the spiritual successor of the [‘modules’
package](https://github.com/klmr/modules), which will forever remain at
version 0.9.*x*. However, the API of ‘box’ intentionally breaks
backwards compatibility, and module code written for use with ‘modules’
will no longer work with ‘box’.

The following guide is intended to ease migration to ‘box’. **Migration
is strongly recommended**, since the authors believe that ‘modules’ had
conceptual shortcomings that have been fixed in ‘box’, and to keep the
ecosystem consolidated. For the time being, ‘modules’ will receive
support (only) in the form of critical bug fixes.

## Importing modules and packages

### General syntax

With the ‘modules’ package, modules and packages were imported via

```{r eval = FALSE}
library(modules)

# …

modname = import('prefix/modname')
pkgname = import_package('pkgname')
```

With ‘box’, modules and packages are imported via
[`box::use`](https://klmr.me/box/reference/use.html):

```{r eval = FALSE}
box::use(prefix/modname)
box::use(pkgname)
```

Notably, module and package names in a use declaration in ‘box’ are
unquoted, unevaluated expressions; and **the ‘box’ package is never
loaded via `library`**. In fact, `library(box)` raises an error.

Furthermore, with ‘box’, module names *must* be fully qualified. The
equivalent of the ‘modules’ code `mod = import('modname')` no longer
exists. To import a *local* module without namespace prefix, instead use

```{r eval = FALSE}
box::use(./modname)
```

`box::use` has no return value. Instead, it automatically creates an
alias with the module/package name in the calling scope *if no names are
attached* (see below). To override the alias name, specify it as as a
named argument:

```{r eval = FALSE}
box::use(mod_alias = prefix/modname)
```

Unlike with `modules::import`, `box::use` allows (and encourages)
multiple use declarations at once:

```{r eval = FALSE}
box::use(
    prefix/mod1,
    mod = prefix/mod2,
    pkg
)
```

### Attaching names

In ‘modules’, the `attach` and `attach_operators` parameters controlled
if and which names were attached. In ‘box’, attachment is controlled via
attach list declarations:

```{r eval = FALSE}
# ‘modules’:
import('prefix/mod', attach = c('name1', 'name2'))

# ‘box’:
box::use(prefix/mod[name1, name2])
```

Wildcard attach lists are also supported:

```{r eval = FALSE}
# ‘modules’:
import('prefix/mod', attach = TRUE)

# ‘box’:
box::use(prefix/mod[...])
```

To introduce a module/package alias when attaching names, specify an
alias name:

```{r eval = FALSE}
# ‘modules’:
mod = import('prefix/mod', attach = c('name1', 'name2'))

# ‘box’:
box::use(mod = prefix/mod[name1, name2])
```

‘box’ also allows declaring aliases for attached names; this feature did
not exist in ‘modules’:

```{r eval = FALSE}
# Declare alias for one name, attach other name unchanged:
box::use(prefix/mod[name_alias = name1, name2])

# Declare alias for two names, attach all other exported names unchanged:
box::use(prefix/mod[name_alias1 = name1, name_alias2 = name2, ...])
```

The `attach_operators` option from ‘modules’ has been dropped. If users
require operators, they need to explicitly attach them when using ‘box’.

### Loading documentation

The `doc` option from `modules::import` has been dropped without
replacement. ‘box’ loads documentation lazily only when it is requested
via `box::help` (see below).

## Executing code during module loading

The ‘modules’ package treated modules as regular R source code files:
upon importing them, the entire code inside a module file was executed.
‘box’ conceptually no longer does this. It regards module source code as
*declarative*: the module source code defines a number of names to be
exported. However, code with side-effects on the module file level is no
longer guaranteed to execute.

Instead, ‘box’ introduces [module event
hooks](https://klmr.me/box/reference/mod-hooks.html), in particular
`.on_load`, which is a function that gets executed whenever a module is
first loaded inside an R session.

## Changed options

In ‘modules’, the module search path was set via
`options('import.path')`. In ‘box’, use `options('box.path')` instead.

The ‘modules’ options `options('import.attach')` and
`options('warn.conflicts')` no longer exist. In particular, ‘box’ no
longer warns of name conflicts when attaching names. Instead, it
encourages consciously choosing which names to attach.

## Changed function names

The ‘modules’ function `module_file` is now called
[`box::file`](https://klmr.me/box/reference/file.html). Its semantics
have also changed: it no longer cares whether files relative to the
module exist or not; it merely constructs appropriate path strings.

The ‘module’ function `module_name` is now called
[`box::name`](https://klmr.me/box/reference/name.html). It no longer has
any arguments.

## Exporting names from modules

With the ‘modules’ packages, module source files exported all non-hidden
names; that is, all names that didn’t start with a dot (`.`).

‘box’ makes exporting explicit. By default, *no* names are exported from
a module, unless they are marked with the directive comment
`#' @export`:

```{r eval = FALSE}
# This function is not exported:
f1 = function () {}

# This nested module is not exported
box::use(./nested1)

# This function is exported:
#' @export
f2 = function () {}

# This nested module is exported
#' @export
box::use(./nested2)
```

`@export` directives that decorate `box::use` declarations apply to
*all* names declared in it:

```{r eval = FALSE}
#' @export
box::use(
    pkg_alias = pkg,
    prefix/mod,
    prefix/mod2[a, b, c]
)
```

The above code will export the names `pkg_alias`, `mod`, `a`, `b` and
`c`. This replaces the function `export_submodule` from ‘modules’, which
no longer exists.

There’s one exception to this: if a module contains *no* declared
export, ‘box’ assumes that it is a plain R script, and treats it as a
*legacy module*. This causes ‘box’ to revert to the export behaviour of
‘modules’. To suppress this behaviour (that is, to create a module which
explicitly doesn’t export any names), module authors can add the
following declaration to their module source code, which explicitly
declares that the source file is to be treated as a module with no
exports:

```{r eval = FALSE}
box::export()
```

## Accessing default packages

In ‘modules’, module source code “sees” the package search path at the
time it is loaded. That is, modules have implicit access to all names in
packages that were attached when the module was loaded. In particular,
this means that legacy modules could access names in the default
packages (in non-interactive sessions, that’s typically ‘datasets’,
‘utils’, ‘grDevices’, ‘graphics’ and ‘stats’; in interactive sessions,
the package ‘methods’ is added).

‘box’ modules no longer attach *any* packages by default, except ‘base’.
If a module needs to use functions from those other packages, it needs
to declare them explicitly (e.g. via `box::use(stats)`). If users want
to import all default R packages, they can import the module `r/core`
for this purpose. The following declaration approximates the behaviour
of ‘modules’:

```{r eval = FALSE}
box::use(r/core[...])
```

## Loading parents of nested modules

Consider the following file hierarchy defining a nested module:

<pre><code><b>a</b>
├─ <i>__init__.r</i>
╰─ <b>b</b>
   ├─ <i>__init__.r</i>
   ╰─ <i><b>c</b>.r</i></code></pre>

In ‘modules’, the declaration `import(a/b/c)` would import the module
`c`, but this would first execute the code of the modules `a` and `a/b`.
In other words it would source the files `a/__init__.r`,
`a/b/__init__.r`, and `a/b/c.r`, in this order.

‘box’ no longer loads the full module hierarchy: `box::use(a/b/c)` loads
only the module defined by `a/b/c.r`. Likewise, `box::use(a/b)` does
*not* automatically load `a/b/c.r` (but the same behaviour was already
present in ‘modules’).

## Displaying documentation

‘modules’ overrode the `help` function and the `?` operator to allow
displaying module documentation. Since ‘box’ is no longer attached,
these functions no longer display module documentation. Instead, the
documentation of anything imported via ‘box’ (both modules and
packages!) can be queried via
[`box::help`](https://klmr.me/box/reference/help.html).

Unlike ‘modules’, ‘box’ also supports displaying the documentation of
nested module names, e.g. `box::help(a$b$c)`.

## Cyclic imports

Cyclic/circular imports are supported by both ‘modules’ and ‘box’.
However, the level of support differs. The details are complicated, and
it is generally recommended to avoid cyclic modules. However, there are
some situations where circularity in the dependencies makes sense, and
‘box’ strives to make this work, where technically possible.

For instance, consider the following (nonsensical) working, circular
definition of the functions `even` and `odd`, which determine whether a
non-negative integer is even or odd:

- `even.r`:

  ``` r
  box::use(./odd[...])

  #' @export
  even = function (n) {
      n == 0L || ! odd(n)
  }
  ```

- `odd.r`:

  ``` r
  box::use(./even[...])

  #' @export
  odd = function (n) {
      n != 0L && even(n - 1L)
  }
  ```

These modules compute `even` in terms of `odd`, and vice-versa. Yet
‘box’ has no trouble importing and using both these modules. However,
this no longer works once we attempt to export imported submodules
themselves. That is, the following version of `odd.r` would cause an
error:

- `odd.r`:

  ``` r
  #' @export
  box::use(./even[...])

  #' @export
  odd = function (n) {
      n != 0L && even(n - 1L)
  }
  ```

---
title: "Module environments"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Module environments}
    %\VignetteEncoding{UTF-8}
---
<div class="dev-note">

> **Note:** This document describes internal implementation details.
> They are not required for users of the ‘box’ package and module
> authors.

</div>

## Preliminaries

To ensure that module environments are properly isolates from each
other, and to enable fine-grained control over imports and exports,
loaded modules (as well as packages loaded via `box::use`) correspond to
a mesh of several interconnected environments.

Unfortunately the exact relationship between these environments isn’t
trivial, so the following document aims at explaining the gist.

Fundamentally, module environments follow a similar architecture to
package environments, but they deviate in crucial ways. For a good
explanation of package environments, see [*How R Searches and Finds
Stuff*](https://blog.thatbuthow.com/how-r-searches-and-finds-stuff/) by
Suraj Gupta, and
[*Environments*](http://adv-r.had.co.nz/Environments.html) in *Advanced
R* by Hadley Wickham. The following assumes that the reader has a more
than passing familiarity with these concepts.

When speaking of “package environments” above and in the following, this
refers to packages as handled by base R and the R package namespace
functionality. For packages that are loaded by `box::use`, the rules are
the same as for modules.

Let’s now consider an example of several loaded modules, and how they
interact.

## An example

This example will use three modules — the minimum number necessary to
show some of the interactions of the module environments. Let’s define
these three modules. We’ll do so in inverse order of their usage:

### <code><span style="background: #1DB100; color: white; padding: 0 5px; border-radius: 2px;">c.r</span></code>

```{r eval = FALSE}
#' @export
f = function () 'c$f'
```

Module `c` defines and exports one name, the function `f`.

### <code><span style="background: #FF9300; color: white; padding: 0 5px; border-radius: 2px;">b.r</span></code>

```{r eval = FALSE}
#' @export
f = function () 'b$f'

g = function () 'b$g'
```

Module `b` defines and exports the function `f`, and in addition defines
the function `g`.

### <code><span style="background: #00A2FF; color: white; padding: 0 5px; border-radius: 2px;">a.r</span></code>

```{r eval = FALSE}
#' @export
box::use(./b[g = f, ...])

box::use(./c[...])

#' @export
box::use(./c)

#' @export
f = function () 'a$f'

f_of_c1 = c$f
f_of_c2 = get('f', parent.env(environment()))
stopifnot(identical(f_of_c1, f_of_c2))
```

Module `a` imports and re-exports all names from `b` (changing the name
of `b$f` to `g`); it also imports, but does not re-export, all names
from `c` (and, additionally, defines an alias for the module, which it
exports).

It further defines and exports the function `f` and defines, but does
not export, the functions `f_of_c1` and `f_of_c2` (which are defined
such that they are identical to each other — hopefully it will become
clear why later).

Finally, let’s use the modules by executing the following code:

```{r eval = FALSE}
box::use(a = ./a[f, g])
```

Now the name `a` is defined in `.GlobalEnv` and refers to the module
environment of module `a`. The exported names `f` and `g` of `a` are
also available in `.GlobalEnv` (but not defined *inside* `.GlobalEnv`).

The loaded modules can be represented by the following schematic, with
each box corresponding to an environment:

![](environment-schema.png)

Let’s go over the different types of environment associated with each
module, and how they are connected.

## Environments

### Module namespace

Modules are loaded into their own dedicated environment, the *module
namespace*. Every name that is defined by a module is defined inside it.
It is thus also the enclosing environment of the module’s functions.
Lastly, the module namespace also stores meta-information about the
module in a hidden member named `.__module__.`.

This corresponds to the package namespace.

### Module imports environment

The parent environment of the module namespace is the imports
environment, which contains all the names that a module imports via
attachment declarations in `box::use` expressions.

The parent environment of the imports environment is the R `base`
namespace environment. The module imports environment thus corresponds
to the package imports environment.

### Module export environment

The module export environment, also called just “module environment” in
the code base, contains all names that are marked as exported by a
module. If users create a module alias in their `box::use` call, the
alias will be a reference to this environment.

Similarly, attached names are selected as a subset from the module
export environment (then copied into the imports environment).

There is no direct equivalent to this environment in R packages, because
R packages do not distinguish between exported names and names imported
by the code that loaded the package. By contrast, ‘box’ needs to
distinguish the set of exports from the set of names that are imported
(= “attached”) by client code.

### Importing into other environments

Module import environments store imports from one or more module into
another. However, modules can also be imported from inside the global
environment, or from inside functions, which have their own local
execution environment (the stack frame). When this happens, a new
imports environment is created on the fly, and attached to the importing
environment’s `parent.env` chain. Calling `box::use` with a non-empty
attach list in `.GlobalEnv` is therefore similar to calling `library`.
In particular, inside `.GlobalEnv` the call `box::use(pkg[...])` in
pretty much equivalent to calling `library(pkg)`, and is discouraged for
the same reason that `library(pkg)` is discouraged.

---
title: "Contributing to ‘box’"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Contributing to ‘box’}
    %\VignetteEncoding{UTF-8}
---
# Installation

Compared to many other R packages, ‘box’ uses a somewhat different
development workflow:

While ‘box’ uses ‘devtools’ and ‘roxygen2’ to generate documentation and
package infrastructure, *generated code and data is not checked into
version control!* This means in particular that the project does not
contain a `NAMESPACE` file, since that is auto-(re)generated by tools.
Therefore, attempting to install the current development version of
‘box’ via `pak::pak('klmr/box')` or equivalent means *will fail!*

Instead, an up-to-date, automatically generated build of the development
version of ‘box’ should be installed from
[R-Universe](https://klmr.r-universe.dev/):

```{r eval = FALSE}
install.packages('box', repos = 'https://klmr.r-universe.dev')
```

… or from the corresponding `build` branch:

```{r eval = FALSE}
pak::pak('klmr/box@build')
```

Alternatively it can be built manually using the instructions below.

# Development

## Building

The project contains a `Makefile` written in the [GNU
Make](https://www.gnu.org/software/make/manual/make.html) dialect that
contains various development utilities. Invoking `make` without target
will show a list of available targets with short descriptions. Using
this `Makefile` isn’t *necessary*, but it helps. In particular:

- `make documentation` builds the `NAMESPACE` file, the shared C library
  and the package documentation.
- `make test` runs the unit tests.
- `make check` runs checks, and should run cleanly before submitting a
  pull request: of note, `make check` performs additional checks that
  are not performed by either `R CMD check` or `rcmdcheck::rcmdcheck()`
  (these checks can be found as individual scripts under `scripts/`;
  they roughly correspond to some undocumented checks performed
  internally on CRAN).

## Branches

All new code should be developed on a new branch with a name prefixed
`fix/` (for bug fixes), `feature/` (for new features/enhancements), and
`chore/` (for any other contributions: fixed typos, project
infrastructure, tests, etc.). Branches for pull requests will be merged
into the [`main` branch](https://github.com/klmr/box/tree/main).

## Code style

The code style of ‘box’ is similar to the [Tidyverse style
guide](https://style.tidyverse.org/), but with several notable
differences:

- The file extension for R code files is lowercase `.r` (not uppercase
  `.R`). The file extension for R Markdown files is lowercase `.rmd`
  (not uppercase `.Rmd`).

  <div class="rationale">

  **Rationale:** using capital letters in file extensions is a pointless
  violation of established convention that creates unnecessary
  inconsistencies.

  </div>

- Use `=` for assignment, not `<-`.

  - On the (very rare!) occasions where assignment inside a function
    call is required, use additional parentheses to make `=`
    syntactically an assignment rather than named argument passing.

    ``` r
    # GOOD:
    x = 5

    # OK (rarely):
    if (is.null((x = function_call()))) {}

    # BAD:
    x <- 5
    if (is.null(x <- function_call())) {}
    ```

  - `<<-` is banned. Instead of `name <<- value`, use
    `env$name = value`, where `env` is a previously-defined name
    referring to the desired target environment. This pattern is used in
    various places in the code, and good, context-dependent names for
    `env` are `ns`, `self`, or similar. `assign()` should generally not
    be used when the name of the assignee is statically known.

    ``` r
    # GOOD:
    self = parent.frame()
    # … later, inside a nested closure:
    self$x = TRUE

    # BAD:
    x <<- TRUE

    # BAD:
    assign('x', TRUE, envir = caller)
    ```

    <div class="rationale">

    **Rationale:** `<<-` leaves it unclear where assignment will happen.
    Subset-assignment via `$` makes the assignment target scope
    explicit, which makes the code clearer and less error-prone.

    </div>

- Leave a space between `function` and the following opening
  parenthesis.

  ``` r
  # GOOD:
  f = function () {}

  # BAD:
  f = function() {}
  ```

  <div class="rationale">

  **Rationale:** a function declaration in R does not use function call
  syntax; rather, it’s a special syntactic construct equivalent to `if`
  and `while`, so the same formatting conventions apply.

  </div>

- Use single quotes, not double quotes, around strings. — Even when the
  string contains `'`, which should be escaped.

  ``` r
  # GOOD:
  'text'
  'text with \'quotes\''

  # BAD:
  "text"
  "text with 'quotes'"
  r'(text with 'quotes')'
  ```

  <div class="rationale">

  **Rationale:** raw strings cannot be used since ‘box’ supports R
  versions pre R 4.0.

  </div>

- Use four spaces for indentation. Do *not* add extra spaces to align
  assignments or named function call arguments.

  ``` r
  # GOOD:
  first = 1
  second = 2

  # BAD:
  first  = 1
  second = 2
  ```

- The line length is hard-limited to 120 columns. *Most* lines should be
  shorter, but there is no (soft, or otherwise) limit at 80 columns.

- Explicitly use integer literals where the logical type of the
  expression is integer.

  ``` r
  # GOOD:
  if (length(x) == 0L) {}

  # BAD:
  if (length(x) == 0) {}
  ```

---
title: "Frequently asked questions"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Frequently asked questions}
    %\VignetteEncoding{UTF-8}
---
## General

### Can I call `library`/`require`/`source` inside a module?

No, this usually won’t work: `library` (and `require`) attaches names
*globally*, rather than inside the calling module. Inside a module,
`box::use` must be used instead.

Likewise, by default `source` evaluates the source code in the global
rather than in a local environment. This behaviour can be changed by
setting its `local` argument to a suitable value.

Calling these functions inside a module will raise a warning which can
be silenced by setting `options(box.warn.legacy = FALSE)`. After
changing this option, ‘box’ needs to be (unloaded and) reloaded for the
change to take effect.

### Can I use “meta packages” like ‘tidyverse’ with ‘box’?

No. You can use *individual* packages from the tidyverse via `box::use`.
But directly loading a “meta package” such as ‘tidyverse’ is
intentionally not supported.

“Meta packages” such as ‘tidyverse’ are antithetical to the purpose of
‘box’: the whole point of ‘box’ is to make package imports *explicit*,
and to limit the number of names that are attached in the current scope.
Using “meta packages” would undermine this and is therefore discouraged.

### How to organise globally installed modules?

Module names need to be *fully qualified*, meaning that using them
requires providing a name consisting of a prefix (at least one
high-level namespace) and the module’s proper name. The choice of prefix
is, to some extent, arbitrary. However, there are some common
conventions that are worth following.

For example, it is common practice to use a company or user name (look
at code sharing websites such as GitHub for inspiration). A project
consisting of several nested modules might also serve the same purpose.
For example, if ‘dplyr’ were implemented as a module, its common fully
qualified name would probably be `tidyverse/dplyr`.

As a concrete example, consider the following modules, stored in the
module search path:

    ‹box.path›
    ├── klmr
    │   ├── fun
    │   ├── ggplots
    │   └── sys
    ├── mschubert
    │   └── ebits
    └── …

Using the `sys` module (more precisely, `klmr/sys`) requires the
following R code:

```{r eval = FALSE}
box::use(klmr/sys)
```

## Common error messages

### “object ‘X’ not found” inside a module

Inside modules, only the ‘base’ package is attached. *All* other
packages need to be attached, including all core R packages that are
otherwise attached by default. This includes ‘stats’, ‘graphics’,
‘grDevices’, ‘utils’, ‘datasets’ and ‘methods’.

To use names from any of these packages, the packages need to be loaded
via `box::use`, same as third-party packages. Alternatively, the
standard module `r/core` can be used (and, optionally, attached):

```{r eval = FALSE}
box::use(r/core[...])
```

This is equivalent to using and attaching the default R packages
‘methods’, ‘stats’, ‘graphics’, ‘grDevices’ and ‘utils’, and is
therefore similar to the state of a regular R session.

This module ships with ‘box’ and is always findable.

### “there is no package called ‘X’” when running `box::use(X)`, even though the module ‘X’ exists

`box::use(name)` never attempts to load a module, it always attempts to
load a *package* called ‘name’, and fails if no such package exists.
Modules need to be either *local* and start with `.` or `..`, or they
need to be fully qualified. This means that the full module name
contains at least one separator (`/`).

In practice, this means that modules inside the global module search
path (set via the environment variable `R_BOX_PATH` or via
`getOption('box.path')`), modules need to be located in nested folders.

See [how to organise globally installed
modules?](#how-to-organise-globally-installed-modules) for details.

---
title: "Get started"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Get started}
    %\VignetteEncoding{UTF-8}
---
## Using modules

For the purpose of this tutorial, we are going to use the example module
`bio/seq`. The module implements some very basic mechanisms for dealing
with DNA sequences (= character strings consisting of the letters `A`,
`C`, `G` and `T`).

First, we load the module:

```{r eval = FALSE}
box::use(./bio/seq)
```

The function `box::use` accepts a list of *unquoted*, *qualified* module
names. Each of these module names will load a single module and make it
available to the caller in some form. In the code above, we’ve loaded a
single module, `bio/seq`. `bio` serves as a *parent module* that may
group several submodules. Since the module name inside `box::use` starts
with `./`, the module location is resolved *locally*, i.e. relative to
the path of the currently running code.

In the above, `seq` is the module’s *proper name*. `bio/seq` is its
*fully qualified name*. And `./bio/seq` is its *`use` declaration*.

To see the effect of this `use` declaration, let’s inspect our
workspace:

```{r eval = FALSE}
ls()
```

    ## [1] "oldwd" "seq"

```{r eval = FALSE}
seq
```

    ## <module: ./bio/seq>

We have used the module’s fully qualified name to load it. But, as shown
by `ls`, loading the module this way only introduces a single new name
into the current scope, the module itself, identified by its proper
(non-qualified) name.

To see which names a module exports, we use `ls` again, this time on the
module itself:

```{r eval = FALSE}
ls(seq)
```

    ## [1] "is_valid" "revcomp"  "seq"      "table"

It appears that `seq` exports 4 different names. To access exported
names, we use the `$` operator: <code>seq\$is_valid</code> allows us to
use the first function in the list of exported names. We can also
display the interactive help for individual names using the `box::help`
function, e.g.:

```{r eval = FALSE}
box::help(seq$revcomp)
```

Now let’s actually *use* the module. The `seq` function inside the
`bio/seq` module constructs a set of (optionally named) biological
sequences:

```{r eval = FALSE}
s = seq$seq(
    gene1 = 'GATTACAGATCAGCTCAGCACCTAGCACTATCAGCAAC',
    gene2 = 'CATAGCAACTGACATCACAGCG'
)

seq$is_valid(s)
```

    ## [1] TRUE

```{r eval = FALSE}
s
```

    ## 2 DNA sequences:
    ##   >gene1
    ##   GATTACAGATCAGCTCAGCACCTAGCA...
    ##   >gene2
    ##   CATAGCAACTGACATCACAGCG

Note how we automatically get pretty-printed
([FASTA](https://en.wikipedia.org/wiki/FASTA_format)) output because the
`print` method (which gets called implicitly here) is specialised for
the `'bio/seq'` S3 class in the `bio/seq` module (prefixing S3 classes
inside modules with the full module name is a convention to avoid name
clashes of S3 classes):

```{r eval = FALSE}
getS3method('print', 'bio/seq')
```

    ## function (x) {
    ##     box::use(stringr[trunc = str_trunc])
    ## 
    ##     if (is.null(names(x))) names(x) = paste('seq', seq_along(x))
    ## 
    ##     cat(
    ##         sprintf('%d DNA sequence%s:\n', length(x), if (length(x) == 1L) '' else 's'),
    ##         sprintf('  >%s\n  %s\n', names(x), trunc(x, 30L)),
    ##         sep = ''
    ##     )
    ##     invisible(x)
    ## }
    ## <environment: 0x13cd393d8>

The source code for `` `print.bio/seq` `` contains an interesting `use`
declaration: it showcases an alternative way of invoking `box::use`,
which we’ll explore now.

## Attaching modules

Let’s have a look at alternative ways of using modules.

To start, let’s unload the `bio/seq` module …

```{r eval = FALSE}
box::unload(seq)
```

… and load it again, via a different route:

```{r eval = FALSE}
options(box.path = getwd())
box::use(bio/seq[revcomp, is_valid])
```

After unloading the already loaded module, `options(box.path = …)` sets
the module search path: this is where `box::use` searches for modules.
If more than one path is given, `box::use` searches them all until a
module of matching name is found. This works analogously to how
`.libPaths` operates on R packages.

The `box::use` directive can now use `bio/seq` instead of `./bio/seq` as
the module name: rather than a relative name we specify a *global* name.
In this example we set the search path to the current working directory
but in normal usage it would be a global library location,
e.g. (following the [XDG base directory
specification](https://specifications.freedesktop.org/basedir/latest/))
`~/.local/share/R/modules` on Linux.

**Note** that non-local module names *must* be fully qualified, nested
modules: `box::use(foo/bar)` works, `box::use(bar)` does not (instead,
it is assumed that `bar` refers to a *package*)!

In the declaration above we use `[revcomp, is_valid]` to specify that
the names `revcomp` and `is_valid` from the `bio/seq` module should be
attached in the calling environment. The `[…]` part is an *attach
specification*: a comma-separated list of names inside the parentheses
specifies which names to attach. The special symbol `...` can be used to
specify that *all exported names* should be attached. This has an effect
similar to conventional package loading via `library` (or `attach`ing an
environment): all the attached names are now available for direct use
without necessitating the `seq$` qualifier:

```{r eval = FALSE}
is_valid(s)
```

    ## [1] TRUE

```{r eval = FALSE}
revcomp(s)
```

    ## 2 DNA sequences:
    ##   >gene1
    ##   GTTGCTGATAGTGCTAGGTGCTGAGCT...
    ##   >gene2
    ##   CGCTGTGATGTCAGTTGCTATG

However, unlike the `attach` function, module attachment happens in the
*current, local scope* only.

Since the above code was executed in the global environment, there’s no
distinction between local and global scope:

```{r eval = FALSE}
search()
```

    ##  [1] ".GlobalEnv"        "mod:bio/seq"       "package:stats"    
    ##  [4] "package:graphics"  "package:grDevices" "package:datasets" 
    ##  [7] "renv:shims"        "package:utils"     "package:methods"  
    ## [10] "Autoloads"         "package:base"

Note the second item, which reads “<code>mod:bio/seq</code>”. But let’s
now undo that, to attach (and use) the module locally instead:

```{r eval = FALSE}
detach()

seq_table = function (s) {
    box::use(./bio/seq[...])
    table(s)
}

seq_table(s)
```

    ## $gene1
    ##  A  C  G  T 
    ## 13 12  6  7 
    ## 
    ## $gene2
    ## A C G T 
    ## 8 7 4 3

Unlike above, we are now attaching *all* exported names instead of
specifying individual names. The subsequent line of code uses the
`seq$table` function rather than `base::table` (which would have a
different output). And note that the `seq` module’s `table` function is
*not* attached outside the local scope:

```{r eval = FALSE}
search()
```

    ##  [1] ".GlobalEnv"        "package:stats"     "package:graphics" 
    ##  [4] "package:grDevices" "package:datasets"  "renv:shims"       
    ##  [7] "package:utils"     "package:methods"   "Autoloads"        
    ## [10] "package:base"

```{r eval = FALSE}
table(s)
```

    ## s
    ##                 CATAGCAACTGACATCACAGCG GATTACAGATCAGCTCAGCACCTAGCACTATCAGCAAC 
    ##                                      1                                      1

This is very powerful, as it isolates separate scopes more effectively
than the `attach` function. What is more, modules which are used and
attached inside another module *remain* inside that module and are not
visible outside the module by default.

Nevertheless, the normal, recommended usage of a module is without an
attach specification, as this makes it clearer which names are being
referring to.

## Writing modules

The module `bio/seq`, which we have used in the previous section, is
implemented in the file [`bio/seq.r`](bio/seq.r). The file `seq.r` is,
by and large, a regular R source file, which happens to live in a
directory named `bio`.

In fact, there are only three things worth mentioning:

1.  Documentation: functions in the module file can be documented using
    ‘[roxygen2](https://cran.r-project.org/package=roxygen2)’ syntax. It
    works the same as for packages. The ‘box’ package parses the
    documentation and makes it available via `box::help`. *Displaying
    module help requires that ‘roxygen2’ is installed.*

2.  Export declarations: similar to packages, modules explicitly need to
    declare which names they export; they do this using the annotation
    comment `#' @export` in front of the name assignment. Again, this
    works similarly to ‘roxygen2’ (but does *not* require having that
    package installed).

3.  [S3 functions](https://adv-r.hadley.nz/s3.html): ‘box’ registers and
    exports such functions automatically as necessary, but this only
    works for *user generics* that are defined inside the same module.
    When overriding “known generics” (such as `print`), we need to
    register these manually via `register_S3_method` (this is necessary
    since these functions are inherently ambiguous and there is no
    automatic way of finding them).

## Nesting modules

Modules can also form nested hierarchies. In fact, here is the
implementation of `bio` (in [`bio/__init__.r`](bio/__init__.r): since
`bio` is a directory rather than a file, the module implementation
resides in the nested file `__init__.r`):

```{r eval = FALSE}
#' @export
box::use(./seq)
```

The submodule is specified as `./seq` rather than `seq`: the explicitly
provided relative path prevents lookup in the import search path (that
we set via `options(box.path = …)`); instead, only the current directory
(that is, the directory containing the `bio` module) is considered.

When applied to a `box::use` declaration, `@export` causes all names
which are imported by that declaration to also be exported: any module
name created by the declaration (here, `seq`) is exported as-is.
Furthermore, any attached name is likewise exported. Refer to the
`box::use` documentation and examples for more details on which names
are exported.

Coming back to our example module, we can now use the `bio` module:

```{r eval = FALSE}
options(box.path = NULL) # Reset search path
box::use(./bio)
ls(bio)
```

    ## [1] "seq"

```{r eval = FALSE}
ls(bio$seq)
```

    ## [1] "is_valid" "revcomp"  "seq"      "table"

```{r eval = FALSE}
bio$seq$revcomp('CAT')
```

    ## 1 DNA sequence:
    ##   >seq 1
    ##   ATG

We could also have implemented `bio` as follows:

```{r eval = FALSE}
#' @export
box::use(./seq[...])
```

This would have made all of `seq`’s definitions immediately available in
`bio`, without having to always write `seq$…`. This is sometimes useful,
but should be employed with care: being explicit about namespaces
generally increases code robustness and readability.

## Code execution on loading

Modules define functions and values. To execute code when a module is
loaded, put it inside a function with the name `.on_load`. This function
is similar to the hook for the `.onLoad` *package* namespace event.

This function is executed the first time the module is loaded in an R
session. Subsequent calls to `box::use` for that module, regardless of
whether they occur in a different scope, will refer to the already
loaded, cached module, and will *not* reload the module.

We can illustrate this by loading a module which has side-effects,
`info`.

```{r eval = FALSE}
.on_load = function (ns) {
    message(
        'Loading module "', box::name(), '"\n',
        'Module path: "', basename(box::file()), '"'
    )
}

box::export() # Mark as a ‘box’ module.
```

Let’s use it:

```{r eval = FALSE}
box::use(./info)
```

    ## Loading module "info"
    ## Module path: "vignettes"

We have imported the module, and get the diagnostic messages. Let’s
re-use the module:

```{r eval = FALSE}
box::use(./info)
```

… no messages are displayed. However, we *can* explicitly reload a
module. This clears the cache, and loads the module again. This can be
useful during development and debugging:

```{r eval = FALSE}
box::reload(info)
```

    ## Loading module "info"
    ## Module path: "vignettes"

And this displays the messages again. The `reload` function is a
shortcut for `unload` followed by `import` (using the exact same
arguments as used on the original `import` call).

### Module helper functions

This `info` module also show-cases two important helper functions:

1.  `box::name` returns the name of the module with which it was loaded.
    This is especially handy because, when called outside of a module,
    `box::name` is `NULL`. This allows testing whether a piece of code
    was loaded as a module, or invoked directly (e.g. via `Rscript` on
    the command line).

2.  `box::file` is similar to `system.file`: it returns the full path to
    any file within the directory where a module is stored. This is
    useful when distributing data files with modules, which are loaded
    from within the module. When invoked without arguments, `box::file`
    returns the full path to the directory containing the module source
    file.

---
title: "Testing modules"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Testing modules}
    %\VignetteEncoding{UTF-8}
---
## Testing is crucial

Writing tests to ensure code correctness is a crucial part of developing
robust software. This is especially true for dynamic languages such as
R, which lack some of the tools that ensure correctness in statically
checked programming languages.

‘box’ does not dictate a specific style of testing, and different kinds
of testing are appropriate in different situations. However, the
following article suggests a structure for the test accompanying a
module which I have found to work well.

Following the suggested structure causes the implementation and the
tests of each module to be located in paths next to each other. This
structure is found in several other conventions across programming
languages; but it differs somewhat from the convention of R packages
(where all implementation code is in a separate directory from all
testing code), and differs drastically from the convention found for
instance in Java. That said, it should be possible to make a Java-like
test project work with ‘box’, too.

## Support for existing testing frameworks

‘box’ is agnostic regarding the testing framework. The following example
employs the widely used ‘[testthat](https://testthat.r-lib.org/)’ unit
testing package, but other frameworks exist, and should also work.

This example uses the [`bio/seq`](bio/seq.r) module from the *[Getting
started](box.html)* vignette. To enable unit testing, this module
contains the following code at the very end:

```{r eval = FALSE}
if (is.null(box::name())) {
    box::use(./`__tests__`)
}
```

This allows us to run the tests by running the module source code from
the command line, e.g. via

``` bash
Rscript bio/seq.r
```

… or inside an IDE such as RStudio by choosing the menu item “Tools” ›
“Jobs” › “Start Local Job…” [^1]. This works because `box::name` returns
the name of the module from which this function is called. But if the
function is invoked from code that wasn’t loaded via `box::use` (as is
the case here), its value is `NULL`. In other words,
`is.null(box::name())` is a way of testing whether the code currently
being run is loaded as a module, or executed directly.

The code inside the `if` imports the `__tests__` submodule: that’s where
we put the unit tests. The name `__tests__` is a convention. You’re free
to choose a different name, but I recommend sticking with this
convention. Note that `__tests__` is not a valid R variable name; that’s
why it needs to be written in backticks, i.e. the qualified local module
name is `` ./`__tests__` ``.

In this case, the `__tests__` submodule consists of a directory with the
following contents:

- `__tests__/`
  - [`__init__.r`](bio/__tests__/__init__.r)
  - [`helper-module.r`](bio/__tests__/helper-module.r)
  - individual test files

The `__init__.r` file corresponds closely to the file `tests/testthat.R`
in a standard R package structure. It loads ‘testthat’ and launches the
tests:

```{r eval = FALSE}
box::use(testthat[...])

.on_load = function (ns) {
    test_dir(box::file())
}

box::export()
```

This first loads and attaches the ‘testthat’ package. Although attaching
is not strictly necessary, ‘testthat’ code is a lot more readable
without cluttering the code with explicit name qualifications.

Next it invokes `test_dir` and passes the tests’ directory via
`box::file` inside the module’s `.on_load` hook — remember that only
declarations should be at file level inside a module! All code execution
should happen inside functions.

The `helper-module.r` file is a ‘testthat’ helper; these are sourced
automatically by ‘testthat’ in the environment where the tests are run.
We use this mechanism to load our module in the test environment:

```{r eval = FALSE}
box::use(../seq[...])
```

And, again, attaching isn’t strictly necessary here. Note also that,
depending on how the tests are run, this helper file might not be
needed, since executing the module script file itself already loads the
module contents into the global namespace; however, not all ways of
loading the tests do this; for instance, RStudio’s “Start Local Job”
doesn’t. So having this helper is sometimes necessary, and never hurts.

With this set-up, the actual unit test files look exactly like regular
‘testthat’ test files. For instance, here’s `__tests__/test-seq.r`:

```{r eval = FALSE}
test_that('valid sequences can be created', {
    expect_error((s = seq('GATTACA')), NA)
    expect_true(is_valid(s))

    expect_error(seq('cattaga'), NA)
})

test_that('invalid sequences cannot be created', {
    expect_error(seq('GATTXA'))
})
```

### A note on RStudio and other IDEs

Most IDEs have an option to “Source” a local file. When doing this it
may *seem* as if the tests are correctly run, but this isn’t actually
the case! This is because `box::use` doesn’t reload code that has
already been loaded previously; instead, it uses an already loaded,
cached version. This means that running the tests via the “Source”
button risks running an outdated version of the tests, or the module, or
both, after modifying their code.

To avoid this, always execute the test module in a new R session. In
RStudio, the easiest way of doing this is by running it as a job, via
the menu “Tools” › “Jobs” › “Start Local Job…” (or using the option
“Source as Local Job…” in the “Source” drop-down).

## Test interfaces, not implementation details

One big difference between testing module code and testing package code
is that, with the testing structure laid out above, the testing code
only sees the module’s public interface, it does not get access to the
internal module implementation.[^2]

This is *by design*: the idea is that we want to test the *observable
behaviour* rather than the (purely incidental) current implementation,
which might be changed. This is the way unit testing works in many other
environments, and how it is often recommended.

But I realise that this may not always be appropriate. Sometimes we
*need* to test implementation details. There are essentially two
workarounds for this. For the moment, I have not yet developed a strong
preference for either of these methods.

1.  Put the tests into the module files themselves.

    ``` r
    # mymod.r

    this_works = function () TRUE

    if (is.null(box::name())) {
        box::use(testthat[...])

        # Define tests

        test_that('implementation detail X works', {
            expect_true(this_works())
        })

        # Invoke tests

        test_file(box::file())
    }
    ```

2.  Get access to the private module namespace for testing. When loading
    a module with `box::use`, the module object has an attribute
    `namespace` that holds the private module namespace. It generally
    shouldn’t be accessed by client code, but access by the testing code
    for a module is entirely legitimate:

    ``` r
    # __tests__/test-something.r

    box::use(../mymod)

    impl = attr(mymod, 'namespace')

    test_that('implementation detail X works', {
        expect_true(impl$this_works())
    })
    ```

[^1]: It may be tempting to instead use the “Source” button but doing so
    is problematic; see the section “A note on RStudio and other IDEs”!

[^2]: Depending on how it’s invoked; but we shouldn’t make assumptions.
    In particular, when invoked as a job in RStudio, the test module is
    loaded via the test helper, and thus the module internals are not
    visible.

---
title: "Similar packages"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{Similar packages}
    %\VignetteEncoding{UTF-8}
---
The need for tools to make R code more modular, and to improve code
reuse, is demonstrated by the wealth of related packages that have
sprung up over the years. Many of them have objectives similar to ‘box’,
although they vary wildly in how they approach their common goal, and
not all objectives are shared. In the following I’ve listed the related
R packages that I know of. The descriptions are (mostly) copied from the
respective package metadata, and the interested reader is invited to
consult the original documentation.

The list is intended to be a fairly exhaustive — so if you find a
missing package please let me know.

## Tools for writing modular code

- ‘**[modules](https://cran.r-project.org/package=modules)**’ (no
  relation to the predecessor of ‘box’, also called ‘modules’): Provides
  modules as an organizational unit for source code. Modules enforce to
  be more rigorous when defining dependencies and have a local search
  path. They can be used as a sub unit within packages or in scripts.

- ‘[trinker/**pysty**](https://trinkerrstuff.wordpress.com/2018/02/22/minimal-explicit-python-style-package-loading-for-r/)’:
  ‘Python’ style packages importing using the common forms of:
  `import PACKAGE`, `import PACKAGE as ALIAS`, or
  `from PACKAGE import FUN1, FUN2, FUN_N`.

- ‘[**Shiny**’
  modules](https://shiny.rstudio.com/articles/modules.html): ‘Shiny’
  modules address the namespacing problem in Shiny UI and server logic,
  adding a level of abstraction beyond functions.

- ‘[Novartis/**tidymodules**](https://opensource.nibr.com/tidymodules/)’:
  ‘tidymodules’ offers a robust framework for developing ‘Shiny’ modules
  based on R6 classes which should facilitates inter-modules
  communication.

## Tools for loading code

The packages listed above also provide means for loading code; however,
I won’t replicate them here. Some of the tools below have subsequently
also gained capabilities for authoring modular code, but since their
initial and primary purpose was *loading* code, they’re instead listed
below.

- ‘**[import](https://cran.r-project.org/package=import)**’: Alternative
  mechanism for importing objects from packages and R modules. The
  syntax allows for importing multiple objects with a single command in
  an expressive way. The import package bridges some of the gap between
  using library (or require) and direct (single-object) imports.
  Furthermore the imported objects are not placed in the current
  environment.

- ‘**[conflicted](https://conflicted.r-lib.org/)**’: R’s default
  conflict management system gives the most recently loaded package
  precedence. This can make it hard to detect conflicts, particularly
  when they arise because a package update creates ambiguity that did
  not previously exist. ‘conflicted’ takes a different approach, making
  every conflict an error and forcing you to choose which function to
  use.

- ‘**[pacman](http://trinker.github.io/pacman/)**’: Tools to more
  conveniently perform tasks associated with add-on packages. pacman
  conveniently wraps library and package related functions and names
  them in an intuitive and consistent fashion. It seeks to combine
  functionality from lower level functions which can speed up workflow.

- ‘[jonocarroll/**importAs**](https://jonocarroll.github.io/importAs/)’:
  Import namespaces as shorthand symbols rather than full package names,
  in a python-esque fashion.

- ‘[**needs**](https://cran.r-project.org/package=needs)’: A simple
  function for easier package loading and auto-installation.

- ‘[**importar**](https://cran.r-project.org/package=importar)’: Enables
  ‘Python’-like importing/loading of packages or functions with aliasing
  to prevent namespace conflicts.

- [**xfun**::pkg_attach()](https://yihui.org/en/2018/09/xfun-pkg-attach/):
  A vectorized version of `library()`

## Tools for organising code

- ‘**[RSuite](https://cran.r-project.org/package=RSuite)**’: Supports
  safe and reproducible solutions development in R. It will help you
  with environment separation per project, dependency management, local
  packages creation and preparing deployment packs for your solutions.
  (Archived as of 2026-04-08)

- ‘**[mvbutils](https://cran.r-project.org/package=mvbutils)**’:
  Hierarchical workspace tree, code editing and backup, easy package
  prep, editing of packages while loaded, per-object lazy-loading, easy
  documentation, macro functions, and miscellaneous utilities.

---
title: "The hierarchy of module environments"
author: Konrad Rudolph
date: "`r Sys.Date()`"
output:
    rmarkdown::html_vignette:
        toc: true
    md_document:
        variant: gfm
vignette: >
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteIndexEntry{The hierarchy of module environments}
    %\VignetteEncoding{UTF-8}
---
> **Note:** This document describes internal implementation details.
> They are not required knowledge for users of the ‘box’ package and
> module authors.

## Preliminaries

To ensure that module environments are properly isolates from each
other, and to enable fine-grained control over imports and exports,
loaded modules (as well as packages loaded via `box::use`) correspond to
a mesh of several interconnected environments.

Unfortunately the exact relationship between these environments isn’t
trivial, so the following document aims at explaining the gist.

Fundamentally, module environments follow a similar architecture to
package environments, but they deviate in crucial ways. For a good
explanation of package environments, see [*How R Searches and Finds
Stuff*](https://blog.thatbuthow.com/how-r-searches-and-finds-stuff/) by
Suraj Gupta, and
[*Environments*](http://adv-r.had.co.nz/Environments.html) in *Advanced
R* by Hadley Wickham. The following assumes that the reader has a more
than passing familiarity with these concepts.

When speaking of “package environments” above and in the following, this
refers to packages as handled by base R and the R package namespace
functionality. For packages that are loaded by `box::use`, the rules are
the same as for modules.

Let’s now consider an example of several loaded modules, and how they
interact.

## An example

This example will use three modules — the minimum number necessary to
show some of the interactions of the module environments. Let’s define
these three modules. We’ll do so in inverse order of their usage:

### <code><span style="background: #1DB100; color: white; padding: 0 5px; border-radius: 2px;">c.r</span></code>

```{r eval = FALSE}
#' @export
f = function () 'c$f'
```

Module `c` defines and exports one name, the function `f`.

### <code><span style="background: #FF9300; color: white; padding: 0 5px; border-radius: 2px;">b.r</span></code>

```{r eval = FALSE}
#' @export
f = function () 'b$f'

g = function () 'b$g'
```

Module `b` defines and exports the function `f`, and in addition defines
the function `g`.

### <code><span style="background: #00A2FF; color: white; padding: 0 5px; border-radius: 2px;">a.r</span></code>

```{r eval = FALSE}
#' @export
box::use(./b[g = f, ...])

box::use(./c[...])

#' @export
box::use(./c)

#' @export
f = function () 'a$f'

f_of_c1 = c$f
f_of_c2 = get('f', parent.env(environment()))
stopifnot(identical(f_of_c1, f_of_c2))
```

Module `a` imports and re-exports all names from `b` (changing the name
of `b$f` to `g`); it also imports, but does not re-export, all names
from `c` (and, additionally, defines an alias for the module, which it
exports).

It further defines and exports the function `f` and defines, but does
not export, the functions `f_of_c1` and `f_of_c2` (which are defined
such that they are identical to each other — hopefully it will become
clear why later).

Finally, let’s use the modules by executing the following code:

```{r eval = FALSE}
box::use(a = ./a[f, g])
```

Now the name `a` is defined in `.GlobalEnv` and refers to the module
environment of module `a`. The exported names `f` and `g` of `a` are
also available in `.GlobalEnv` (but not defined *inside* `.GlobalEnv`).

The loaded modules can be represented by the following schematic, with
each box corresponding to an environment:

![](environment-schema.png)

Let’s go over the different types of environment associated with each
module, and how they are connected.

## Environments

### Module namespace

Modules are loaded into their own dedicated environment, the *module
namespace*. Every name that is defined by a module is defined inside it.
It is thus also the enclosing environment of the module’s functions.
Lastly, the module namespace also stores meta-information about the
module in a hidden member named `.__module__.`.

This corresponds to the package namespace.

### Module imports environment

The parent environment of the module namespace is the imports
environment, which contains all the names that a module imports via
attachment declarations in `box::use` expressions.

The parent environment of the imports environment is the R `base`
namespace environment. The module imports environment thus corresponds
to the package imports environment.

### Module export environment

The module export environment, also called just “module environment” in
the code base, contains all names that are marked as exported by a
module. If users create a module alias in their `box::use` call, the
alias will be a reference to this environment.

Similarly, attached names are selected as a subset from the module
export environment (then copied into the imports environment).

There is no direct equivalent to this environment in R packages, because
R packages do not distinguish between exported names and names imported
by the code that loaded the package. By contrast, ‘box’ needs to
distinguish the set of exports from the set of names that are imported
(= “attached”) by client code.

### Importing into other environments

Module import environments store imports from one or more module into
another. However, modules can also be imported from inside the global
environment, or from inside functions, which have their own local
execution environment (the stack frame). When this happens, a new
imports environment is created on the fly, and attached to the importing
environment’s `parent.env` chain. Calling `box::use` with a non-empty
attach list in `.GlobalEnv` is therefore similar to calling `library`.
In particular, inside `.GlobalEnv` the call `box::use(pkg[...])` in
pretty much equivalent to calling `library(pkg)`, and is discouraged for
the same reason that `library(pkg)` is discouraged.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.r
\name{has_export_tag}
\alias{has_export_tag}
\title{Find \code{@export} tags in code regions}
\usage{
has_export_tag(ref)
}
\arguments{
\item{ref}{The code region \code{srcref} to search.}
}
\value{
\code{TRUE} if the given region is annotated with a \code{@export}
tag, \code{FALSE} otherwise.
}
\description{
Find \code{@export} tags in code regions
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.r
\name{help-internal}
\alias{help-internal}
\alias{help_topic_target}
\alias{find_env}
\alias{call_help}
\title{Helper functions for the help functionality}
\usage{
help_topic_target(topic, caller)

find_env(name, caller)

call_help(call, caller)
}
\arguments{
\item{topic}{the unevaluated expression passed to \code{help}.}

\item{caller}{the environment from which \code{help} was called.}

\item{name}{the name to look for.}

\item{call}{the patched \code{help} call expression.}
}
\value{
\code{help_topic_target} returns a list of two elements containing
the innermost module of the \code{help} call, as well as the name of the
object that’s the subject of the \code{help} call. For \code{help(a$b$c$d)},
it returns \code{list(c, quote(d))}.
}
\description{
\code{help_topic_target} parses the expression being passed to the
\code{help} function call to find the innermost module subset expression in
it.
\code{find_env} acts similarly to \code{\link[utils]{find}}, except that it
looks in the current environment’s parents rather than in the global
environment search list, it returns only one hit (or zero), and it returns
the environment rather than a character string.
\code{call_help} invokes a \code{help()} call expression for a package help
topic, finding the first \code{help} function definition, ignoring the one
from this package.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use.r
\name{importing}
\alias{importing}
\alias{use_one}
\alias{load_and_register}
\alias{register_as_import}
\alias{defer_import_finalization}
\alias{finalize_deferred}
\alias{export_and_attach}
\alias{load_from_source}
\alias{load_mod}
\alias{mod_exports}
\alias{mod_export_names}
\alias{attach_to_caller}
\alias{attach_list}
\alias{assign_alias}
\alias{assign_temp_alias}
\title{Import a module or package}
\usage{
use_one(declaration, alias, caller, use_call)

load_and_register(spec, info, caller)

register_as_import(spec, info, mod_ns, caller)

defer_import_finalization(spec, info, mod_ns, caller)

finalize_deferred(info)

export_and_attach(spec, info, mod_ns, caller)

load_from_source(info, mod_ns)

load_mod(info)

mod_exports(info, spec, mod_ns)

mod_export_names(info, mod_ns)

attach_to_caller(spec, info, mod_exports, mod_ns, caller)

attach_list(spec, exports)

assign_alias(spec, mod_exports, caller)

assign_temp_alias(spec, caller)
}
\arguments{
\item{declaration}{an unevaluated use declaration expression without the
surrounding \code{use} call}

\item{alias}{the use alias, if given, otherwise \code{NULL}}

\item{caller}{the client’s calling environment (parent frame)}

\item{use_call}{the \code{use} call which is invoking this code}

\item{spec}{a module use declaration specification}

\item{info}{the physical module information}

\item{mod_ns}{the module namespace environment of the newly loaded module}
}
\value{
\code{use_one} does not currently return a value. — This might change
in the future.

\code{load_mod} returns a named \code{list(mod_ns, is_apex)}
containing the module or package namespace environment of the specified
module or package info, as well as a flag specifying whether the loaded
module is an “apex” module, i.e. not loaded from within another module. This
information is then used to lock all environments that were created during
the loading. — This must happen after nested modules are fully loaded, since
deferred finalization otherwise cannot modify these environments during
cyclic imports.

\code{mod_exports} returns an export environment containing the
exported names of a given module.

\code{mode_export_names} returns a vector containing the same names as
\code{names(mod_exports(info, spec, mod_ns))} but does not create an export
environment.

\code{attach_list} returns a named character vector of the names in
an attach specification. The vector’s names are the aliases, if provided, or
the attach specification names themselves otherwise.
}
\description{
Actual implementation of the import process
}
\details{
\code{use_one} performs the actual import. It is invoked by \code{use} given
the calling context and unevaluated expressions as arguments, and only uses
standard evaluation.

\code{load_and_register} performs the loading, attaching and exporting of a
module identified by its spec and info.

\code{register_as_import} registers a \code{use} declaration in the calling
module so that it can be found later on, if the declaration is reexported by
the calling module.

\code{defer_import_finalization} is called by \code{load_and_register} to
earmark a module for deferred initialization if it hasn’t been fully loaded
yet.

\code{finalize_deferred} exports and attaches names from a module use
declaration which has been deferred due to being part of a cyclic loading
chain.

\code{export_and_attach} exports and attaches names from a given module use
declaration.

\code{load_from_source} loads a module source file into its newly created,
empty module namespace.

\code{load_mod} tests whether a module or package was already loaded and, if
not, loads it.

\code{mod_exports} returns an export environment containing a copy of the
module’s exported objects.

\code{attach_to_caller} attaches the listed names of an attach specification
for a given use declaration to the calling environment.

\code{assign_alias} creates a module/package object in calling environment,
unless it contains an attach declaration, and no explicit alias is given.

\code{assign_temp_alias} creates a placeholder object for the module in the
calling environment, to be replaced by the actual module export environment
once the module is completely loaded (which happens in the case of cyclic
imports).
}
\note{
If a module is still being loaded (because it is part of a cyclic
import chain), \code{load_and_register} earmarks the module for deferred
registration and holds off on attaching and exporting for now, since not all
its names are available yet.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.r
\name{export}
\alias{export}
\title{Explicitly declare module exports}
\usage{
\special{box::export(\dots)}
}
\arguments{
\item{\dots}{zero or more unquoted names that should be exported from the
module.}
}
\value{
\code{box::export} has no return value. It is called for its
side effect.
}
\description{
\code{box::export} explicitly marks a source file as a \pkg{box} module. If
can be used as an alternative to the \code{@export} tag comment to declare a
module’s exports.
}
\details{
\code{box::export} can be called inside a module to specify the module’s
exports. If a module contains a call to \code{box::export}, this call
overrides any declarations made via the \code{@export} tag comment. When a
module contains multiple calls to \code{box::export}, the union of all thus
defined names is exported.

A module can also contain an argument-less call to \code{box::export}. This
ensures that the module does not export any names. Otherwise, a module that
defines names but does not mark them as exported would be treated as a
\emph{legacy module}, and all default-visible names would be exported from
it. Default-visible names are names not starting with a dot (\code{.}).
Another use of \code{box::export()} is to enable a module without exports to
use \link[=mod-hooks]{module event hooks}.
}
\note{
The preferred way of declaring exports is via the \code{@export} tag
comment. The main purpose of \code{box::export} is to explicitly prevent
exports, by being called without arguments.
}
\seealso{
\code{\link[=use]{box::use}} for information on declaring exports via
\code{@export}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/env.r
\name{name}
\alias{name}
\title{Get a module’s name}
\usage{
\special{box::name()}
}
\value{
\code{box::name} returns a character string containing the name of
the module, or \code{NULL} if called from outside a module.
}
\description{
Get a module’s name
}
\note{
Because this function returns \code{NULL} if not invoked inside a
module, the function can be used to check whether a code is being imported as
a module or called directly.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.r
\name{lsf}
\alias{lsf}
\title{Return a list of function names in an environment}
\usage{
lsf(envir)
}
\arguments{
\item{envir}{the environment to search in.}
}
\value{
\code{lsf} returns a vector of function names in the given environment.
}
\description{
Return a list of function names in an environment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use.r
\name{use}
\alias{use}
\title{Import a module or package}
\usage{
\special{box::use(prefix/mod, \dots)}

\special{box::use(pkg, \dots)}

\special{box::use(alias = prefix/mod, \dots)}

\special{box::use(alias = pkg, \dots)}

\special{box::use(prefix/mod[attach_list], \dots)}

\special{box::use(pkg[attach_list], \dots)}
}
\arguments{
\item{\dots}{further import declarations}

\item{prefix/mod}{a qualified module name}

\item{pkg}{a package name}

\item{alias}{an alias name}

\item{attach_list}{a list of names to attached, optionally with aliases of
the form \code{alias = name}; or the special placeholder name \code{\dots}}
}
\value{
\code{box::use} has no return value. It is called for its
side effect.
}
\description{
\code{box::use} imports one or more modules and/or packages, and makes them
available in the calling environment.
}
\details{
\code{box::use(\dots)} specifies a list of one or more import declarations,
given as individual arguments to \code{box::use}, separated by comma.
\code{box::use} permits using a trailing comma after the last import
declaration. Each import declaration takes one of the following forms:

\describe{
\item{\code{\var{prefix}/\var{mod}}:}{
     Import a module given the qualified module name
     \code{\var{prefix}/\var{mod}} and make it available locally using the
     name \code{\var{mod}}. The \code{\var{prefix}} itself can be a nested
     name to allow importing specific submodules. \emph{Local imports} can be
     specified via the prefixes starting with \code{.} and \code{..}, to
     override the search path and use the local path instead. See the
     \sQuote{Search path} below for details.
}
\item{\code{\var{pkg}}:}{
     Import a package \code{\var{pkg}} and make it available locally using its own
     package name.
}
\item{\code{\var{alias} = \var{prefix}/\var{mod}} or \code{\var{alias} = \var{pkg}}:}{
     Import a module or package, and make it available locally using the name
     \code{\var{alias}} instead of its regular module or package name.
}
\item{\code{\var{prefix}/\var{mod}[\var{attach_list}]} or \code{\var{pkg}[\var{attach_list}]}:}{
     Import a module or package and attach the exported symbols listed in
     \code{\var{attach_list}} locally. This declaration does \emph{not} make
     the module/package itself available locally. To override this, provide
     an alias, that is, use \code{\var{alias} =
     \var{prefix}/\var{mod}[\var{attach_list}]} or \code{\var{alias} =
     \var{pkg}[\var{attach_list}]}.

     The \code{\var{attach_list}} is a comma-separated list of names,
     optionally with aliases assigned via \code{alias = name}. The list can
     also contain the special symbol \code{\dots}, which causes \emph{all}
     exported names of the module/package to be imported.
}
}

See the vignette at \code{vignette('box', 'box')} for detailed examples of
the different types of use declarations listed above.
}
\section{Import semantics}{

Modules and packages are loaded into dedicated namespace environments. Names
from a module or package can be selectively attached to the current scope as
shown above.

Unlike with \code{\link[base]{library}}, attaching happens \emph{locally},
i.e. in the caller’s environment: if \code{box::use} is executed in the
global environment, the effect is the same. Otherwise, the effect of
importing and attaching a module or package is limited to the caller’s local
scope (its \code{environment()}). When used \emph{inside a module} at module
scope, the newly imported module is only available inside the module’s scope,
not outside it (nor in other modules which might be loaded).

Member access of (non-attached) exported names of modules and packages
happens via the \code{$} operator. This operator does not perform partial
argument matching, in contrast with the behavior of the \code{$} operator in
base \R, which matches partial names.

\strong{Note} that replacement functions (i.e. functions of the form
\code{fun<-}) must be \emph{attached} to be usable, because \R syntactically
does not allow assignment calls where the left-hand side of the assignment
contains \code{$}.
}

\section{Export specification}{


Names defined in modules can be marked as \emph{exported} by prefixing them
with an \code{@export} tag comment; that is, the name needs to be immediately
prefixed by a comment that reads, verbatim, \code{#' @export}. That line may
optionally be part of a \pkg{roxygen2} documentation for that name.

Alternatively, exports may be specified via the
\code{\link[=export]{box::export}} function, but using declarative
\code{@export} tags is generally preferred.

A module which has not declared any exports is treated as a \emph{legacy
module} and exports \emph{all} default-visible names (that is, all names that
do not start with a dot (\code{.})). This usage is present only for backwards
compatibility with plain \R scripts, and its usage is \emph{not recommended}
when writing new modules.

To define a module that exports no names, call \code{box::export()} without
arguments. This prevents the module from being treated as a legacy module.
}

\section{Search path}{

Modules are searched in the module search path, given by
\code{getOption('box.path')}. This is a character vector of paths to search,
from the highest to the lowest priority. The current directory is always
considered last. That is, if a file \file{a/b.r} exists both locally in the
current directory and in a module search path, the local file \file{./a/b.r}
will \emph{not} be loaded, unless the import is explicitly declared as
\code{box::use(./a/b)}.

Modules in the module search path \emph{must be organised in subfolders}, and
must be imported fully qualified. Keep in mind that \code{box::use(name)}
will \emph{never} attempt to load a module; it always attempts to load a
package. A common module organisation is by project, company or user name;
for instance, fully qualified module names could mirror repository names on
source code sharing websites (such as GitHub).

Given a declaration \code{box::use(a/b)} and a search path \file{\var{p}}, if
the file \file{\var{p}/a/b.r} does not exist, \pkg{box} alternatively looks
for a nested file \file{\var{p}/a/b/__init__r} to load. Module path names are
\emph{case sensitive} (even on case insensitive file systems), but the file
extension can be spelled as either \file{.r} or \file{.R} (if both exist,
\code{.r} is given preference).

The module search path can be overridden by the environment variable
\env{R_BOX_PATH}. If set, it may consist of one or more search paths,
separated by the platform’s path separator (i.e. \code{;} on Windows, and
\code{:} on most other platforms).

\strong{Deprecation warning:} in the next major version, \pkg{box} will read
environment variables only \emph{once}, at package load time. Modifying the
value of \env{R_BOX_PATH} afterwards will have no effect, unless the package
is unloaded and reloaded.

The \emph{current directory} is context-dependent: inside a module, the
directory corresponds to the module’s directory. Inside an \R code file
invoked from the command line, it corresponds to the directory containing
that file. If the code is running inside a \pkg{Shiny} application or a
\pkg{knitr} document, the directory of the execution is used. Otherwise (e.g.
in an interactive \R session), the current working directory as given by
\code{getwd()} is used.

Local import declarations (that is, module prefixes that start with \code{./}
or \code{../}) never use the search path to find the module. Instead,
only the current module’s directory (for \code{./}) or the parent module’s
directory (for \code{../}) is looked at. \code{../} can be nested:
\code{../../} denotes the grandparent module, etc.
}

\section{S3 support}{

Modules can contain S3 generics and methods. To override known generics
(= those defined outside the module), methods inside a module need to be
registered using \code{\link[=register_S3_method]{box::register_S3_method}}.
See the documentation there for details.
}

\section{Module names}{

A module’s full name consists of one or more \R names separated by \code{/}.
Since \code{box::use} declarations contain \R expressions, the names need to
be valid \R names. Non-syntactic names need to be wrapped in backticks; see
\link[base]{Quotes}.

Furthermore, since module names usually correspond to file or folder names,
they should consist only of valid path name characters to ensure portability.
}

\section{Encoding}{

All module source code files are assumed to be UTF-8 encoded.
}

\examples{
# Set the module search path for the example module.
old_opts = options(box.path = system.file(package = 'box'))
old_env = Sys.getenv('R_BOX_PATH', NA)
Sys.unsetenv('R_BOX_PATH')

# Basic usage
# The file `mod/hello_world.r` exports the functions `hello` and `bye`.
box::use(mod/hello_world)
hello_world$hello('Robert')
hello_world$bye('Robert')

# Using an alias
box::use(world = mod/hello_world)
world$hello('John')

# Attaching exported names
box::use(mod/hello_world[hello])
hello('Jenny')
# Exported but not attached, thus access fails:
try(bye('Jenny'))

# Attach everything, give `hello` an alias:
box::use(mod/hello_world[hi = hello, ...])
hi('Eve')
bye('Eve')

# Reset the module search path
on.exit({
  options(old_opts)
  if (! is.na(old_env)) Sys.setenv(R_BOX_PATH = old_env)
})

\dontrun{
# The following code illustrates different import declaration syntaxes
# inside a single `box::use` declaration:

box::use(
    global/mod,
    mod2 = ./local/mod,
    purrr,
    tbl = tibble,
    dplyr = dplyr[filter, select],
    stats[st_filter = filter, ...],
)

# This declaration makes the following names available in the caller’s scope:
#
# 1. `mod`, which refers to the module environment for  `global/mod`
# 2. `mod2`, which refers to the module environment for `./local/mod`
# 3. `purrr`, which refers to the package environment for ‘purrr’
# 4. `tbl`, which refers to the package environment for ‘tibble’
# 5. `dplyr`, which refers to the package environment for ‘dplyr’
# 6. `filter` and `select`, which refer to the names exported by ‘dplyr’
# 7. `st_filter`, which refers to `stats::filter`
# 8. all other exported names from the ‘stats’ package
}
}
\seealso{
\code{\link[=name]{box::name}} and \code{\link[=file]{box::file}} give
information about loaded modules.
\code{\link[=help]{box::help}} displays help for a module’s exported names.
\code{\link[=unload]{box::unload}} and \code{\link[=reload]{box::reload}} aid
during module development by performing dynamic unloading and reloading of
modules in a running \R session.
\code{\link[=export]{box::export}} can be used as an alternative to
\code{@export} comments inside a module to declare module exports.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/env.r
\name{namespace}
\alias{namespace}
\alias{make_namespace}
\alias{is_namespace}
\alias{namespace_info}
\alias{namespace_info<-}
\alias{mod_topenv}
\alias{is_mod_topenv}
\title{Module namespace handling}
\usage{
make_namespace(info)

is_namespace(env)

namespace_info(ns, which, default = NULL)

namespace_info(ns, which) <- value

mod_topenv(env = parent.frame())

is_mod_topenv(env)
}
\arguments{
\item{info}{the module info.}

\item{env}{an environment that may be a module namespace.}

\item{ns}{the module namespace environment.}

\item{which}{the key (as a length 1 character string) of the info to get/set.}

\item{default}{default value to use if the key is not set.}

\item{value}{the value to assign to the specified key.}
}
\value{
\code{make_namespace} returns the newly created module namespace for
the module described by \code{info}.
}
\description{
\code{make_namespace} creates a new module namespace.
}
\details{
The namespace contains a module’s content. This schema is very much like R
package organisation. A good resource for this is:
<http://obeautifulcode.com/R/How-R-Searches-And-Finds-Stuff/>
}
\note{
Module namespaces aren’t actual R package namespaces. This is
intentional, since R makes strong assumptions about package namespaces that
are violated here. In particular, such namespaces would have to be registered
in R’s internal namespace registry, and their (de)serialisation is handled by
R code which assumes that they belong to actual packges that can be loaded
via `loadNamespace`.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info.r
\name{info}
\alias{info}
\alias{mod_info}
\alias{pkg_info}
\title{Information about a physical module or package}
\usage{
mod_info(spec, source_path)

pkg_info(spec)
}
\arguments{
\item{spec}{a \code{mod_spec} or \code{pkg_spec} (for \code{mod_info} and
\code{pkg_info}, respectively)}

\item{source_path}{character string full path to the physical module
location.}
}
\value{
\code{mod_info} and \code{pkg_info} return a structure representing
the module/package information for the given specification/source location.
}
\description{
A \code{mod_info} represents an existing, installed module and its runtime
physical location (usually in the file system).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/box-package.r
\docType{package}
\name{box}
\alias{box-package}
\alias{box}
\title{An alternative module system for R}
\description{
Use \code{box::use(prefix/mod)} to import a module, or \code{box::use(pkg)}
to import a package. Fully qualified names are supported for nested modules,
reminiscent of module systems in many other modern languages.
}
\section{Using modules & packages}{


\itemize{
 \item \code{\link[=use]{box::use}}
}
}

\section{Writing modules}{


Infrastructure and utility functions that are mainly used inside modules.

\itemize{
 \item \code{\link[=file]{box::file}}
 \item \code{\link[=name]{box::name}}
 \item \code{\link[=register_S3_method]{box::register_S3_method}}
 \item \link{mod-hooks}
}
}

\section{Interactive use}{


Functions for use in interactive sessions and for testing.

\itemize{
 \item \code{\link[=help]{box::help}}
 \item \code{\link[=unload]{box::unload}},
     \code{\link[=reload]{box::reload}},
     \code{\link[=unload]{box::purge_cache}}
 \item \code{\link[=set_script_path]{box::set_script_path}}
 \item \code{\link[=script_path]{box::script_path}},
     \code{\link[=script_path]{box::set_script_path}}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://klmr.me/box/}
  \item \url{https://github.com/klmr/box}
  \item Report bugs at \url{https://github.com/klmr/box/issues}
}

}
\author{
\strong{Maintainer}: Konrad Rudolph \email{konrad.rudolph@gmail.com} (\href{https://orcid.org/0000-0002-9866-7051}{ORCID})

Other contributors:
\itemize{
  \item Michael Schubert \email{mschu.dev@gmail.com} (\href{https://orcid.org/0000-0002-6862-5221}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.r
\name{s3}
\alias{s3}
\alias{is_S3_user_generic}
\alias{make_S3_methods_known}
\title{Internal S3 infrastructure helpers}
\usage{
is_S3_user_generic(function_name, envir = parent.frame())

make_S3_methods_known(module)
}
\arguments{
\item{function_name}{function name as character string.}

\item{envir}{the environment this function is invoked from.}

\item{module}{the module object for which to register S3 methods}
}
\value{
\code{is_S3_user_generic} returns \code{TRUE} if the specified
function is a user-defined S3 generic, \code{FALSE} otherwise.
}
\description{
The following are internal S3 infrastructure helper functions.
}
\details{
\code{is_S3_user_generic} checks whether a function given by name is a
user-defined generic. A user-defined generic is any function which, at some
point, calls \code{UseMethod}.

\code{make_S3_methods_known} finds and registers S3 methods inside a module.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.r
\name{parse_documentation}
\alias{parse_documentation}
\alias{parse_roxygen_tags}
\alias{patch_mod_doc}
\title{Parse a module’s documentation}
\usage{
parse_documentation(info, mod_ns)

parse_roxygen_tags(info, mod_ns)

patch_mod_doc(docs)
}
\arguments{
\item{info}{The module info.}

\item{mod_ns}{The module namespace.}

\item{docs}{the list of \pkg{roxygen2} documentation objects.}
}
\value{
\code{parse_documentation} returns a list of character strings with
the Rd documentation source code for each documented name in a module.
}
\description{
Parse a module’s documentation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.r
\name{add_comments}
\alias{add_comments}
\title{Extend code regions to include leading comments and whitespace}
\usage{
add_comments(refs)
}
\arguments{
\item{refs}{a list of the code region \code{srcref}s to extend.}
}
\value{
\code{add_comment} returns a list of \code{srcref}s corresponding to
\code{srcref}, but extended to include the preceding comment block.
}
\description{
Extend code regions to include leading comments and whitespace
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.r
\docType{data}
\name{parse_export_specs}
\alias{parse_export_specs}
\alias{use_call}
\alias{static_assign_calls}
\alias{assign_calls}
\alias{is_static_assign_call}
\alias{is_assign_call}
\alias{block_is_assign}
\alias{block_is_use_call}
\alias{block_is_exported}
\alias{block_name}
\title{Find exported names in parsed module source}
\format{
An object of class \code{call} of length 3.

An object of class \code{list} of length 4.

An object of class \code{list} of length 6.
}
\usage{
parse_export_specs(info, exprs, mod_ns)

use_call

static_assign_calls

assign_calls

is_static_assign_call(call)

is_assign_call(call)

block_is_assign(block)

block_is_use_call(block)

block_is_exported(block)

block_name(block)
}
\arguments{
\item{info}{The module info.}

\item{exprs}{The list of expressions of the parsed module.}

\item{mod_ns}{The module namespace after evaluating the expressions.}

\item{call}{A call to test.}

\item{block}{A roxygen2 block to inspect.}
}
\value{
\code{parse_export_specs} returns a potentially empty character
vector of exported names.
}
\description{
Find exported names in parsed module source
}
\note{
There are two situations in which the \code{@export} tag can be applied:
\enumerate{
  \item When applied to assignments, the assigned name is exported.
  \item When applied to a \code{box::use} call, the imported names are
    exported. This can be the module name itself, any attached names, or
    both. All names introduced by the \code{box::use} call are exported. See
    \code{\link[box]{use}} for the rules governing what names are introduced
    into the scope, and thus exported.
}
In any other situation, applying the \code{@export} tag is an error.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spec.r
\name{spec}
\alias{spec}
\alias{parse_spec}
\alias{mod_spec}
\alias{pkg_spec}
\alias{spec_name}
\title{Parse a mod or pkg spec expression passed to \code{use}}
\usage{
parse_spec(expr, alias)

mod_spec(spec, ...)

pkg_spec(spec, ...)

spec_name(spec)
}
\arguments{
\item{expr}{the mod or pkg spec expression to parse}

\item{alias}{the mod or pkg spec alias as a character, or \code{NULL}}

\item{spec}{named list of information the parser constructed from a given
spec expression}

\item{\dots}{further information about a spec, not represented by the spec
expression parse tree}
}
\value{
\code{parse_spec} returns a named list that contains information
about the parsed mod specification. Currently it contains:
\describe{
 \item{\code{name}}{the module or package name}
 \item{\code{prefix}}{the prefix, if the spec is a module}
 \item{\code{attach}}{a named vector of symbols to attach, or
     \code{TRUE} to attach all symbols, or \code{NULL} to attach nothing}
 \item{\code{alias}}{the module or package alias}
 \item{\code{explicit}}{a logical value indicating whether the caller
     provided an explicit alias}
}
}
\description{
Parse a mod or pkg spec expression passed to \code{use}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hooks.r
\name{mod-hooks}
\alias{mod-hooks}
\alias{.on_load}
\alias{.on_unload}
\title{Hooks for module events}
\usage{
.on_load(ns)

.on_unload(ns)
}
\arguments{
\item{ns}{the module namespace environment}
}
\value{
Any return values of the hook functions are ignored.
}
\description{
Modules can declare functions to be called when a module is first loaded.
}
\details{
To create module hooks, modules should define a function with the specified
name and signature. Module hooks should \emph{not} be exported.

When \code{.on_load} is called, the unlocked module namespace environment is
passed to it via its parameter \code{ns}. This means that code in
\code{.on_load} is permitted to modify the namespace by adding names to,
replacing names in, or removing names from the namespace.

\code{.on_unload} is called when modules are unloaded. The (locked) module
namespace is passed as an argument. It is primarily useful to clean up
resources used by the module. Note that, as for packages, \code{.on_unload}
is \emph{not} necessarily called when R is shut down.

\emph{Legacy modules} cannot use hooks. To use hooks, the module needs to
contain an export specification (if the module should not export any names,
specify an explicit, empty export list via
\code{\link[=export]{box::export()}}.
}
\note{
The API for hook functions is still subject to change. In particular,
there might in the future be a way to subscribe to module events of other
modules and packages, equivalently to R package \link[base]{userhooks}.
}
\concept{experimental}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unload.r
\name{unload}
\alias{unload}
\alias{reload}
\alias{purge_cache}
\title{Unload or reload modules}
\usage{
\special{box::unload(mod)}

\special{box::reload(mod)}

\special{box::purge_cache()}
}
\arguments{
\item{mod}{a module object to be unloaded or reloaded}
}
\value{
These functions are called for their side effect. They do not return
anything.
}
\description{
Given a module which has been previously loaded and is assigned to an alias
\code{mod}, \code{box::unload(mod)} unloads it; \code{box::reload(mod)}
unloads and reloads it from its source. \code{box::purge_cache()} marks all
modules as unloaded.
}
\details{
Unloading a module causes it to be removed from the internal cache such that
the next subsequent \code{box::use} declaration will reload the module from
its source. \code{box::reload} unloads and reloads the specified modules and
all its transitive module dependencies. \code{box::reload} is \emph{not}
merely a shortcut for calling \code{box::unload} followed by \code{box::use},
because \code{box::unload} only unloads the specified module itself, not any
dependent modules.
}
\note{
Any other references to the loaded modules remain unchanged, and will
(usually) still work. Unloading and reloading modules is primarily useful for
testing during development, and \emph{should not be used in production code:}
in particular, unloading may break other module references if the
\code{.on_unload} hook unloaded any binary shared libraries which are still
referenced.

These functions come with a few restrictions.
\code{box::unload} attempts to detach names attached by the corresponding
\code{box::use} call.
\code{box::reload} attempts to re-attach these same names. This only works if
the corresponding \code{box::use} declaration is located in the same scope.
\code{box::purge_cache} only removes the internal cache of modules, it does
not actually invalidate any module references or names attached from loaded
modules.

\code{box::unload} will execute the \code{.on_unload} hook of the module, if
it exists.
\code{box::reload} will re-execute the \code{.on_load} hook of the module and
of all dependent modules during loading (after executing the corresponding
\code{.on_unload} hooks during unloading).
\code{box::purge_cache} will execute any existing \code{.on_unload} hooks in
all loaded modules.
}
\seealso{
\code{\link[=use]{box::use}}, \link[=mod-hooks]{module hooks}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/info.r
\name{find_in_path}
\alias{find_in_path}
\title{Find a module’s source location}
\usage{
find_in_path(spec, base_paths)
}
\arguments{
\item{spec}{a \code{mod_spec}.}

\item{base_paths}{a character vector of paths to search the module in, in
order of preference.}
}
\value{
\code{find_in_path} returns a \code{mod_info} that specifies the
module source location.
}
\description{
Find a module’s source location
}
\details{
A module is physically represented in the file system either by
\file{‹spec_name(spec)›.r} or by \file{‹spec_name(spec)›/__init__.r}, in that
order of preference in case both exist. File extensions are case insensitive
to allow for R’s obsession with capital-R extensions (but lower-case are
given preference, and upper-case file extensions are discouraged).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format.r
\name{fmt}
\alias{fmt}
\alias{chr}
\alias{html_escape}
\alias{interleave}
\title{String formatting helpers}
\usage{
fmt(..., envir = parent.frame())

chr(x)

html_escape(x)

interleave(a, b)
}
\arguments{
\item{\dots}{one or more unnamed character string arguments, followed
optionally by named arguments}

\item{x}{an object to convert}

\item{a}{a character vector of length \code{n}}

\item{b}{a character vector of length \code{n - 1}}
}
\value{
\code{fmt(\dots)} concatenates any unnamed arguments, and
interpolates all embedded expressions as explained in the \sQuote{Details}.
Named arguments are treated as locally defined variables, and are added to
(and override, in case of name reuse) names defined in the calling scope.

\code{chr(x)} returns a string representation of a value or
unevaluated expression \code{x}.

\code{html_escape(x)} returns the HTML-escaped version of \code{x}.

\code{interleave(a, b)} returns a vector \code{c(a[1], b[1], a[2],
b[2], \dots, a[n - 1], b[n - 1], a[n])}.
}
\description{
Interpolate expressions in a string
}
\details{
\code{fmt} interpolates embedded expressions in a string.
\code{chr} converts a value to a character vector; unlike
\code{as.character}, it correctly deparses unevaluated names and expressions.
\code{interleave} is a helper that interleaves two vectors \code{a = c(a[1],
\dots, a[n])} and \code{b = c(b[1], \dots, b[n - 1])}.

The general format of an interpolation expression inside a \code{fmt} string
is: \code{\{\dots\}} interpolates the expression \code{\dots}. To insert
literal braces, double them (i.e. \code{\{\{}, \code{\}\}}). Interpolated
expressions can optionally be followed by a \emph{format modifier}: if
present, it is specified via the syntax \code{\{\dots;modifier\}}. The
following modifiers are supported:
\describe{
 \item{\code{\"}}{like \code{dQuote(\dots)}}
 \item{\code{\'}}{like \code{sQuote(\dots)}}
 \item{\code{‹fmt›f}}{like \code{sprintf('\%‹fmt›f', \dots)}}
}
Vectors of length > 1 will be concatenated as if using
\code{\link[base]{toString}} before interpolation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/default.r
\name{default}
\alias{default}
\alias{\%||\%}
\alias{\%|\%}
\title{Retrieve a value or a default}
\usage{
a \%||\% b

lhs \%|\% rhs
}
\arguments{
\item{a}{the value to return if non-empty}

\item{b}{default value}

\item{lhs}{vector with potentially missing values, or \code{NULL}}

\item{rhs}{vector with default values, same length as \code{lhs} unless that
is \code{NULL}}
}
\value{
\code{a \%||\% b} returns \code{a}, unless it is \code{NULL}, empty,
\code{FALSE} or \code{""}; in which case \code{b} is returned.

\code{lhs \%|\% rhs} returns a vector of the same length as
\code{rhs} with all missing values in \code{lhs} replaced by the
corresponding values in \code{rhs}.
}
\description{
\code{a \%||\% b} returns \code{a} unless it is empty, in which case
\code{b} is returned.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/env.r
\name{topenv}
\alias{topenv}
\title{Get a module’s namespace environment}
\usage{
\special{box::topenv()}

\special{box::topenv(env)}
}
\arguments{
\item{module}{a module environment}
}
\value{
\code{box::topenv()} returns the top-level module environment of the
module it is called from, or the nearest top-level non-module environment
otherwise; this is usually \code{.GlobalEnv}.

\code{box::topenv(env)} returns the nearest top-level environment that is a
direct or indirect parent of \code{env}.
}
\description{
Called inside a module, \code{box::topenv()} returns the module namespace
environment. Otherwise, it behaves similarly to \code{\link[base]{topenv}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.r
\name{help}
\alias{help}
\title{Display module documentation}
\usage{
\special{box::help(topic, help_type = getOption("help_type", "text"))}
}
\arguments{
\item{topic}{either the fully-qualified name of the object or function to get
help for, in the format \code{module$function}; or a name that was exported
and attached from an imported module or package.}

\item{help_type}{character string specifying the output format; currently,
only \code{'text'} is supported.}
}
\value{
\code{box::help} is called for its side effect when called directly
from the command prompt.
}
\description{
\code{box::help} displays help on a module’s objects and functions in much
the same way \code{\link[utils]{help}} does for package contents.
}
\details{
See the vignette at \code{vignette('box', 'box')} for more information about
displaying help for modules.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.r
\name{paths}
\alias{paths}
\alias{mod_search_path}
\alias{calling_mod_path}
\alias{split_path}
\alias{merge_path}
\alias{sanitize_path_fragment}
\title{Path related functions}
\usage{
mod_search_path(caller)

calling_mod_path(caller)

split_path(path)

merge_path(components)

sanitize_path_fragment(path)
}
\arguments{
\item{caller}{the environment from which \code{box::use} was invoked.}

\item{path}{the path}

\item{components}{character string vector of path components to merge}
}
\value{
\code{calling_mod_path} the path of the source module that is calling
\code{box::use}, or the script’s path if the calling code is not a module.

\code{split_path} returns a character vector of path components that
logically represent \code{path}.

\code{merge_path} returns a single character string that is
logically equivalent to the \code{path} passed to \code{split_path}.
logically represent \code{path}.
}
\description{
\code{mod_search_path} returns the character vector of paths where module
code can be located and will be found by \pkg{box}.
}
\note{
The search paths are ordered from highest to lowest priority.
The current module’s path always has the lowest priority.

There are two ways of modifying the module search path: by default,
\code{getOption('box.path')} specifies the search path as a character vector.
Users can override its value by separately setting the environment variable
\env{R_BOX_PATH} to one or more paths, separated by the platform’s path
separator (\dQuote{:} on UNIX-like systems, \dQuote{;} on Windows).

\code{merge_path} is the inverse function to \code{split_path}.
However, this does not mean that its result will be identical to the
original path. Instead, it is only guaranteed that it will refer to the same
logical path given the same working directory.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.r
\name{parse_export_tags}
\alias{parse_export_tags}
\title{Extract comment tags from Roxygen block comments}
\usage{
parse_export_tags(info, exprs, mod_ns)
}
\arguments{
\item{exprs}{The unevaluated expressions to parse.}
}
\value{
\code{parse_export_tags} returns a list of \code{roxy_block}s for all
exported declarations.
}
\description{
Extract comment tags from Roxygen block comments
}
\note{
The following code performs the same function as roxygen2 with a custom
\code{@} tag roclet. Unfortunately roxygen2 itself pulls in many
dependencies, making it less suitable for an infrastructure package such as
this one. Furthermore, the code license of roxygen2 is incompatible with
ours, so we cannot simply copy and paste the relevant code out. Luckily the
logic is straightforward to reimplement.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/env.r
\name{wrap_unsafe_function}
\alias{wrap_unsafe_function}
\title{Wrap \dQuote{unsafe calls} functions}
\usage{
wrap_unsafe_function(ns, name)
}
\arguments{
\item{ns}{The namespace of the unsafe function.}

\item{name}{The name of the unsafe function.}
}
\value{
\code{wrap_unsafe_calls} returns a wrapper function with the same
argument as the wrapped function that can be called without causing a NOTE.
}
\description{
\code{wrap_unsafe_function} declares a function wrapper to a function that
causes an \command{R CMD check} NOTE when called directly. We should usually
not call these functions, but we need some of them because we want to
explicitly support features they provide.
}
\note{
Using an implementation that simply aliases \code{getExportedValue}
does not work, since \command{R CMD check} sees right through this
\dQuote{ruse}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loaded.r
\docType{data}
\name{loaded}
\alias{loaded}
\alias{loaded_mods}
\alias{is_mod_loaded}
\alias{register_mod}
\alias{deregister_mod}
\alias{loaded_mod}
\alias{is_mod_still_loading}
\alias{mod_loading_finished}
\title{Environment of loaded modules}
\format{
\code{loaded_mods} is an environment of the loaded module and package
namespaces.
}
\usage{
loaded_mods

is_mod_loaded(info)

register_mod(info, mod_ns)

deregister_mod(info)

loaded_mod(info)

is_mod_still_loading(info)

mod_loading_finished(info, mod_ns)
}
\arguments{
\item{info}{the mod info of a module}

\item{mod_ns}{module namespace environment}
}
\description{
Each module is stored as an environment inside \code{loaded_mods} with the
module’s code location path as its identifier. The path rather than the
module name is used because module names are not unique: two modules called
\code{a} can exist nested inside modules \code{b} and \code{c}, respectively.
Yet these may be loaded at the same time and need to be distinguished.
}
\details{
\code{is_mod_loaded} tests whether a module is already loaded.

\code{register_mod} caches a module namespace and marks the module as loaded.

\code{deregister_mod} removes a module namespace from the cache, unloading
the module from memory.

\code{loaded_mod} retrieves a loaded module namespace given its info.

\code{is_mod_still_loading} tests whether a module is still being loaded.

\code{mod_loading_finished} signals that a module has been completely loaded.
}
\note{
\code{is_mod_still_loading} and \code{mod_loading_finished} are used to
break cycles during the loading of modules with cyclic dependencies.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.r
\name{path}
\alias{path}
\alias{base_path}
\alias{module_path}
\alias{mod_path}
\alias{explicit_path}
\alias{r_path}
\alias{knitr_path}
\alias{shiny_path}
\alias{testthat_path}
\alias{rstudio_path}
\alias{wd_path}
\title{Get a module’s path}
\usage{
path(mod)

base_path(mod)

module_path(mod)

mod_path(mod)

explicit_path(...)

r_path(...)

knitr_path(...)

shiny_path(...)

testthat_path(...)

rstudio_path(...)

wd_path(...)
}
\arguments{
\item{mod}{a module environment or namespace}
}
\value{
\code{path} returns a character string containing the module’s full
path.

\code{base_path} returns a character string containing the module’s
base directory, or the current working directory if not invoked on a module.

\code{module_path} returns a character string that contains the
directory in which the calling R code is run. See \sQuote{Details}.

\code{mod_path} returns the script path associated with a \pkg{box}
module

\code{explicit_path} returns the script path explicitly set by the
user, if such a path was set.

\code{r_path} returns the directory in which the current script is
run via \command{Rscript}, \command{R CMD BATCH} or \command{R -f}.

\code{knitr_path} returns the directory in which the currently knit
document is run, or \code{NULL} if not called from within a \pkg{knitr}
document.

\code{shiny_path} returns the directory in which a \pkg{Shiny}
application is running, or \code{NULL} if not called from within a
\pkg{Shiny} application.

\code{testthat_path} returns the directory in which \pkg{testthat}
code is being executed, or \code{NULL} if not called from within a
\pkg{testthat} test case.

\code{rstdio_path} returns the directory in which the currently
active RStudio script file is saved.

\code{wd_path} returns the current working directory.
}
\description{
The following functions retrieve information about the path of the directory
that a module or script is running in.
}
\details{
\code{module_path} takes a best guess at a script’s path, since R does not
provide a sure-fire way for determining the path of the currently executing
code. The following calling situations are covered:

\enumerate{
 \item Path explicitly set via \code{set_script_path}
 \item Path of a running document/application (\pkg{knitr}, \pkg{Shiny})
 \item Path of unit test cases (\pkg{testthat})
 \item Path of the currently opened source code file in RStudio
 \item Code invoked as \command{Rscript script.r}
 \item Code invoked as \command{R CMD BATCH script.r}
 \item Code invoked as \command{R -f script.r}
 \item Script run interactively (use \code{getwd()})
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file.r
\name{file}
\alias{file}
\title{Find the full paths of files in modules}
\usage{
\special{box::file(\dots)}

\special{box::file(\dots, module)}
}
\arguments{
\item{\dots}{character vectors of files or subdirectories inside a module; if
none is given, return the root directory of the module}

\item{module}{a module environment}
}
\value{
A character vector containing the absolute paths to the files
 specified in \code{\dots}.
}
\description{
Find the full paths of files in modules
}
\note{
If called from outside a module, the current working directory is used.

This function is similar to \code{system.file} for packages. Its semantics
differ in the presence of non-existent files: \code{box::file} always returns
the requested paths, even for non-existent files; whereas \code{system.file}
returns empty strings for non-existent files, or fails (if requested via the
argument \code{mustWork = TRUE}).
}
\seealso{
\code{\link[base]{system.file}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/errors.r
\name{throw}
\alias{throw}
\alias{rethrow}
\alias{rethrow_on_error}
\alias{box_error}
\title{Throw informative error messages}
\usage{
throw(..., call = sys.call(sys.parent()), subclass = NULL)

rethrow(error, call = sys.call(sys.parent()))

rethrow_on_error(expr, call = sys.call(sys.parent()))

box_error(message, call = NULL, subclass = NULL)
}
\arguments{
\item{\dots}{arguments to be passed to \code{fmt}}

\item{call}{the calling context from which the error is raised}

\item{subclass}{an optional subclass name for the error condition to be
raised}

\item{error}{an object of class \code{c("error", "condition")} to rethrow}

\item{expr}{an expression to evaluate inside \code{tryCatch}}

\item{message}{the error message}
}
\value{
If it does not throw an error, \code{rethrow_on_error} returns the
value of evaluating \code{expr}.

\code{box_error} returns a new \sQuote{box} error condition object
with a given message and call, and optionally a given subclass type.
}
\description{
Helpers to generate readable and informative error messages for package
users.
}
\details{
For \code{rethrow}, the \code{call} argument overrides the rethrown error’s
own stored call.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3.r
\name{register_S3_method}
\alias{register_S3_method}
\title{Register S3 methods}
\usage{
\special{box::register_S3_method(name, class, method)}
}
\arguments{
\item{name}{the name of the generic as a character string.}

\item{class}{the class name as a character string.}

\item{method}{the function to register as a method (optional).}
}
\value{
\code{box::register_S3_method} is called for its side effect.
}
\description{
\code{box::register_S3_method} makes an S3 method for a given generic and
class known inside a module.
}
\details{
If \code{method} is missing, it defaults to a function named
\code{name.class} in the calling module. If no such function exists, an error
is raised.

Methods for generics defined in the same module do not need to be registered
explicitly, and indeed \emph{should not} be registered. However, if the user
wants to add a method for a known generic (defined outside the module, e.g.
\code{\link{print}}), then this needs to be made known explicitly.

See the vignette at \code{vignette('box', 'box')} for more information about
defining S3 methods inside modules.
}
\note{
\strong{Do not} call \code{\link[base]{registerS3method}} inside a
module, only use \code{box::register_S3_method}. This is important for the
module’s own book-keeping.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.r
\name{create_export_block}
\alias{create_export_block}
\alias{parse_object}
\alias{roxygen2_object}
\title{Collect export tag information}
\usage{
create_export_block(expr, ref, info, mod_ns)

parse_object(info, expr, mod_ns)

roxygen2_object(alias, value, type)
}
\arguments{
\item{expr}{The unevaluated expression represented by the tag.}

\item{ref}{The code reference \code{srcref} represented by the tag.}

\item{alias}{The object name.}

\item{value}{The object value.}

\item{type}{The object type.}
}
\value{
\code{create_export_block} returns an object of type
\code{roxy_block} represents an exported declaration expression, along with
its source code location.
}
\description{
Collect export tag information
}
\note{
This could be represented much simpler but we keep compatibility with
roxygen2 — at least for the time being — to make integration with the
roxygen2 API easier, should it become necessary.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/map.r
\name{map}
\alias{map}
\alias{flatmap}
\alias{flatmap_chr}
\alias{vmap}
\alias{map_int}
\alias{map_lgl}
\alias{map_chr}
\alias{transpose}
\title{Apply function to elements in list}
\usage{
map(.f, ...)

flatmap(.f, ..., .default)

flatmap_chr(.f, ...)

vmap(.f, .x, ..., .default)

map_int(.f, ...)

map_lgl(.f, ...)

map_chr(.f, ...)

transpose(...)
}
\arguments{
\item{.f}{an n-ary function where n is the number of further arguments given}

\item{\dots}{lists of arguments to map over in parallel}

\item{.default}{the default value returned by \code{flatmap} for an empty
input}
}
\value{
\code{map} returns a (potentially nested) list of values resulting
from applying \code{.f} to the arguments.

\code{flatmap} returns a vector with type given by \code{.default},
or \code{.default}, if the input is empty.

\code{transpose} returns a list of the element-wise concatenated
input vectors; that is, a \dQuote{transposed list} of those elements.
}
\description{
\code{map} applies a function to lists of arguments, similar to \code{Map} in
base R, with the argument \code{USE.NAMES} set to \code{FALSE}.
\code{flatmap} performs a recursive map: the return type is always a vector
of some type given by the \code{.default}, and if the return value of calling
\code{.f} is a vector, it is flattened into the enclosing vector (see
\sQuote{Examples}).
\code{transpose} is a special \code{map} application that concatenates its
inputs to compute a transposed list.
}
\section{Examples}{

\preformatted{
flatmap_chr(identity, NULL)
# character(0)

flatmap_chr(identity, c('a', 'b'))
# [1] "a" "b"

flatmap_chr(identity, list(c('a', 'b'), 'c'))
# [1] "a" "b" "c"

transpose(1 : 2, 3 : 4)
# [[1]]
# [1] 1 3
#
# [[2]]
# [1] 2 4
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.r
\name{script_path}
\alias{script_path}
\alias{set_script_path}
\title{Set the base path of the script}
\usage{
\special{box::set_script_path(path)}

\special{box::script_path()}
}
\arguments{
\item{path}{character string containing the relative or absolute path to the
currently executing \R code file, or \code{NULL} to reset the path.}
}
\value{
Both \code{box::script_path} and \code{box::set_script_path} return
the previously set script path, or \code{NULL} if none was explicitly set.
\code{box::set_script_path} returns its value invisibly.
}
\description{
\code{box::set_script_path(path)} explicitly tells \pkg{box} the path of a
given script from which it is called; \code{box::script_path()} returns the
previously set path.
}
\details{
\pkg{box} needs to know the base path of the topmost calling \R context (i.e.
the script) to find relative import locations. In most cases, \pkg{box} can
figure the path out automatically. However, in some cases third-party
packages load code in a way in which \pkg{box} cannot find the correct path
of the script any more. \code{box::set_script_path} can be used in these
cases to set the path of the currently executing \R script manually.
}
\note{
\pkg{box} \emph{should} be able to figure out the script path automatically.
Using \code{box::set_script_path} should therefore never be necessary.
\href{https://github.com/klmr/box/issues/new?assignees=&labels=\%E2\%9A\%A0\%EF\%B8\%8F+bug&title=\%5Bset_script_path\%5D\%20&template=bug-report.yml}{Please
file an issue} if you encounter a situation that necessitates using
\code{box::set_script_path}!
}
\examples{
box::set_script_path('scripts/my_script.r')
}
